C------------------------------------------------------------------------
C     
C     Copyright (C) 2001-2003 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE ADVECTION(U, V, W, CLESP, Zclx, Zcly, Zclz, dt, C)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This routine solves advection along X, Y and Z for one timestep.
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     U: 3D wind field along X ([m/s]).
C     V: 3D wind field along Y ([m/s]).
C     W: 3D wind field along Z ([m/s]).
C     CLESP: TRUE (FALSE) if boundary conditions are given (not given).
C     ZCLX: 2D field for boundary conditions along X.
C     ZCLY: 2D field for boundary conditions along Y.
C     ZCLZ: 2D field for boundary conditions along Z.
C     DT: timestep ([s]).
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     C: 3D field of concentrations for a given chemical species.
C     
C     -- OUTPUT VARIABLES
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Vivien Mallet, CEREA, May 2003.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

C     -- INCLUDE FILES
C     PARADOM: parameters for all forced fields.
C     COMDOM: global variables for mesh and forcing fields. 

      INCLUDE 'PARADOM.INC'
      INCLUDE 'COMDOM.INC'
      INCLUDE 'COMUSE.INC'

      DOUBLE PRECISION C(NX,NY,NZ)
      DOUBLE PRECISION DLC(NX+4,NY+4,NZ+4)

      DOUBLE PRECISION dt

      DOUBLE PRECISION U(NX+1,NY,NZ), V(NX,NY+1,NZ), W(NX,NY,NZ+1)

      DOUBLE PRECISION Zclx(2,Nyl,Nzl), Zcly(Nxl,2,Nzl), Zclz(Nxl,Nyl)
      LOGICAL CLESP

      DOUBLE PRECISION Fx_0, Fx_1, Fy_0, Fy_1, Fz_0, Fz_1

      INTEGER i,j,k
      INTEGER x, y, z

C------------------------------------------------------------------------
C     1 - Adds ghost cells

      DO k = 1, Nz
         DO j = 1, Ny
            DO i = 1, Nx
               DLC(i+2, j+2, k+2) = C(i, j, k)
            ENDDO
         ENDDO
      ENDDO

C     1.1 - With boundary conditions

      IF (CLESP) THEN
         
         DO k = 1, Nz
            DO j = 1, Ny
               DLC(2, j+2, k+2) = Zclx(1, j, k)
               DLC(Nx+3, j+2, k+2) = Zclx(2, j, k)
            ENDDO
         ENDDO
         
         DO k = 1, Nz
            DO i = 1, Nx
               DLC(i+2, 2, k+2) = Zcly(i, 1, k)
               DLC(i+2, Ny+3, k+2) = Zcly(i, 2, k)
            ENDDO
         ENDDO
         
         DO j = 1, Ny
            DO i = 1, Nx
               DLC(i+2, j+2, 2) = C(i, j, 1)
               DLC(i+2, j+2, Nz+3) = Zclz(i, j)
            ENDDO
         ENDDO

C     1.2 - Without boundary conditions
         
      ELSE

         DO k = 1, Nz
            DO j = 1, Ny
               DLC(2, j+2, k+2) = C(1, j, k)
               DLC(Nx+3, j+2, k+2) = C(Nx, j, k)
            ENDDO
         ENDDO
         
         DO k = 1, Nz
            DO i = 1, Nx
               DLC(i+2, 2, k+2) = C(i, 1, k)
               DLC(i+2, Ny+3, k+2) = C(i, Ny, k)
            ENDDO
         ENDDO
         
         DO j = 1, Ny
            DO i = 1, Nx
               DLC(i+2, j+2, 2) = C(i, j, 1)
               DLC(i+2, j+2, Nz+3) = C(i, j, Nz)
            ENDDO
         ENDDO

      ENDIF
      
C     1.3 - Outermost ghost cells
      
      DO k = 1, Nz
         DO j = 1, Ny
            DLC(1, j+2, k+2) = DLC(2, j+2, k+2)
            DLC(Nx+4, j+2, k+2) = DLC(Nx+3, j+2, k+2)
         ENDDO
      ENDDO
      
      DO k = 1, Nz
         DO i = 1, Nx
            DLC(i+2, 1, k+2) = DLC(i+2, 2, k+2)
            DLC(i+2, Ny+4, k+2) = DLC(i+2, Ny+3, k+2)
         ENDDO
      ENDDO

      DO j = 1, Ny
         DO i = 1, Nx
            DLC(i+2, j+2, 1) = DLC(i+2, j+2, 2)
            DLC(i+2, j+2, Nz+4) = DLC(i+2, j+2, Nz+3)
         ENDDO
      ENDDO

C------------------------------------------------------------------------
C     2 - Fluxes computations

      DO k = 1, Nz
         DO j = 1, Ny
            DO i = 1, Nx

               x = i + 2
               y = j + 2
               z = k + 2

C     Flux limiter cancelled for outgoing flux.

               IF((i.EQ.1).AND.(U(i, j, k).LT.0.d0))THEN 
                  DLC(x-1, y, z)=DLC(x, y, z)
               ENDIF

               IF((i.EQ.Nx).AND.(U(i+1, j, k).GT.0.d0))THEN 
                  DLC(x+1, y, z)=DLC(x, y, z)
               ENDIF

               IF((j.EQ.1).AND.(V(i, j, k).LT.0.d0))THEN 
                  DLC(x, y-1, z)=DLC(x, y, z)
               ENDIF

               IF((j.EQ.Ny).AND.(V(i, j+1, k).GT.0.d0))THEN 
                  DLC(x, y+1, z)=DLC(x, y, z)
               ENDIF

               IF((k.EQ.Nz).AND.(W(i, j, k+1).GT.0.d0))THEN 
                  DLC(x, y, z+1)=DLC(x, y, z)
               ENDIF

C     Along X.

               CALL numerical_flux( U(i, j, k),
     $              DLC(x-2, y, z), DLC(x-1, y, z),
     $              DLC(x, y, z), DLC(x+1, y, z),
     $              dt / Dmx(i), Fx_0 )

               CALL numerical_flux( U(i+1, j, k),
     $              DLC(x-1, y, z), DLC(x, y, z),
     $              DLC(x+1, y, z), DLC(x+2, y, z),
     $              dt / Dmx(i), Fx_1 )

C     Along Y.

               CALL numerical_flux( V(i, j, k),
     $              DLC(x, y-2, z), DLC(x, y-1, z),
     $              DLC(x, y, z), DLC(x, y+1, z),
     $              dt / Dmy(j), Fy_0 )
               
               CALL numerical_flux( V(i, j+1, k),
     $              DLC(x, y-1, z), DLC(x, y, z),
     $              DLC(x, y+1, z), DLC(x, y+2, z),
     $              dt / Dmy(j), Fy_1 )
               
C     Along Z.

               CALL numerical_flux( W(i, j, k),
     $              DLC(x, y, z-2), DLC(x, y, z-1),
     $              DLC(x, y, z), DLC(x, y, z+1),
     $              dt / Dmz(i, j, k), Fz_0 )
               
               CALL numerical_flux( W(i, j, k+1),
     $              DLC(x, y, z-1), DLC(x, y, z),
     $              DLC(x, y, z+1), DLC(x, y, z+2),
     $              dt / Dmz(i, j, k), Fz_1 )

C     Updates.

               C(i, j, k) = C(i, j, k)
     $              - Fx_1 + Fx_0
     $              - Fy_1 + Fy_0
     $              - Fz_1 + Fz_0

               IF (C(i, j, k) .LT. 0.D0) THEN
                  IF (LCOMMENT) THEN
                     WRITE(*,*) 'CLIP advection', i, j, k, C(i, j, k)
                  ENDIF
                  C(i, j, k) = 0.D0
               ENDIF

            ENDDO
         ENDDO
      ENDDO

      END
