C------------------------------------------------------------------------
C     
C     Copyright (C) 2001-2003 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE numerical_flux(wind, c_0, c_1, c_2, c_3, dt_dx, flux)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This routine computes the numerical flux between cells #1 and #2.
C     It uses a third-order upwind-biased scheme with a Sweby-type flux
C     limiter. Refer to "An efficient horizontal advection scheme for
C     the modelling of global transport of constituents", W. Hundsdorfer
C     and E.J. Spee.
C     
C     `````````````````flux``````````````````
C     `````````````````---->`````````````````
C     ----o----|----o----|----o----|----o----
C     ```c_0```````c_1```````c_2```````c_3```
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     WIND: wind at the interface between cells #1 and #2.
C     C_0, C_1, C_2, C_3: concentrations in cells #0, #1, #2 and #3.
C     DT_DX: ratio of the timestep to the cell size.
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     -- OUTPUT VARIABLES
C     
C     FLUX: the numerical flux between cell #1 and cell #2.
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Vivien Mallet, CEREA, May 2003.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

C     -- INCLUDE FILES
C     PARADOM: parameters for all forced fields.

      INCLUDE 'PARADOM.INC'

      DOUBLE PRECISION wind
      DOUBLE PRECISION c_0, c_1, c_2, c_3
      DOUBLE PRECISION dt_dx
      DOUBLE PRECISION flux

      DOUBLE PRECISION nu, temp, nu_1, diff_c, nu_diff_c
      DOUBLE PRECISION phi


      nu = dabs(wind) * dt_dx

      temp = (c_2 - c_1) * nu

C     Positive wind and without flux limiter.
      IF ((wind.ge.0.d0).and.(temp * (c_1 - c_0) .le. 0.d0)) THEN
         
         flux = nu * c_1

C     Negative wind and without flux limiter.
      ELSE IF ((wind.le.0.d0).and.(temp * (c_3 - c_2) .le. 0.d0)) THEN
         
         flux = - nu * c_2
         
C     With flux limiter.
      ELSE
         
         IF (wind.ge.0.d0) THEN
            diff_c = c_1 - c_0
            flux = nu * c_1
         ELSE
            diff_c = c_3 - c_2
            flux = - nu * c_2
         ENDIF

         nu_1 = 1.d0 - nu
         nu_diff_c = nu_1 * diff_c
         
         IF (temp.ge.0.d0) THEN

            phi = dmin1(temp,
     $           ( temp * (2.d0 - nu) * nu_1
     $           + nu * (1.d0 + nu) * nu_diff_c ) / 6.d0 , nu_diff_c)

            flux = flux + dmax1(0.d0, phi)

         ELSE

            phi = dmax1(temp,
     $           ( temp * (2.d0 - nu) * nu_1
     $           + nu * (1.d0 + nu) * nu_diff_c ) / 6.d0 , nu_diff_c )

            flux = flux + dmin1(0.d0, phi)

         ENDIF

      ENDIF

      END
