C------------------------------------------------------------------------
C     
C     Copyright (C) 2001-2003 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE chem (ts,DLrain,DLhcloud,DLattenuation,DLhumid,DLtemp,
     s     DLpress,DLCsourc,DLCforc2d,DLCphotolysis_rates,
     s     tf,DLrainf,DLhcloudf,DLattenuationf,DLhumidf,DLtempf,
     s     DLpressf,DLCsourcf,DLCforc2df,DLCphotolysis_ratesf,
     s     DLconc)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This routine computes one timestep for gas-phase chemistry EMEP.
C     Chemical kinetics is solved in each grid cell. There are two parts
C     in EMEP mechanism: a linear part (gas-phase chemistry) and a
C     nonlinear part (heterogeneous equilibrium).
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     TS: initial time (GMT, computed from January 1st, [s]).
C     DLRAIN: 2D rain intensity field at initial time ([mm/h]).
C     DLHCLOUD: 2D height of cloud basis ([m])
C     DLATTENUATION: 3D cloud attenuation field at initial time.
C     DLHUMID: 3D specific humidity field at initial time ([-]).
C     DLTEMP: 3D temperature field at initial time ([K]).
C     DLPRESS: 3D pressure field at initial time ([Pa]).
C     DLCFORC: array of 2D forced chemical concentration fields at
C     # initial time ([\mu.g/m^3]).
C     # The third index labels the chemical species in the order:
C     # O3-OH-ROH.
C     DLCSOURC: array of chemical volumic emissions at initial time
C     # ([\mu.g/m^3/s]).
C     DLCPHOTOLYSIS_RATES: photochemical kinetic rates
C     # at initial time ([s^{-1}]).
C     
C     The same variables are defined at final time of the timestep.
C     'f' is then put at the end of the name.
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     DLCONC: array of 3D chemical concentrations ([\mu.g/m^3]). 
C     # Before entry, it is given at initial time of the timestep.
C     # On exit, it is computed at final time of the timestep.
C     
C     -- OUTPUT VARIABLES
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     2001/05/11: EMEP chemistry (Jaouad Boutahar).
C     2002/01/14: implementation 0D version (Jaouad Boutahar).
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Jaouad Boutahar, CEREA, June 2001.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

C     -- INCLUDE FILES
C     CONST: constants.
C     PARADOM: parameters for all forced fields.
C     PARACHEM: parameters for sizes of 'chemical' arrays.
C     COMUSE: flags and parameters for numerics. 
C     COMDOM: global variables for mesh and forcing fields. 
C     COMCHEM: global variables for chemical species. 

      INCLUDE 'CONST.INC'
      INCLUDE 'PARADOM.INC'
      INCLUDE 'PARACHEM.INC'
      INCLUDE 'COMDOM.INC'
      INCLUDE 'COMUSE.INC'
      INCLUDE 'COMCHEM.INC'
      
      DOUBLE PRECISION DLtemp(NXt,NYt,NZt),DLtempf(NXt,NYt,NZt)
      DOUBLE PRECISION DLrain(NXr,NYr),DLrainf(NXr,NYr)
      DOUBLE PRECISION DLhcloud(NXr,NYr),DLhcloudf(NXr,NYr)
      DOUBLE PRECISION DLattenuation(NXatt,NYatt,NZatt)
      DOUBLE PRECISION DLattenuationf(NXatt,NYatt,NZatt)
      DOUBLE PRECISION DLhumid(NXh,NYh,NZh),DLhumidf(NXh,NYh,NZh)
      DOUBLE PRECISION DLCsourc(Nx,Ny,Nzs,Nsrc)
      DOUBLE PRECISION DLCsourcf(Nx,Ny,Nzs,Nsrc)
      DOUBLE PRECISION DLpress(NXp,NYp,NZp),DLpressf(NXp,NYp,NZp)
      DOUBLE PRECISION DLCforc2d(Nxf,Nyf,NESPF)
      DOUBLE PRECISION DLCforc2df(Nxf,Nyf,NESPF)
      DOUBLE PRECISION DLCphotolysis_rates(NRphot,Nx,Ny,Nz)
      DOUBLE PRECISION DLCphotolysis_ratesf(NRphot,Nx,Ny,Nz)

      DOUBLE PRECISION ts,tf
      DOUBLE PRECISION O3,OH,ROH
      DOUBLE PRECISION O3f,OHf,ROHf
      DOUBLE PRECISION DLRki(Nr),DLRkf(Nr)

      DOUBLE PRECISION DLconc(NX,NY,NZ,nesp),ZC(nespl)
      DOUBLE PRECISION ZCn(nesph)

      DOUBLE PRECISION ZCsourc(NESP),ZCsourcf(NESP)
      DOUBLE PRECISION Zrain,Zrainf
      DOUBLE PRECISION Zangzen,Zangzenf

      DOUBLE PRECISION keq,DLkeq
      DOUBLE PRECISION DLmu,DLmuf
      DOUBLE PRECISION muzero
      EXTERNAL keq,muzero

      INTEGER Ji,Jj,Jk,Jsp,Ncase
      
C     Initialization for volumic sources.

      DO Jsp=1,Nesp
         ZCsourc(Jsp)=0.D0
         ZCsourcf(Jsp)=0.D0
      ENDDO

C     Conversion \mu.g/m3 to molecules/cm3.

      Ncase=1
      CALL CONVERS(Ncase,DLconc,DLCsourc,DLCsourcf,
     s     DLCforc2d,DLCforc2df)

C     Spatial extraction (loop on species).

      DO Jk=1,NZ
         DO Jj=1,NY
            DO Ji=1,NX
               
C     Forced concentrations.

               O3=DLCforc2d(Ji,Jj,1)
               O3f=DLCforc2df(Ji,Jj,1)
               
               OH=DLCforc2d(Ji,Jj,2)
               OHf=DLCforc2df(Ji,Jj,2)
               
               ROH=DLCforc2d(Ji,Jj,3)
               ROHf=DLCforc2df(Ji,Jj,3)
               
               Zrain=DLrain(Ji,Jj)
               Zrainf=DLrainf(Ji,Jj)

C     Diagnostic for rain (if rain: CC=1).

               IF(Zrain.GT.0.D0)then
                  Zrain=1.D0
               ELSE
                  Zrain=0.D0
               ENDIF

               IF(Zrainf.GT.0.D0)then
                  Zrainf=1.D0
               ELSE
                  Zrainf=0.D0
               ENDIF

C     Computation of zenithal angle in degree as a function of
C     GMT hour (TS), latitude and longitude.
C     The cosine of the zenithal angle is MUZERO.

               DLmu=muzero(ts,Dlon(Ji,Jj,Jk),Dlat(Ji,Jj,Jk))
               Zangzen=dabs(DACOS(DLmu)*180.D0/PI)
               DLmuf=muzero(tf,Dlon(Ji,Jj,Jk),Dlat(Ji,Jj,Jk))
               Zangzenf=dabs(DACOS(DLmuf)*180.D0/PI)

C     Reaction rates at initial and final time.

               CALL Kinetic(ts,dcos(Zangzen*PI/180.D0),
     s              DLtemp(Ji,Jj,Jk),Zrain,DLRKi)
               CALL Kinetic(tf,dcos(Zangzenf*PI/180.D0),
     s              DLtempf(Ji,Jj,Jk),Zrainf,DLRKf)

C     Volumic source terms.

               IF (LSOURC)THEN
                  DO Jsp=1,Nsrc

                     IF(Jk.LE.Nzs)THEN
                        ZCsourc(Nesps(Jsp))=DLCsourc(Ji,Jj,Jk,Jsp)
                        ZCsourcf(Nesps(Jsp))=DLCsourcf(Ji,Jj,Jk,Jsp)
                     ELSE
                        ZCsourc(Nesps(Jsp))=0.D0
                        ZCsourcf(Nesps(Jsp))=0.D0
                     ENDIF

                  ENDDO
               ENDIF

C     Spatial extraction for "linear species"
C     (species not implied in thermodynamical equilibria).

               DO Jsp=1,NESPL   ! linear species
                  ZC(Jsp)=DLconc(Ji,Jj,Jk,Jsp)
               ENDDO

C     Source term to be added for heterogeneous species.

               DO Jsp=1,NESPH   ! heterogeneous species

                  IF(.not.LSOURC)THEN
                     ZCn(Jsp)=DLconc(Ji,Jj,Jk,Jsp+NESPL)
                  ELSE
                     ZCn(Jsp)=DLconc(Ji,Jj,Jk,Jsp+NESPL)+
     s                    ZCsourc(Jsp+NESPL)*Dtstep
                  ENDIF 

               ENDDO

C     Solve linear part.

               CALL roschem (ZC,ZCsourc,ZCsourcf,ts,tf,
     s              DLRki,DLRkf,O3,OH,ROH,O3f,OHf,ROHf)

C     Equilibrium constant for thermodynamical equilibrium

               DLkeq=keq(DLtemp(Ji,Jj,Jk),DLhumid(Ji,Jj,Jk))
               DLkeq=DLkeq*1.D+4/((8.314D0*DLtemp(Ji,Jj,Jk))**2)

C     Solve nonlinear part.

               CALL SOLNLIN (ZCn,ZC,DLkeq)

               DO Jsp=1,NESPL
                  DLconc(Ji,Jj,Jk,Jsp)=ZC(Jsp)
               ENDDO

C     NESP=NESPL+NESPH

               DO Jsp=NESPL+1,NESP
                  DLconc(Ji,Jj,Jk,Jsp)=ZCn(Jsp-Nespl)
               ENDDO

            ENDDO
         ENDDO
      ENDDO                     ! loop on cells.

C     Conversion of concentrations molecules/cm3  to \mu. g/m3

      Ncase=2
      CALL CONVERS(Ncase,DLconc,DLCsourc,DLCsourcf,
     s     DLCforc2d,DLCforc2df)
      
      END
