C------------------------------------------------------------------------
C     
C     Copyright (C) 2001-2003 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE FEXCHEM (DLconc,ZCsourc,RK,O3,OH,ROH,DLchem)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This routine computes the chemical production term for the
C     gas-phase mechanism EMEP (linear part).
C     
C     NO  + O3             --k1--> NO2 + O2
C     NO2 + hv             --k2--> NO  + O
C     NO2 + OH             --k3--> HNO3
C     2NO2 + O3 + H2O      --k4--> 2NO3- + 2H+
C     NO2 + CH3COO2 (ROH)  --k5--> CH3COO2NO2 (PAN)
C     PAN                  --k6--> NO2 + ROH
C     HNO3                 --k7--> NO3-
C     NO3-                 --k8--> HNO3
C     SO2 + (OH,H2O2,O3)   --k9--> SO4,2-
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     DLCONC: chemical concentrations.
C     RK: kinetic rates.
C     ZCSOURC: volumic emissions.
C     O3, OH, ROH: forced concentrations for O3, OH and ROH.
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     -- OUTPUT VARIABLES
C     
C     DLCHEM: array of chemical production terms.
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Jaouad Boutahar.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

C     -- INCLUDE FILES
C     PARACHEM: parameters for sizes of 'chemical' arrays.
C     COMUSE: flags and parameters for numerics. 
C     COMCHEM: global variables for chemistry. 
C     PARADOM: parameters for all forced fields.

      INCLUDE 'PARADOM.INC'
      INCLUDE 'PARACHEM.INC'
      INCLUDE 'COMCHEM.INC'
      INCLUDE 'COMUSE.INC'

      DOUBLE PRECISION DLconc(nespl), DLchem(nespl)
      DOUBLE PRECISION W(nr),Rk(Nr)
      DOUBLE PRECISION O3,OH,ROH
      DOUBLE PRECISION ZCsourc(NESP)

      INTEGER Ji

C     Linear species.

      NO=DLconc(1)
      NO2=DLconc(2)
      HNO3=DLconc(3)
      PAN=DLconc(4)
      NO3=DLconc(5)
      SO2=DLconc(6)

C     Compute reaction rates.

      W(1)=RK(1)*O3*NO
      W(2)=RK(2)*NO2
      W(3)=RK(3)*OH*NO2
      W(4)=RK(4)*O3*NO2
      W(5)=RK(5)*NO2*ROH
      W(6)=RK(6)*PAN
      W(7)=RK(7)*HNO3
      W(8)=RK(8)*NO3
      W(9)=RK(9)*SO2

C     Compute chemical production terms.

      DLchem(1)=-W(1)+W(2)
      DLchem(2)=W(1)-W(2)-W(3)-2.D0*W(4)-W(5)+W(6)
      DLchem(3)=W(3)-W(7)+W(8)
      DLchem(4)=W(5)-W(6)
      DLchem(5)=2.D0*W(4)+W(7)-W(8)
      DLchem(6)=-W(9)
      DLchem(7)=W(9)

C     Add volumic source terms if necessary.

      IF(LSOURC)THEN
         DO Ji=1,nespl
            DLchem(Ji)=DLchem(Ji)+ZCsourc(Ji)
         ENDDO
      ENDIF   

      END
