C------------------------------------------------------------------------
C     
C     Copyright (C) 2001-2003 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE roschem (DLconc,
     s     ZCsourc,ZCsourcf,
     s     DLtstart,DLtend,Rki,Rkf,
     s     O3,OH,ROH,O3f,OHf,ROHf) 
      
C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This routine computes one timestep for gas-phase chemistry EMEP
C     in one grid cell. The solver is the second-order Rosenbrock
C     method (see the user's guid).
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     ZCSOURC: array of chemical volumic emissions at initial time.
C     ZCSOURCF: array of chemical volumic emissions at final time.
C     DLTSTART: initial time (GMT, computed from January 1st, [s]).
C     DLTEND: final time (GMT, computed from January 1st, [s]).
C     DLRKI: kinetic rates at initial time.
C     DLRKF: kinetic rates at final time.
C     O3/O3F: initial/final forced concentration for O3.
C     The same holds for OH and ROH.
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     DLCONC: array of chemical concentrations ([\mu.g/m^3]). 
C     # Before entry, it is given at initial time of the timestep.
C     # On exit, it is computed at final time of the timestep.
C     
C     -- OUTPUT VARIABLES
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Denis Qulo, Jaouad Boutahar, November 2001.
C     
C------------------------------------------------------------------------


C     -- INCLUDE FILES
C     PARADOM: parameters for all forced fields.
C     PARACHEM: parameters for sizes of 'chemical' arrays.

      IMPLICIT NONE
      INCLUDE 'PARADOM.INC'
      INCLUDE 'PARACHEM.INC'

      INTEGER Ji, Jj
      INTEGER IPVT(nespl)
      DOUBLE PRECISION DLconcbis(nespl), DLconc(nespl),DLr(nespl)
      DOUBLE PRECISION DLstep, DLtend, DLtstart
      DOUBLE PRECISION Igamma
      DOUBLE PRECISION DLk1(nespl), DLk2(nespl)
      DOUBLE PRECISION DLb1(nespl), DLb2(nespl)
      DOUBLE PRECISION DLmat(nespl,nespl)
      DOUBLE PRECISION DLmatlu(nespl,nespl)
      DOUBLE PRECISION DLdrdc(nespl,nespl)
      DOUBLE PRECISION ZCsourc(NESP),ZCsourcf(NESP)
      DOUBLE PRECISION Rki(Nr),Rkf(Nr)
      DOUBLE PRECISION O3,ROH,OH
      DOUBLE PRECISION O3f,ROHf,OHf

C     Numerical initialization.

      DLstep=DLtend-DLtstart
      Igamma = 1.D0 + 1.D0 / DSQRT(2.D0)
      
C------------------------------------------------------------------------
C     1 - First step

C     Compute chemical production terms at initial time (DLR).

      CALL fexchem (DLconc,ZCsourc,Rki,O3,OH,ROH,Dlr)

C     Compute the Jacobian at initial time (DLRDC).
      
      CALL jacdchemdc (RKi,O3,OH,ROH,DLdrdc)		

C     Compute K1 by solving (1-Igamma*dt*DLRDC) DLK1 = DLR

      DO Ji=1,nespl
         DLb1(Ji) = DLr(Ji)
         DO Jj=1,nespl
            DLmat(Ji,Jj) = - Igamma*DLstep*DLdrdc(Ji,Jj)
         ENDDO
         DLmat(Ji,Ji) = 1.D0 + DLmat(Ji,Ji)
      ENDDO

      CALL solvlin (0,DLmat,DLmatlu,IPVT,DLk1,DLb1)

C------------------------------------------------------------------------
C     2 - Second step

C     Compute first-order approximation (DLCONCBIS).

      DO Ji=1,nespl
         DLconcbis(Ji) = DLconc(Ji) + DLstep * DLk1(Ji)
         IF (DLconcbis(Ji) .LT. 0.D0) THEN
            DLconcbis(Ji)=0.D0
            DLk1(Ji) = (DLconcbis(Ji) - DLconc(Ji)) / DLstep
         ENDIF
      ENDDO

C     Compute chemical production terms (DLR) at final time with
C     the first-order approximation.
      
      CALL fexchem (DLconcbis,ZCsourcf,Rkf,O3f,OHf,ROHf,DLr)

C     Compute DLK2 by solving (optimized scheme)
C     (1-Igamma*dt*DLRDC) DLK2 = DLR-2 DLK1 

      DO Ji=1,nespl
         DLb2(Ji) =  DLr(Ji) - 2.D0*DLk1(Ji)
      ENDDO
      
      CALL solvlin (1,DLmat,DLmatlu,IPVT,DLk2,DLb2)

C------------------------------------------------------------------------
C     3 - Compute concentrations at final time (optimized scheme):
C     DLCONC+(3*DLRK1+DLRK2)*dt/2
      
      DO Ji=1,nespl
         DLconc(Ji) = DLconc(Ji) + 1.5D0 * DLstep * DLk1(Ji) 
     &        + 0.5D0 * DLstep * DLk2(Ji)

         IF (DLconc(Ji) .LT. 0.D0) THEN
            IF (DLconc(Ji) .LT. -1.D-8) THEN
               WRITE(*,*)'CLIP chemistry',DLconc(Ji),Ji
            ENDIF
            DLconc(Ji) = 0.D0
         ENDIF
      ENDDO

      END
