C------------------------------------------------------------------------
C     
C     Copyright (C) 2001-2003 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE SCAMER(ts,DLtemp,DLrain,DLhcloud,DLcforc2d,
     $     tf,DLtempf,DLrainf,DLhcloudf,DLcforc2df,
     $     DLconc)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This routine computes one timestep for scavenging.
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     DLTEMP: 3D temperature field at initial time. ([K])
C     DLRAIN: 2D rain intensity field at initial time. ([mm/h])
C     DLHCLOUD: 2D height of cloud basis at initial time. ([m])
C     DLCFORC: array of 2D forced chemical concentration fields at
C     # initial time ([\mu.g/m^3]).
C     # The third index labels the chemical species in the order:
C     # O3-Part.
C     
C     The same variables are defined at final time of the timestep.
C     'f' is then put at the end of the name.
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     DLCONC: array of 3D chemical concentrations ([\mu.g/m^3]). 
C     # Before entry, it is given at initial time of the timestep.
C     # On exit, it is computed at final time of the timestep.
C     
C     -- OUTPUT VARIABLES
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Yelva Roustan, CEREA, February 2003.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

C     -- INCLUDE FILES
C     CONST: constants.
C     PARADOM: parameters for all forced fields.
C     PARACHEM: parameters for sizes of 'chemical' arrays.
C     COMDOM: global variables for the domain.
C     COMUSE: flags and parameters for numerics. 

      INCLUDE 'CONST.INC'
      INCLUDE 'PARADOM.INC'
      INCLUDE 'PARACHEM.INC'
      INCLUDE 'COMDOM.INC'
      INCLUDE 'COMFILES.INC'
      INCLUDE 'COMUSE.INC'

      DOUBLE PRECISION DLtemp(NXt,NYt,NZt),DLtempf(NXt,NYt,NZt)
      DOUBLE PRECISION DLrain(NXr,NYr),DLrainf(NXr,NYr)
      DOUBLE PRECISION DLhcloud(NXr,NYr),DLhcloudf(NXr,NYr)

      DOUBLE PRECISION ts,tf

      DOUBLE PRECISION DLscavg(NXr,NYr,nesp)
      DOUBLE PRECISION DLscavgf(NXr,NYr,nesp)
      DOUBLE PRECISION DLconc(NX,NY,NZ,nesp)
      DOUBLE PRECISION DLCforc2d(NX,NY,nespf)
      DOUBLE PRECISION DLCforc2df(NX,NY,nespf)
      DOUBLE PRECISION DLCf(NX,NY,nespf)
      DOUBLE PRECISION DLCff(NX,NY,nespf)
      DOUBLE PRECISION DLtemptot(NXt,NYt)
      DOUBLE PRECISION DLtemptotf(NXt,NYt)
      DOUBLE PRECISION DLconctot(NX,NY,nesp)
      DOUBLE PRECISION DLconctotf(NX,NY,nesp)

      INTEGER i,j,k,l

C------------------------------------------------------------------------
C     1 - Initializations

      DO l=1,nesp
         DO j=1,NY
            DO i=1,NX
               DLconctot(i,j,l) = 0.d0
               DLconctotf(i,j,l) = 0.d0
            ENDDO
         ENDDO
      ENDDO

      DO j=1,NY
         DO i=1,NX
            DLtemptot(i,j) = 0.d0
            DLtemptotf(i,j) = 0.d0
         ENDDO
      ENDDO
C------------------------------------------------------------------------
C     1 - Scavenging coefficient

      DO j=1,NYr
         DO i=1,NXr

            DO k=1,NZ

               IF (maillage(k).LE.DLhcloud(i,j)) THEN
                  DLtemptot(i,j) = DLtemptot(i,j)
     &                 + (maillage(k+1)-maillage(k)) * DLtemp(i,j,k)
                  DLtemptotf(i,j) = DLtemptotf(i,j) + DLtemp(i,j,k)
     &                 + (maillage(k+1)-maillage(k)) * DLtempf(i,j,k)
               ELSE
                  DLtemptot(i,j) = DLtemptot(i,j)
                  DLtemptot(i,j) = DLtemptot(i,j)
               ENDIF

            ENDDO

            DLtemptot(i,j) = DLtemptot(i,j) / DLhcloud(i,j)
            DLtemptotf(i,j) = DLtemptotf(i,j) / DLhcloudf(i,j)

         ENDDO
      ENDDO

C     Conversion to compute scav.

      DO j=1,NYf
         DO i=1,NXf

            DLCf(i,j,1) = DLCforc2d(i,j,1) / 48.d9 !ug/m^3 --> mol/L.
            DLCff(i,j,1) = DLCforc2df(i,j,1) / 48.d9 !ug/m^3 --> mol/L.

            DLCf(i,j,2) = 1.d-6 * DLCforc2d(i,j,2) !ug/m^3 --> g/m^3.
            DLCff(i,j,2) = 1.d-6 * DLCforc2df(i,j,2) !ug/m^3 --> g/m^3.

         ENDDO
      ENDDO

      CALL SCAVRATMER(DLrain,DLrainf,DLtemptot,DLtemptotf
     &     ,DLhcloud,DLhcloudf,DLCf,DLCff
     &     ,DLscavg,DLscavgf)

C------------------------------------------------------------------------
C     2 - Scavenging computing 

      DO l=1,nesp
         DO j=1,NYr
            DO i=1,NXr

               DO k=1,Nz

                  IF (maillage(k).LE.DLhcloud(i,j)) THEN
                     DLconctot(i,j,l) = DLconctot(i,j,l)
     &                    + (maillage(k+1)-maillage(k))
     $                    * DLconc(i,j,k,l)
                  ELSE
                     DLconctot(i,j,l) = DLconctot(i,j,l)
                  ENDIF

               ENDDO

C     Average concentration.

               DLconctot(i,j,l) = DLconctot(i,j,l) / DLhcloud(i,j)

            ENDDO
         ENDDO
      ENDDO
      
C     Computing scavenging.
      
      DO l=1,nesp

         DO j=1,NYr
            DO i=1,NXr

               DLconctotf(i,j,l) = DLconctot(i,j,l) * DEXP(
     &              - ( DLscavg(i,j,l) + DLscavgf(i,j,l) ) / 2.d0 * 
     &              (tf-ts) )
            ENDDO        
         ENDDO        

         IF(MOD(ts,(NFSAV*(tf-ts))).EQ.0.d0)THEN
            OPEN(unit=1,file=file_res_scavg(l),
     s           access='direct',form='unformatted',recl=4*Nxr*Nyr)
            WRITE(unit=1,rec=1+INT(0.1d0+ts/(NFsav*(tf-ts))))
     s           ((REAL((DLconctot(i,j,l)-DLconctotf(i,j,l))/5233.d0
     $           ),i=1,nxr),j=1,nyr)
            CLOSE(1)
         ENDIF


      ENDDO

C     Vertical losses distribution.

      DO l=1,nesp
         DO k=1,NZ
            DO j=1,NYr
               DO i=1,NXr

                  IF (DLconctot(i,j,l).GT.0.d0) THEN
                     DLconc(i,j,k,l) = DLconc(i,j,k,l) *
     &                    ( DLconctotf(i,j,l) / DLconctot(i,j,l) )
                  ELSE
                     DLconc(i,j,k,l) = 0.d0
                  ENDIF

               ENDDO
            ENDDO
         ENDDO
      ENDDO

      END
