C------------------------------------------------------------------------
C     
C     Copyright (C) 2001-2003 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE SCAVRATMER(DLrain,DLrainf,DLtemptot,DLtemptotf
     &     ,DLhcloud,DLhcloudf,DLCforc2d,DLCforc2df
     &     ,DLscavg,DLscavgf)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This routine computes scavenging coefficient for gazeous mercury
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     DLTEMPTOT: 2D average temperature field at initial time. ([K])
C     DLRAIN: 2D rain intensity field at initial time. ([mm/h])
C     DLHCLOUD: 2D height of cloud basis at initial time. ([m])
C     DLCFORC: array of 2D forced chemical concentration fields at
C     # initial time ([\mu.g/m^3]).
C     # The third index labels the chemical species in the order:
C     # O3-Part.
C     
C     The same variables are defined at final time of the timestep.
C     'f' is then put at the end of the name.
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     
C     -- OUTPUT VARIABLES
C     
C     DLSCAVG: array of 2D scavenging ratio([/s]). 
C     The same variable is defined at final time of the timestep.
C     'f' is then put at the end of the name.
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Yelva Roustan, CEREA, February 2003.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

C     -- INCLUDE FILES
C     CONST: constants.
C     PARADOM: parameters for all forced fields.
C     PARACHEM: parameters for sizes of 'chemical' arrays.
C     COMPHYS: physical data for chemical species (molar mass). 

      INCLUDE 'CONST.INC'
      INCLUDE 'PARADOM.INC'
      INCLUDE 'PARACHEM.INC'
      INCLUDE 'COMPHYS.INC'

      DOUBLE PRECISION DLCforc2d(NX,NY,nespf)
      DOUBLE PRECISION DLCforc2df(NX,NY,nespf)
      DOUBLE PRECISION DLtemptot(NXt,NYt),DLtemptotf(NXt,NYt)
      DOUBLE PRECISION DLrain(NXr,NYr),DLrainf(NXr,NYr)
      DOUBLE PRECISION DLhcloud(NXr,NYr),DLhcloudf(NXr,NYr)

C     Mercury washing ratio.
      DOUBLE PRECISION DLWHg(NX,NY),DLWHgf(NX,NY)
C     Adimensionnal henry constant.
      DOUBLE PRECISION DLHe(NX,NY,nscav+1),DLHef(NX,NY,nscav+1)
C     Chemical constant.
      DOUBLE PRECISION k1,k3,Kstar,rsoot

      DOUBLE PRECISION DLscavg(NXr,NYr,nesp),DLscavgf(NXr,NYr,nesp)

      INTEGER i,j,l

C------------------------------------------------------------------------
C     1 - Initializations

      k1=4.7d7
      k3=4.d-4
      Kstar=5.d-6
      rsoot=0.5d-6      

C------------------------------------------------------------------------
C     2 - Preliminary calculations

C     Adimensional Henry constant.

      DO l=1,Nscav
         DO j=1,NYr
            DO i=1,NXr

               DLHe(i,j,l)= 100.d0 / ( Hhenryscav(l)
     $              * Pr * DLtemptot(i,j) )
               DLHef(i,j,l)= 100.d0 / ( Hhenryscav(l)
     $              * Pr * DLtemptotf(i,j) )

            ENDDO
         ENDDO
      ENDDO

      DO j=1,NYr
         DO i=1,NXr

            DLHe(i,j,Nscav+1)= 100.d0 / ( 0.013d0
     $           * Pr * DLtemptot(i,j) )
            DLHef(i,j,Nscav+1)= 100.d0 / ( 0.013d0
     $           * Pr * DLtemptotf(i,j) )

         ENDDO
      ENDDO
      
C     Mercury washing ratio.

c     Time = ts.

      DO j=1,NYr
         DO i=1,NXr

C     Computing if raining.

            IF(DLrain(i,j).GT.0.d0)THEN
               DLWHg(i,j)=k1 * DLCforc2d(i,j,1)
     &              / ( k3 * DLHe(i,j,1) * DLHe(i,j,2) )
     &              * ( 1.d0 + Kstar * 5.d5
     $              * DLCforc2d(i,j,2) / rsoot )
            ENDIF

         ENDDO
      ENDDO

C     Time = tf.

      DO j=1,NYr
         DO i=1,NXr

C     Computing if raining.

            IF(DLrainf(i,j).GT.0.d0)THEN
               DLWHgf(i,j)=k1 * DLCforc2df(i,j,1)
     &              / (k3 * DLHef(i,j,1) * DLHef(i,j,2) )
     &              * ( 1.d0 + Kstar * 5.d5
     $              * DLCforc2df(i,j,2) / rsoot )
            ENDIF

         ENDDO
      ENDDO
C------------------------------------------------------------------------
C     3 - Scavenging coefficient computing

      DO l=1,nesp
         DO j=1,NYr
            DO i=1,NXr

               IF (DLrain(i,j).GT.0.d0) THEN
                  DLscavg(i,j,l) = DLWHg(i,j) * DLrain(i,j)
     $                 / DLhcloud(i,j) / 3600.d3 
               ELSE
                  DLscavg(i,j,l) = 0.d0
               ENDIF

               IF (DLrainf(i,j).GT.0.d0) THEN
                  DLscavgf(i,j,l) = DLWHgf(i,j) * DLrainf(i,j)
     $                 / DLhcloudf(i,j) / 3600.d3
               ELSE
                  DLscavgf(i,j,l) = 0.d0
               ENDIF

            ENDDO
         ENDDO
      ENDDO

      END
