C------------------------------------------------------------------------
C     
C     Copyright (C) 2001-2003 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE chem (ts,DLrain,DLhcloud,DLattenuation,DLhumid,DLtemp,
     s     DLpress,DLCsourc,DLCforc2d,DLCphotolysis_rates,
     s     tf,DLrainf,DLhcloudf,DLattenuationf,DLhumidf,DLtempf,
     s     DLpressf,DLCsourcf,DLCforc2df,DLCphotolysis_ratesf,
     s     DLconc)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This routine computes one timestep for gas-phase chemistry MOCA.
C     Chemical kinetics is solved in each grid cell.
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     TS: initial time (GMT, computed from January 1st, [s]).
C     DLRAIN: 2D rain intensity field at initial time ([mm/h]).
C     DLHCLOUD: 2D height of cloud basis ([m])
C     DLATTENUATION: 3D cloud attenuation field at initial time.
C     DLHUMID: 3D specific humidity field at initial time ([-]).
C     DLTEMP: 3D temperature field at initial time ([K]).
C     DLPRESS: 3D pressure field at initial time ([Pa]).
C     DLCFORC: array of 2D forced chemical concentration fields at
C     # initial time ([\mu.g/m^3]).
C     DLCSOURC: array of chemical volumic emissions at initial time
C     # ([\mu.g/m^3/s]).
C     DLCPHOTOLYSIS_RATES: photochemical kinetic rates
C     # at initial time ([s^{-1}]).
C     
C     The same variables are defined at final time of the timestep.
C     'f' is then put at the end of the name.
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     DLCONC: array of 3D chemical concentrations ([\mu.g/m^3]). 
C     # Before entry, it is given at initial time of the timestep.
C     # On exit, it is computed at final time of the timestep.
C     
C     -- OUTPUT VARIABLES
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     2002/02/26: new treatment of sources (Jaouad Boutahar, CEREA).
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Denis Qulo, CEREA, June 2001.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

C     -- INCLUDE FILES
C     CONST: constants.
C     PARADOM: parameters for all forced fields.
C     PARACHEM: parameters for sizes of 'chemical' arrays.
C     COMUSE: flags and parameters for numerics. 
C     COMDOM: flags, global variables and parameters for forced fields. 
C     COMCHEM: global variables for chemical species. 

      INCLUDE 'CONST.INC'
      INCLUDE 'PARADOM.INC'
      INCLUDE 'PARACHEM.INC'
      INCLUDE 'COMUSE.INC'  
      INCLUDE 'COMDOM.INC'  
      INCLUDE 'COMCHEM.INC'
      
      DOUBLE PRECISION ts,tf
      DOUBLE PRECISION tschem,tfchem

      DOUBLE PRECISION DLconc(NX,NY,NZ,nesp),ZC(NESP)
      DOUBLE PRECISION DLtemp(NXt,NYt,NZt),DLtempf(NXt,NYt,NZt)
      DOUBLE PRECISION DLrain(NXr,NYr),DLrainf(NXr,NYr)
      DOUBLE PRECISION DLhcloud(NXr,NYr),DLhcloudf(NXr,NYr)
      DOUBLE PRECISION DLattenuation(NXatt,NYatt,NZatt)
      DOUBLE PRECISION DLattenuationf(NXatt,NYatt,NZatt)
      DOUBLE PRECISION DLhumid(NXh,NYh,NZh),DLhumidf(NXh,NYh,NZh)
      DOUBLE PRECISION DLCsourc(Nx,Ny,Nzs,Nsrc)
      DOUBLE PRECISION DLCsourcf(Nx,Ny,Nzs,Nsrc)
      DOUBLE PRECISION ZCsourc(NESP)
      DOUBLE PRECISION ZCsourcf(NESP)
      DOUBLE PRECISION DLRki(Nr),DLRkf(Nr)
      DOUBLE PRECISION DLpress(NXp,NYp,NZp),DLpressf(NXp,NYp,NZp)
      DOUBLE PRECISION DLCforc2d(Nxf,Nyf,NESPF)
      DOUBLE PRECISION DLCforc2df(Nxf,Nyf,NESPF)
      DOUBLE PRECISION DLCphotolysis_rates(NRphot,Nx,Ny,Nz)
      DOUBLE PRECISION DLCphotolysis_ratesf(NRphot,Nx,Ny,Nz)

      DOUBLE PRECISION Zangzen,Zangzenf
      DOUBLE PRECISION Zatt,Zattf

      DOUBLE PRECISION muzero
      EXTERNAL muzero

      INTEGER Jt,Ji,Jj,Jk,Jsp,i,NCASE

C     Initialization for volumic sources.

      DO Jsp=1,Nesp
         ZCsourc(Jsp)=0.D0
         ZCsourcf(Jsp)=0.D0
      ENDDO   

C     Conversion \mu.g/m3 to molecules/cm3.

      Ncase=1
      CALL CONVERS(Ncase,DLconc,DLCsourc,DLCsourcf)

C     Loop on grid cells.

      DO Jk=1,NZ
         DO Jj=1,NY
            DO Ji=1,NX

C     Spatial extraction for volumic sources.

               IF (LSOURC)THEN
                  DO Jsp=1,Nsrc
                     IF(Jk.LE.Nzs)THEN
                        ZCsourc(Nesps(Jsp))=DLCsourc(Ji,Jj,Jk,Jsp)
                        ZCsourcf(Nesps(Jsp))=DLCsourcf(Ji,Jj,Jk,Jsp)
                     ELSE
                        ZCsourc(Nesps(Jsp))=0.D0
                        ZCsourcf(Nesps(Jsp))=0.D0
                     ENDIF
                  ENDDO
               ENDIF

C     Cloud attenuation.

               IF (LATTENUATION) THEN
                  Zatt = DLattenuation(Ji, Jj, Jk)
                  Zattf = DLattenuationf(Ji, Jj, Jk)
               ELSE
                  Zatt = 1.D0
                  Zattf = 1.D0
               ENDIF

C     Projection.

               DO Jsp=1,Nesp
                  ZC(Jsp) = DLconc(Ji,Jj,Jk,Jsp) 
               ENDDO

C     Integration of chemistry (eventually with subcycling).

               DO Jt=1,Ncycle
                  tschem=ts+(Jt-1)*Dtstep/Ncycle
                  tfchem=tschem+Dtstep/Ncycle
                  Zangzen=dabs(DACOS(muzero(tschem,Dlon(Ji,Jj,Jk),
     s                 Dlat(Ji,Jj,Jk)))*180.D0/PI)
                  Zangzenf=dabs(DACOS(muzero(tfchem,Dlon(Ji,Jj,Jk),
     s                 Dlat(Ji,Jj,Jk)))*180.D0/PI)
                  CALL Kinetic(DLRKi,DLtemp(Ji,Jj,Jk),
     s                 DLhumid(Ji,Jj,Jk),DLpress(Ji,Jj,Jk), Zangzen, 
     s                 Zatt)
                  CALL Kinetic(DLRKf,DLtempf(Ji,Jj,Jk),
     s                 DLhumidf(Ji,Jj,Jk),DLpressf(Ji,Jj,Jk), Zangzenf,
     s                 Zattf)

C     Photolytic reactions may be read.

                  IF (LREAD_PHOTOLYSIS) THEN
                     DO i=1,Nrphot
                        DLRKi(Nreactphot(i)) = Zatt *
     $                       DLCphotolysis_rates(i, Ji, Jj, Jk)
                        DLRKf(Nreactphot(i)) = Zattf *
     $                       DLCphotolysis_ratesf(i, Ji, Jj, Jk)
                     ENDDO
                  ENDIF                  

                  CALL roschem (ZC,ZCsourc,ZCsourcf,tschem,tfchem,
     s                 DLRki,DLRkf,Ji,Jj,Jk)
               ENDDO

C     Storage in the 3D array of chemical concentrations.

               DO i=1,NESP
                  DLconc(Ji,Jj,Jk,i) = ZC(i) 
               ENDDO

            ENDDO               !x
         ENDDO                  !y
      ENDDO                     !z

C     Conversion of concentrations molecules/cm3  to \mu. g/m3.

      Ncase=2
      CALL CONVERS(Ncase,DLconc,DLCsourc,DLCsourcf)
      
      END
