C------------------------------------------------------------------------
C     
C     Copyright (C) 2001-2003 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE rosdiff_y (DLky,DLconc,ts,tf)
      
C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This routine solves diffusion along Y for one timestep.
C     The numerical scheme is a second order Rosenbrock.
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     DLKY: array of K_y (diffusion coefficient) along Y.
C     TS: initial time.
C     TF: final time.
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     DLCONC: array of chemical concentrations along Y.
C     
C     -- OUTPUT VARIABLES
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Denis Qulo, CEREA, June 2001.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

C     -- INCLUDE FILES
C     PARADOM: parameters for all forced fields.
C     COMKDIFY: global variables for diffusion along Y.
C     COMUSE: flags and parameters for numerics. 

      INCLUDE 'PARADOM.INC'
      INCLUDE 'COMKDIFY.INC'
      INCLUDE 'COMUSE.INC'

      INTEGER  Ji
      DOUBLE PRECISION DLconcbis(Ny), DLconc(Ny)
      DOUBLE PRECISION DLky(Ny)
      DOUBLE PRECISION DLstep,ts,tf
      DOUBLE PRECISION Igamma
      DOUBLE PRECISION DLk1(Ny), DLk2(Ny)
      DOUBLE PRECISION DLb1(Ny), DLb2(Ny)


      DLstep=tf-ts
      Igamma = 1.D0 + 1.D0 / DSQRT(2.D0)

C     -- ROSENBROCK
C     The numerical scheme reads:
C     DLCONC = DLCONC + 3/2 (TF-TS) K1 + 1/2 (TF-TS) K2
C     where K1 and K2 are computed as follows.

C------------------------------------------------------------------------
C     1 - Compute K1
      
C     div(K_y \partial_y c).
      CALL fexdiff_y (DLconc,DLky,DLb1)
      
      IF (LITERDIFFY) THEN
         
C     Jacobian matrix.
         CALL jacddiffdc_y(DLky)

         DO Ji=1,Ny
            DLmaty(Ji) = 1.D0-Igamma*DLstep*DLdy(Ji)
            IF (Ji.NE.Ny) THEN  
               DLmatyu(Ji) = -Igamma*DLstep*DLdyu(Ji)
               DLmatyl(Ji) = -Igamma*DLstep*DLdyl(Ji)
            ENDIF
         ENDDO

      ENDIF

      CALL solvtridiag_y (DLmatyl,DLmaty,DLmatyu,DLk1,DLb1)

C------------------------------------------------------------------------
C     2 - Compute K2

      DO Ji=1,Ny
         DLconcbis(Ji) = DLconc(Ji) + DLstep * DLk1(Ji)
         IF (DLconcbis(Ji) .LT. 0.D0) THEN
            DLconcbis(Ji)=0.D0
            DLk1(Ji) = (DLconcbis(Ji) - DLconc(Ji)) / DLstep
         ENDIF
      ENDDO
      
      CALL fexdiff_y (DLconcbis,DLky,DLb2)

      DO Ji=1,Ny
         DLb2(Ji) =  DLb2(Ji) - 2.D0*DLk1(Ji)
      ENDDO
      
      CALL solvtridiag_y (DLmatyl,DLmaty,DLmatyu,DLk2,DLb2)

C------------------------------------------------------------------------
C     3 - Compute DLCONC
      
      DO Ji=1,Ny
         DLconc(Ji) = DLconc(Ji) + 1.5D0 * DLstep * DLk1(Ji) 
     &        + 0.5D0 * DLstep * DLk2(Ji)

C------------------------------------------------------------------------
C     4 - Clipping

         IF (DLconc(Ji) .LT. 0.D0) THEN
            IF (LCOMMENT.AND.(DLconc(Ji).LT.-1.D-8))
     $           WRITE(*,*)'CLIP diffusion y',DLconc(Ji),Ji
            DLconc(Ji) = 0.D0
         ENDIF
      ENDDO

      END
