C------------------------------------------------------------------------
C     
C     Copyright (C) 2001-2003 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE rosdiff_z (DLconc,ts,tf,Dkz,Dkzf,Zdm,Zdc,
     s     Dcdep,Dcemis,Dcdepf,Dcemisf)
      
C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This routine solves diffusion along Z for one timestep.
C     The numerical scheme is a second order Rosenbrock.
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     DLKZ: array of K_z (diffusion coefficient) along Z.
C     TS: initial time.
C     TF: final time.
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     DLCONC: array of chemical concentrations along Z.
C     
C     -- OUTPUT VARIABLES
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Denis Qulo, CEREA, June 2001.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

C     -- INCLUDE FILES
C     PARADOM: parameters for all forced fields.
C     COMUSE: flags and parameters for numerics. 

      INCLUDE 'PARADOM.INC'
      INCLUDE 'COMUSE.INC'

      INTEGER  Ji
      DOUBLE PRECISION DLconcbis(Nz),DLconc(Nz)
      DOUBLE PRECISION DLstep,ts,tf
      DOUBLE PRECISION Zdm(Nz),Zdc(Nz)
      DOUBLE PRECISION Igamma
      DOUBLE PRECISION DLk1(Nz),DLk2(Nz)
      DOUBLE PRECISION DLb1(Nz),DLb2(Nz)
      DOUBLE PRECISION DLmatz(Nz)
      DOUBLE PRECISION DLmatzu(Nz)
      DOUBLE PRECISION DLmatzl(Nz)
      DOUBLE PRECISION DLdz(Nz)
      DOUBLE PRECISION DLdzu(Nz)
      DOUBLE PRECISION DLdzl(Nz)
      DOUBLE PRECISION Dkz(Nz+1),Dkzf(Nz+1)
      DOUBLE PRECISION Dcdep,Dcdepf
      DOUBLE PRECISION Dcemis,Dcemisf


      DLstep=tf-ts
      Igamma = 1.D0+1.D0/DSQRT(2.D0)

C     -- ROSENBROCK
C     The numerical scheme reads:
C     DLCONC = DLCONC + 3/2 (TF-TS) K1 + 1/2 (TF-TS) K2
C     where K1 and K2 are computed as follows.

C------------------------------------------------------------------------
C     1 - Compute K1
      
C     div(K_z \partial_z c).
      CALL fexdiff_z (DLconc,DLb1,Dkz,Zdm,Zdc,Dcdep,Dcemis)
      
C     Jacobian matrix.
      CALL jacddiffdc_z (DLdz,DLdzu,DLdzl,Dkz,Zdm,Zdc,Dcdep)

      DO Ji=1,Nz
         DLmatz(Ji) = 1.D0-Igamma*DLstep*DLdz(Ji)
         IF (Ji.NE.Nz) THEN  
            DLmatzu(Ji) = -Igamma*DLstep*DLdzu(Ji)
            DLmatzl(Ji) = -Igamma*DLstep*DLdzl(Ji)
         ENDIF
      ENDDO

      CALL solvtridiag_z (DLmatzl,DLmatz,DLmatzu,DLk1,DLb1)

C------------------------------------------------------------------------
C     2 - Compute K2

      DO Ji=1,Nz
         DLconcbis(Ji) = DLconc(Ji) + DLstep * DLk1(Ji)
         IF (DLconcbis(Ji) .LT. 0.D0) THEN
            DLconcbis(Ji)=0.D0
            DLk1(Ji) = (DLconcbis(Ji) - DLconc(Ji)) / DLstep
         ENDIF
      ENDDO
      
      CALL fexdiff_z (DLconcbis,DLb2,Dkzf,Zdm,Zdc,Dcdepf,Dcemisf)

      DO Ji=1,Nz
         DLb2(Ji) =  DLb2(Ji) - 2.D0*DLk1(Ji)
      ENDDO
      
      CALL solvtridiag_z (DLmatzl,DLmatz,DLmatzu,DLk2,DLb2)

C------------------------------------------------------------------------
C     3 - Compute DLCONC
      
      DO Ji=1,Nz
         DLconc(Ji) = DLconc(Ji) + 1.5D0 * DLstep * DLk1(Ji) 
     &        + 0.5D0 * DLstep * DLk2(Ji)

C------------------------------------------------------------------------
C     4 - Clipping

         IF (DLconc(Ji) .LT. 0.D0) THEN
            IF (LCOMMENT.AND.(DLconc(Ji).LT.-1.D-8)) THEN
               WRITE(*,*) 'CLIP diffusion z',DLconc(Ji),Ji,Dkz(Ji),
     $              Dkzf(Ji)
            ENDIF
            DLconc(Ji) = 0.D0
         ENDIF
      ENDDO

      END

