C------------------------------------------------------------------------
C     
C     Copyright (C) 2001-2003 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE aero_QBL_rst (Iseason,typ,Dmz1,temp,tempsurf,
     $     press,presurf,snow,modwind,DLRa,DLRb)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This routine computes aerodynamic and quasi-laminar boundary layer
C     resistances for a given LUC
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     Iseason: season index
C     typ: Land use coverage percentage in a cell. 
C     Dmz1: First level height. ([m])
C     temp: temperature ([K])
C     tempsurf: surface temperature ([K])
C     press: pressure ([Pa])
C     presurf: surface pressure ([Pa])
C     snow: snow height ([m])
C     modwind: wind velocity module ([m/s])
C     
C     -- OUTPUT VARIABLES
C     
C     DLra: aerodynamic resistance ([s/m])
C     DLrb: quasi-laminar boundary layer resistance ([s/m])
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C     Parameterization used in this subroutine is presented in
C     " Paramtrisation de la vitesse de dpt sec dans
C     un code de transport diffusion de polluants " (A. Boudrot and
C     L. Musson Genon, Tech. Report CEREVE 2001 )
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     2001/06/01: A. BOUDROT, change to DOUBLE PRECISION
c     change in declaration of parameters, DLmodv as an argument
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Luc Musson-Genon, CEREA.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

C     -- INCLUDE FILES
C     PARADOM: parameters for all forced fields.
C     CONST: constants.
C     PARACHEM: parameters for sizes of 'chemical' arrays.
C     COMVDEPO: global variables for dry deposition velocities.
C     COMPHYS: physical data for chemical species (molar mass).

      INCLUDE 'PARADOM.INC'
      INCLUDE 'CONST.INC'
      INCLUDE 'PARACHEM.INC'
      INCLUDE 'COMVDEPO.INC'
      INCLUDE 'COMPHYS.INC'

      INTEGER Ji,Jdepot
      INTEGER Iland,Iseason

      DOUBLE PRECISION rugosd,rugost
      DOUBLE PRECISION DLtpot,DLtpotsurf
      DOUBLE PRECISION temp,tempsurf
      DOUBLE PRECISION press,presurf
      DOUBLE PRECISION snow
      DOUBLE PRECISION modwind
      DOUBLE PRECISION typ(Nland)

      DOUBLE PRECISION Dmz1,DLzr,DLustar,DLfh
      DOUBLE PRECISION DLra(Nland),DLrb(Nland,Ndep),DLrich

C     Constants of J.F. LOUIS's formulas.
      DOUBLE PRECISION DPb,DLc,DPd,DPc1
      PARAMETER (DPb=5.d0,DPc1=DPb,DPd=DPb)

      DOUBLE PRECISION DLzsz0t
      DOUBLE PRECISION DLrii
      DOUBLE PRECISION DLau,DLaut

      DOUBLE PRECISION DLnuair,DLeta,DLrho
      DOUBLE PRECISION DLeta1,DLrho1
      DOUBLE PRECISION DLeta2,DLrho2

C     Perfect gas constant for air.
      DOUBLE PRECISION DLrair

C------------------------------------------------------------------------
C     0. Setup

      DLrair = 28.8D0           ! Pr * 1.013D5 / 28.8d0

C------------------------------------------------------------------------
C     1. Computation of Potentiel temperature and Richardson number

      DLtpotsurf=tempsurf*(presurf/1.d5)**(-DLrair/PCp)
      DLtpot=temp*(press/1.d5)**(-DLrair/PCp)

      DO Iland=1,Nland
         IF (typ(Iland).GT.0.D0) THEN

            DLzr=Dmz1-DLhdep(Iland,Iseason)

            DLrich = 2.d0 * PG * DLzr
     &           * (DLtpot-DLtpotsurf)/(DLtpot+DLtpotsurf)
     &           / modwind**2.D0

C------------------------------------------------------------------------
C     2. Computation of dynamical and thermal roughness length

C     If Iland is sea and zo depends on u*, it is then recompute
C     iteratively using Charnock formula.

            IF(Iland.EQ.Ilandsea) THEN ! Case of the sea


               rugosd = DLrugdyn(Iland,Iseason)
               rugost = DLrugthe(Iland,Iseason)

               DO Ji=1,5        ! 5 loops are necessary to converge

                  CALL ustar(DLRich,modwind,DLzr,rugosd,rugost,DLustar)

                  rugosd=0.0144d0*DLustar*DLustar/PG
                  rugost=rugosd*DEXP(-2.d0)

               ENDDO

C     When snow depth is > 1cm, ground's characteristics change,
C     we take then Iseason=5.

            ELSEIF(snow.GE.1.e-2) THEN ! Case of snow 
               rugosd=DLrugdyn(Iland,5)
               rugost=DLrugthe(Iland,5)

            ELSE
               rugosd=DLrugdyn(Iland,Iseason)
               rugost=DLrugthe(Iland,Iseason)
            ENDIF

C------------------------------------------------------------------------
C     3. Computation of thermic stability

            DLzsz0t=(DLzr+rugost)/rugost

            DLau = PKa / DLOG( (DLzr+rugosd)/rugosd )
            DLaut = PKa / DLOG( DLzsz0t )

            DLc=DPc1*sqrt(1.d0-1.d0/DLzsz0t)
     $           *((DLzsz0t**(0.33d0)-1.d0)**1.5D0)

            DLrii=dabs(DLRich)
            IF(DLrich.LE.0.d0)THEN
               DLfh=1.d0+3.d0*DPb*DLrii
     &              /(1.d0+3.d0*DPb*DLc*DLau*DLaut*DLrii**.5d0)
            ELSE
               DLfh=1.d0/(1.d0+3.d0*DPb*DLrii*(1.d0+DPd*DLrii)**.5d0)
            ENDIF

C------------------------------------------------------------------------
C     4. Computation of aerodynamic resistance

            DLra(Iland) = DLau * DLaut * DLfh * modwind
            DLra(Iland) = DMAX1(DLra(Iland),1.d-10)
            DLra(Iland) = 1.d0/DLra(Iland)
            
C------------------------------------------------------------------------
C     5. Computation of friction velocity

            CALL ustar(DLRich,modwind,DLzr,rugosd,rugost,DLustar)

C------------------------------------------------------------------------
C     6. Computation of quasi-laminar boundary layer resistance

            DLeta1 = 1.83d-5 * (416.16d0/(tempsurf + 120.d0)) *
     $           ( tempsurf / 296.16d0 )**1.5d0
            DLeta2 = 1.83d-5 * (416.16d0/(temp + 120.d0)) *
     $           ( temp / 296.16d0 )**1.5d0
            DLeta = ( DLeta1 + DLeta2 ) / 2.d0

C     Mair/R USI = 28.97D-3/8.314 = 3.48d-3 
            DLrho1 = presurf * 3.48d-3 / tempsurf
            DLrho2 = press * 3.48d-3 / temp
            DLrho  = ( DLrho1 + DLrho2 ) / 2.d0

            DLnuair = DLeta / DLrho
            
            DO Jdepot=1,Ndep
               DLrb(Iland,Jdepot) = 2.d0/PKa
     s              *(1.d4*DLnuair/Ddifdep(Jdepot))**0.66d0
               DLrb(Iland,Jdepot) = DLrb(Iland,Jdepot) / DLustar
            ENDDO

         ENDIF
      ENDDO                     ! End loop on LUC

      END
