C------------------------------------------------------------------------
C     
C     Copyright (C) 2001-2003 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE canopy_rst (Iseason,temp,hum,esat,wsol,
     s     raysol,snow,cwc,rstcan)

C-----------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This routine computes canopy resistance for a given season and
C     a given species.
C     
C-----------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     Iseason: season index
C     temp: temperature 3D field ([K])
C     hum: relative humidity
C     esat: saturation water pressure ([Pa])
C     wsol: soil water content
C     raysol: solar irradiance ([w/m^2])
C     snow: snow height ([m])
C     cwc: canopy water content
C     
C     -- OUTPUT VARIABLES
C     
C     rstcan:  canopy resistance ([s/m])
C     
C-----------------------------------------------------------------------
C     
C     -- REMARKS
C     
C     Parameterization used in this subroutine is presented in
C     " Paramtrisation de la vitesse de dpt sec dans
C     un code de transport diffusion de polluants " (A. Boudrot and
C     L. Musson Genon, Tech. Report CEREVE 2001 )
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     01/07/04, by A. BOUDROT, new input DLcwc, new 
C     variable DLrwcut
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Luc Musson-Genon, CEREA.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

C     -- INCLUDE FILES
C     PARADOM: parameters for all forced fields.
C     CONST: constants.
C     PARACHEM: parameters for sizes of 'chemical' arrays.
C     COMVDEPO: global variables for dry deposition velocities.
C     COMFILES: global variables for files names.
C     COMCHEM: global variables for chemical species.

      INCLUDE 'PARADOM.INC'
      INCLUDE 'CONST.INC'
      INCLUDE 'PARACHEM.INC'
      INCLUDE 'COMVDEPO.INC'
      INCLUDE 'COMFILES.INC'
      INCLUDE 'COMCHEM.INC'

      INTEGER Jdepot,Iland,Iseas,Iseason
      
      DOUBLE PRECISION temp,hum,wsol
      DOUBLE PRECISION raysol
      DOUBLE PRECISION cwc
      DOUBLE PRECISION rstcan(Nland,Ndep)
      DOUBLE PRECISION snow
      DOUBLE PRECISION rstom(Nland,Ndep)
      DOUBLE PRECISION rwcutSO2,rwcutO3
      DOUBLE PRECISION esat
      DOUBLE PRECISION rinter

C-----------------------------------------------------------------------
C     0. Setup

C     Definition of DLrwcut for SO2 and O3 according to Brook & al
C     (cf README in ~boudrot/3d).

      rwcutSO2=50.d0
      rwcutO3=500.d0

C     When snow depth is > 1cm, ground's characteristics change, we take
C     then Iseason=5.
      
      IF (snow.LT.1.e-2) THEN
         Iseas=Iseason
      ELSE
         Iseas=5
      ENDIF

C-----------------------------------------------------------------------
C     1. Computing of the stomatal resistance.

      CALL stomatal_rst(temp,hum,esat,wsol,raysol,rstom)

C-----------------------------------------------------------------------
C     2. Computing of the canopy resistance.

      DO Iland=1,Nland
         DO Jdepot=1,Ndep

            IF((species_names(Nespd(Jdepot)).NE.'SO2').AND.
     $           (species_names(Nespd(Jdepot)).NE.'O3')) THEN
               rinter=1.D0/(rstom(Iland,Jdepot)+DLrstmes(Jdepot)) 
     $              + 1.D0/DLrstcut(Jdepot)
            ELSE
               rinter=(1.D0-cwc)/(rstom(Iland,Jdepot)+DLrstmes(Jdepot))
     $              + (1.D0-cwc)/DLrstcut(Jdepot)
               IF (species_names(Nespd(Jdepot)).EQ.'SO2')
     $              rinter = rinter + cwc/rwcutSO2
               IF (species_names(Nespd(Jdepot)).EQ.'O3')
     $              rinter = rinter + cwc/rwcutO3
            ENDIF
            rstcan(Iland,Jdepot) = 1.d0/DLrstsol(Jdepot,Iland,Iseas)
     $           + DLlai(Iland,Iseas)*rinter

            rstcan(Iland,Jdepot) = 1.d0/rstcan(Iland,Jdepot)

         ENDDO
      ENDDO

      END
