C------------------------------------------------------------------------
C     
C     Copyright (C) 2001-2003 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE stomatal_rst(temp,hum,esat,wsol,raysol,rstom)
      
C-----------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This routine computes stomatal resistance for a given season and
C     a given species.
C     
C-----------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     temp: temperature 3D field ([K])
C     hum: relative humidity
C     esat: saturation water pressure ([Pa])
C     wsol: soil water content
C     raysol: solar irradiance (w/m^2)
C     
C     -- OUTPUT VARIABLES
C     
C     rstom:  stomatal layer resistance ([s/m])
C     
C-----------------------------------------------------------------------
C     
C     -- REMARKS
C     
C     Parameterization used in this subroutine is presented in
C     " Paramtrisation de la vitesse de dpt sec dans
C     un code de transport diffusion de polluants " (A. Boudrot and
C     L. Musson Genon, Tech. Report CEREVE 2001 )
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     01/06, by A. BOUDROT, change to DOUBLE PRECISION
C     change in declaration of parameters
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Luc Musson-Genon, CEREA.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

C     -- INCLUDE FILES
C     PARADOM: parameters for all forced fields.
C     CONST: constants.
C     PARACHEM: parameters for sizes of 'chemical' arrays.
C     COMPHYS: physical data for chemical species (molar mass). 
C     COMVDEPO: global variables for dry deposition velocities.

      INCLUDE 'PARADOM.INC'
      INCLUDE 'CONST.INC'
      INCLUDE 'PARACHEM.INC'
      INCLUDE 'COMPHYS.INC'
      INCLUDE 'COMVDEPO.INC'

      INTEGER Jdepot,Iland

      DOUBLE PRECISION rstom(Nland,Ndep)

      DOUBLE PRECISION temp
      DOUBLE PRECISION hum
      DOUBLE PRECISION wsol 
      DOUBLE PRECISION raysol

      DOUBLE PRECISION dfsat
      DOUBLE PRECISION strhyd
      DOUBLE PRECISION strtmp
      DOUBLE PRECISION DLd_h2o,DLfd

      DOUBLE PRECISION DLbe

      DOUBLE PRECISION esat

      DOUBLE PRECISION DLbt

      
C------------------------------------------------------------------------
C     0. Setup

      DLd_h2o = 0.22d0          ! Water molecular difusivity (cm^2/s).

C     DLbe in 1/kPa 
C     Specific constant of plants (1/kpa) (jarvis, 1976).
      DLbe = 0.2d0

C------------------------------------------------------------------------
C     1. Water vapor pressure deficit

      dfsat = 1.d0 - DLbe*esat*(1.d0-hum)*1.d-3

C------------------------------------------------------------------------
C     2. water stress

      IF (wsol.GT.DLwcap) THEN
         strhyd=1.d0
      ELSEIF (wsol.GE.DLwfl) THEN
         strhyd=(wsol-DLwfl)/(DLwcap-DLwfl)
      ELSE
         strhyd=0.D0
      ENDIF
      
C------------------------------------------------------------------------
C     3. thermal stress

      DO Iland=1,Nland

         IF ((temp.LE.DLtmin(Iland)).OR.(temp.GE.DLtmax(Iland))) THEN
            strtmp=0.01d0
         ELSE
            DLbt=( DLtmax(Iland) - DLtze(Iland) )
     $           /( DLtze(Iland)-DLtmin(Iland) )
            strtmp=( temp - DLtmin(Iland) )
     $           /( DLtze(Iland)-DLtmin(Iland) )
     &           *( ( DLtmax(Iland) - temp )
     &           /( DLtmax(Iland) - DLtze(Iland) )   )**DLbt
         ENDIF
         
C------------------------------------------------------------------------
C     4.  Stomatal resistance
         
         DO Jdepot=1,Ndep

            DLfd = Ddifdep(Jdepot) / DLd_h2o

            rstom(Iland,Jdepot) = DLrstmin(Iland)
     &           * (1.d0+DLb1(Iland)/DMAX1(0.55d0*raysol,1.D-3))
     s           /DMAX1(dfsat*strtmp*strhyd*DLfd,1.D-16)

         ENDDO
      ENDDO

      END
