C------------------------------------------------------------------------
C     
C     Copyright (C) 2001-2003 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE vdepot(ts,DLmodwind,DLhum,DLtemp,DLtempsurf,
     $     DLpress,DLpresurf,
     $     DLsnow,DLraysol,DLlhflux,DLswat,DLrain,DLVdep)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This routine computes gas dry deposition velocity.
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     TS: TS past from the begining of the year. ([s])
C     DLMODWIND: 2D wind module. ([m/s])
C     DLHUM: 2D specific humidity field. ([-])
C     DLTEMP: 2D temperature field. ([K])
C     DLTEMPSURF: 2D surface temperature. ([K])
C     DLPRESS: 2D pressure field. ([Pa])
C     DLPRESURF: 2D surface pressure. ([Pa])
C     DLSNOW: 2D snow height field. ([m])
C     DLRAYSOL: 2D solar irradiance field.
C     DLLHFLUX: 2D latent heat flux. ([W/m^2])
C     DLSWAT: 3D soil water content field.
C     DLRAIN: 2D rain intensity field. ([mm/h])
C     
C     
C     -- OUTPUT VARIABLES
C     
C     DLVDEP: 3D dry deposition velocity field. ([m/s])
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C     Parameterization used in this subroutine is presented in
C     " Paramtrisation de la vitesse de dpt sec dans
C     un code de transport diffusion de polluants " (A. Boudrot and
C     L. Musson Genon, Tech. Report CEREVE 2001 )
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     01/06/21, by A. BOUDROT, DLvdro read in vdepot and 
C     no longer in SUDEPOT.
C     01/07/04, by A. BOUDROT, new meteorological inputs
C     (rain+dew), new variable CWC (Canopy Water Content) for the new
C     parametrisation of the canopy resistance.
C     02/04/01, by Y. ROUSTAN, addition of particle models
C     03/01/30, by Y. ROUSTAN, addition of gaseous mercury
C     03/03/01, by Y. ROUSTAN, merge in POLAIR
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Luc Musson-Genon, CEREA.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

C     -- INCLUDE FILES
C     PARADOM: parameters for all forced fields.
C     CONST: constants.
C     PARACHEM: parameters for sizes of 'chemical' arrays.
C     COMVDEPO: global variables for dry deposition velocities.
C     COMDOM: global variables for mesh and forcing fields. 

      INCLUDE 'PARADOM.INC'
      INCLUDE 'CONST.INC'
      INCLUDE 'PARACHEM.INC'
      INCLUDE 'COMVDEPO.INC'
      INCLUDE 'COMDOM.INC'

      DOUBLE PRECISION ts,time
      DOUBLE PRECISION DLmodwind(NXdd,NYdd),modwind
      DOUBLE PRECISION DLhum(NX,NY),hum
      DOUBLE PRECISION DLtemp(NXt,NYt),temp
      DOUBLE PRECISION DLtempsurf(NXdd,NYdd),tempsurf
      DOUBLE PRECISION DLpress(NXdd,NYdd),press
      DOUBLE PRECISION DLpresurf(NXdd,NYdd),presurf
      DOUBLE PRECISION DLsnow(NXdd,NYdd),snow
      DOUBLE PRECISION DLraysol(NXdd,NYdd),raysol
      DOUBLE PRECISION DLlhflux(NXdd,NYdd),lhflux
      DOUBLE PRECISION DLswat(NXdd,NYdd,Nlayer)
      DOUBLE PRECISION DLrain(NXr,NYr),rain
      DOUBLE PRECISION esat

C     Soil water content 
C     (adimensioned by the depth of the first layer 7 cm).
      DOUBLE PRECISION wsol
C     Canopy water content.
      DOUBLE PRECISION cwc

C     Averaged dry deposition velocity on a grid for each pollutant     
C     vdep for gas.
      DOUBLE PRECISION DLvdep(NXdd,NYdd,Ndep)

      INTEGER Ji,Jj,Jk,Jdepot

      INTEGER Iseason,Iland

      DOUBLE PRECISION Dmz1
C     Canopy resistance.
      DOUBLE PRECISION DLrstcan(Nland,Ndep)
C     Total resistance.
      DOUBLE PRECISION DLrstot
C     Aerodynamic resistance.
      DOUBLE PRECISION DLra(Nland)
C     Quasi-laminar surface boundary layer resistance.
      DOUBLE PRECISION DLrb(Nland,Ndep)
C     Land use coverage percentage in a cell.
      DOUBLE PRECISION typ(Nland)

C------------------------------------------------------------------------
C     1 - Setup

      DO Jdepot = 1, Ndep
         DO Jj = 1, NYdd
            DO Ji = 1, NXdd
               DLvdep(Ji,Jj,Jdepot) = 0.d0
            ENDDO
         ENDDO
      ENDDO

C     initialisation of soil moisture at permanent wilting point and 
C     soil moisture at field capacity since 1995 (Viterbo et Beljaars)
C     DLwfl=0.171
C     DLwcap=0.323
C     Initialisation of soil moisture at permanent wilting point and 
C     soil moisture at field capacity before 1995 (Viterbo et Beljaars).
      DLwfl=0.086d0
      DLwcap=0.171d0            ! DLwcap > DLwfl

C-----------------------------------------------------------------------
C     2 - Computing season index 

      time=MOD(ts,31536000.d0)

      IF((time.LE.5097600.d0).OR.(time.GT.26179200.d0))THEN
C     Iseason=1 winter; november, december, january,february.
         Iseason=1
      ELSEIF(time.LE.10368000.d0)THEN
C     Iseason=2 spring; march, april.
         Iseason=2
      ELSEIF(time.LE.20908800.d0)THEN
C     Iseason=3 summer; may june july august.
         Iseason=3
      ELSE
C     Iseason=4 fall; september october.
         Iseason=4
      ENDIF
      
C----------------------------------------------------------------------
C     3 - Computing fields not directly provided by meteo

C     Computing soil water content for water stress (adimensioned by the
C     depth of the first layer 7 cm).
C     Computing potential temperature at levels 1 and surface.
C     Computing canopy water content.

      DO Jj=1,NYdd
         DO Ji=1,NXdd

            wsol=0.D0
            tempsurf=DLtempsurf(Ji,Jj)
            temp=DLtemp(Ji,Jj)
            presurf=DLpresurf(Ji,Jj)
            press=DLpress(Ji,Jj)

            hum=DLhum(Ji,Jj)    
c     saturation water pressure.
            esat = 610.78d0*
     $           DEXP(17.2694d0*(temp-Ptkelvin)/(temp-35.86d0))
c     relative humidity at the first level in air.
            hum=hum*press/(0.622d0+0.378d0*hum)/esat
            hum=DMIN1(1.d0,hum)
            hum=DMAX1(0.d0,hum)

            lhflux=DLlhflux(Ji,Jj)
            rain=DLrain(Ji,Jj)
            snow=DLsnow(Ji,Jj)
            modwind=DLmodwind(Ji,Jj)
            raysol=DLraysol(Ji,Jj)
            Dmz1=Dmz(Ji,Jj,1)
            
            DO Iland=1,Nland
               typ(Iland)=DLtyp(Ji,Jj,Iland)
            ENDDO

            DO Jk=1,Nlayer
               wsol=wsol+DLswat(Ji,Jj,Jk)
            ENDDO

            wsol=wsol/Nlayer/0.07d0

            cwc=0.0d0
            IF (hum.GT.0.8d0) cwc=0.2d0
            IF (lhflux.GT.0.d0) cwc=0.7d0
            IF (rain.GT.0.d0) cwc=0.9d0

C------------------------------------------------------------------------
C     5 - Computing aerodynamic resistance and quasi-laminar surface
C     boundary layer resistance

            CALL aero_QBL_rst(Iseason,typ,Dmz1,temp,tempsurf,
     $           press,presurf,snow,modwind,DLRa,DLRb)

C------------------------------------------------------------------------
C     7 - Computing canopy resistance

            CALL canopy_rst(Iseason,temp,hum,esat,wsol,
     $           raysol,snow,cwc,DLrstcan)

C------------------------------------------------------------------------
C     8 - Deposition velocity on land type Iland

            DO Jdepot = 1,Ndep  ! Begin cycle on pollutants.

               DO Iland=1,Nland
                  IF (DLtyp(Ji,Jj,Iland).GT.0.D0) THEN

                     DLrstot = DLra(Iland) + DLrb(Iland,Jdepot) 
     $                    + DLrstcan(Iland,Jdepot)
                     DLrstot = 1.d0 / DLrstot
                     DLvdep(Ji,Jj,Jdepot) = DLvdep(Ji,Jj,Jdepot)
     &                    + DLrstot * DLtyp(Ji,Jj,Iland)

                  ENDIF
               ENDDO            !End of cycle on LUC.

            ENDDO               !End of cycle on gaseous pollutants.

         ENDDO
      ENDDO

C     threshold value of 10 cm/s

      DO Jdepot=1,Ndep
         DO Jj=1,NYdd
            DO Ji=1,NXdd
               IF(DLvdep(Ji,Jj,Jdepot).GT.0.1D0)THEN
                  DLvdep(Ji,Jj,Jdepot)=0.1d0
               ENDIF
            ENDDO
         ENDDO
      ENDDO
      
      END
