C------------------------------------------------------------------------
C     
C     Copyright (C) 2001-2003 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE COMP_SCAV(DLtemp,DLpress,DLrain,DLhcloud,DLscavg)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     DLTEMP: temperature. ([K])
C     DLPRESS: pressure. ([Pa])
C     DLRAIN: rain intensity. ([mm/h])
C     DLHCLOUD: height of cloud basis. ([m])
C     MAILLAGE: level height. ([m]) 
C     Dzz: height of cell center. ([m])
C     HHENRY: Henry constant. ([mol/L/atm])
C     DDIF: gas phase diffusivity. ([cm^2/s])
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     -- OUTPUT VARIABLES
C     
C     DLSCAVG: scavenging coefficient. ([1./s]).
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C     Parameterization used in this subroutine is presented and
C     justified in " Numerical and theoretical investigation of a
C     simplified model for the parameterization of below-cloud
C     scavenging by falling raindrops " (B. Sportisse and L. Du Bois,
C     Atmospheric Environment, Vol. 36, pp 5719-5727, 2002 )
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Yelva Roustan, CEREA, February 2003.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

C     -- INCLUDE FILES
C     CONST: constants.
C     PARADOM: parameters for all forced fields.
C     PARACHEM: parameters for sizes of 'chemical' arrays.
C     COMDOM: global variables for the domain.

      INCLUDE 'CONST.INC'
      INCLUDE 'PARADOM.INC'
      INCLUDE 'PARACHEM.INC'
      INCLUDE 'COMDOM.INC'
      INCLUDE 'COMPHYS.INC'

      INCLUDE 'COMCHEM.INC'

      INTEGER i,j,k,Jsca

      DOUBLE PRECISION DLtemp(NXt,NYt,NZt)
      DOUBLE PRECISION DLpress(NXp,NYp,NZp)
      DOUBLE PRECISION DLrain(NXr,NYr)
      DOUBLE PRECISION DLhcloud(NXr,NYr)

      DOUBLE PRECISION DLscavg(NXr,NYr,NZ,Nscav)

      DOUBLE PRECISION DLeta,DLnuair,DLrho
      DOUBLE PRECISION DLurain,DLddrop,DLrey
      
      DOUBLE PRECISION DLSh,DLKtrans

C------------------------------------------------------------------------
C     0. Setup

      DO Jsca=1,Nscav
         DO k=1,NZ
            DO j=1,NYr
               DO i=1,NXr
                  DLscavg(i,j,k,Jsca)=0.D0
               ENDDO
            ENDDO
         ENDDO
      ENDDO

      DO j=1,NYr
         DO i=1,NXr
            IF (DLrain(i,j).GT.0.D0) THEN
               DO k=1,NZ
                  IF (DLhcloud(i,j).GT.Dzz(i,j,k)) THEN

C------------------------------------------------------------------------
C     1. Computation of kinematic viscosity through Sutherland's law.

                     DLeta = 1.83d-5 * ( 416.16d0 / 
     $                    ( Dltemp(i,j,k) + 120.d0) ) *
     $                    ( Dltemp(i,j,k) / 296.16d0 )**1.5d0

C     Mair/R USI = 28.97d-3/8.314 .

                     DLrho = DLpress(i,j,k) * 3.48d-3 / DLtemp(i,j,k)

                     DLnuair = DLeta / DLrho

C------------------------------------------------------------------------
C     2. Computation of rain drop diameter, velocity and Reynolds number.

c     DLddrop in cm
                     DLddrop = 0.1d0 * 0.976d0 * DLrain(i,j)**0.21d0
c     DLurain in m/s
                     DLurain = 9.58d0 * (1.d0 - DEXP(
     &                    (- ( DLddrop / 0.171d0 )**1.147d0 ) ) )

                     DLrey = 1.d-2 * DLurain * DLddrop / DLnuair

C------------------------------------------------------------------------
C     3. Computation of mass transfer coefficient.

                     DO Jsca=1, Nscav

C     Sherwood number.
                        DLSh = 2.d0 + 0.6d0 * DLrey**0.5d0
     &                       * ( 1.d4 * DLnuair / Ddifscav(Jsca) )
     $                       **(1.d0/3.d0)

C     Transfert coefficient ([cm/s]).
                        DLKtrans = ( Ddifscav(Jsca) / DLddrop ) * DLSh

C------------------------------------------------------------------------
C     4. Computation of wet scavenging coefficient.

C     1.67d-6 = 6 * 1.d-6 / 3.6d0.
                        DLscavg(i,j,k,Jsca) = 1.67d-6 * DLrain(i,j) / 
     $                       ( DLurain * DLddrop / DLKtrans )
     $                       * DEXP ( - (DLhcloud(i,j)-Dzz(i,j,k)) /
     $                       ( DLddrop / 6.d0 / DLKtrans * DLurain
     $                       * Hhenryscav(Jsca) * Pr * DLtemp(i,j,k)) ) 

                     ENDDO
                  ENDIF
               ENDDO
            ENDIF
         ENDDO
      ENDDO

      END
