C------------------------------------------------------------------------
C     
C     Copyright (C) 2001-2003 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE BASCULE(Nmeteo,Nmeteof,NKz,NKzf,
     s     DGUi,DGVi,DGWi,DGKz,DGattenuation,DGhumid,DGtemp,DGrain,
     s     DGhcloud,DGpress,DGtempsurf,DGpresurf,DGsnow,DGraysoli,
     $     DGlhflux,DGswat,DGmodwind,
     s     DGUf,DGVf,DGWf,DGKzf,DGattenuationf,DGhumidf,DGtempf,DGrainf,
     s     DGhcloudf,DGpressf,DGtempsurff,DGpresurff,DGsnowf,DGraysolif,
     $     DGlhfluxf,DGswatf,DGmodwindf)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     Let F be a field. Let Fi and Ff be the values of F at time Ti
C     and Tf respectively. Depending on the current time T, this routine
C     sets Fi to Ff (if T > Tf) and reads the new field Ff, or does
C     nothing (if T <= Tf).
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     NMETEO: iteration at which initial fields (at time Ti) are
C     # provided.
C     NMETEOF: iteration at which final fields (at time Tf) are provided.
C     NKZ: iteration at which initial vertical-diffusion (at time Ti) is
C     # provided.
C     NMETEOF: iteration at which final vertical-diffusion (at time Tf)
C     # is provided.
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     DGUI, DGVI, DGWI: 3D wind fields at time Ti.
C     DGKZ: 3D K_z field at time Ti.
C     DGATTENUATION: 3D cloud attenuation field at time Ti.
C     DGHUMID: 3D relative humidity field at time Ti.
C     DGTEMP: 3D temperature field at time Ti.
C     DGRAIN: 2D rain field at time Ti.
C     DGHCLOUD: 2D height of cloud basis at time Ti.
C     DGPRESS: 3D pressure field at time Ti.
C     DGTEMPSURF: 2D surface temperature field at initial time.
C     DGPRESURF: 2D surface pressure field at initial time.
C     DGSNOW: 2D snow height field at initial time.
C     DGRAYSOLI: 2D surface solar irradiance field at initial time.
C     DGLHFLUX: 2D latent heat flux field at initial time.
C     DGSWAT: 3D soil water content field at initial time (3 level).
C     DGMODWIND: 2D wind module in first level.
C     
C     The same variables are defined at final time of the timestep.
C     'f' is then put at the end of the name.
C     
C     -- OUTPUT VARIABLES
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Jaouad Boutahar, CEREA, June 2001.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

C     -- INCLUDE FILES
C     PARADOM: parameters for all forced fields.
C     COMUSE: flags and parameters for numerics.

      INCLUDE 'PARADOM.INC'
      INCLUDE 'COMUSE.INC'

      REAL DGUi(NX+1,NY,NZ),DGUf(NX+1,NY,NZ)
      REAL DGVi(NX,NY+1,NZ),DGVf(NX,NY+1,NZ)
      REAL DGWi(NX,NY,NZ+1),DGWf(NX,NY,NZ+1)
      REAL DGKz(NX,NY,NZ+1),DGKzf(NX,NY,NZ+1)
      REAL DGattenuation(NXatt,NYatt,NZatt)
      REAL DGattenuationf(NXatt,NYatt,NZatt)
      REAL DGhumid(NXh,NYh,NZh),DGhumidf(NXh,NYh,NZh)
      REAL DGtemp(NXt,NYt,NZt),DGtempf(NXt,NYt,NZt)
      REAL DGrain(NXr,NYr),DGrainf(NXr,NYr)
      REAL DGhcloud(NXr,NYr),DGhcloudf(NXr,NYr)
      REAL DGpress(NXp,NYp,NZp),DGpressf(NXp,NYp,NZp)
      REAL DGtempsurf(NXdd,NYdd),DGtempsurff(NXdd,NYdd)
      REAL DGpresurf(NXdd,NYdd),DGpresurff(NXdd,NYdd)
      REAL DGsnow(NXdd,NYdd),DGsnowf(NXdd,NYdd)
      REAL DGraysoli(NXdd,NYdd),DGraysolif(NXdd,NYdd)
      REAL DGlhflux(NXdd,NYdd),DGlhfluxf(NXdd,NYdd)
      REAL DGswat(NXdd,NYdd,Nlayer),DGswatf(NXdd,NYdd,Nlayer)
      REAL DGmodwind(NXdd,NYdd),DGmodwindf(NXdd,NYdd)

      INTEGER Nmeteo,Nmeteof,NKz,Nkzf
      INTEGER Ji,Jj,Jk


C     If T > Tf.
      IF (iabs(Nmeteo-Nmeteof).NE.1) THEN

         IF (LADVEC) THEN

            DO Jk=1,Nz
               DO Jj=1,Ny
                  DO Ji=1,Nx+1
                     DGUi(Ji,Jj,Jk)=DGUf(Ji,Jj,Jk)
                  ENDDO
               ENDDO
            ENDDO

            DO Jk=1,Nz
               DO Jj=1,Ny+1
                  DO Ji=1,Nx
                     DGVi(Ji,Jj,Jk)=DGVf(Ji,Jj,Jk)
                  ENDDO
               ENDDO
            ENDDO

            IF (LREAD_VERTICAL_WIND) THEN
               DO Jk=1,Nz+1
                  DO Jj=1,Ny
                     DO Ji=1,Nx
                        DGWi(Ji,Jj,Jk)=DGWf(Ji,Jj,Jk)
                     ENDDO
                  ENDDO
               ENDDO
            ENDIF

         ENDIF

         IF (LCHIM) THEN

            DO Jk=1,Nzatt
               DO Jj=1,Nyatt
                  DO Ji=1,Nxatt
                     DGattenuation(Ji,Jj,Jk)=DGattenuationf(Ji,Jj,Jk)
                  ENDDO
               ENDDO
            ENDDO

            DO Jj=1,Nyr
               DO Ji=1,Nxr
                  DGrain(Ji,Jj)=DGrainf(Ji,Jj)
               ENDDO
            ENDDO

         ENDIF

         IF (LCHIM.OR.LVDEPOCOMP) THEN

            DO Jk=1,Nzt
               DO Jj=1,Nyt
                  DO Ji=1,Nxt
                     DGtemp(Ji,Jj,Jk)=DGtempf(Ji,Jj,Jk)
                  ENDDO
               ENDDO
            ENDDO

            DO Jk=1,Nzp
               DO Jj=1,Nyp
                  DO Ji=1,Nxp
                     DGpress(Ji,Jj,Jk)=DGpressf(Ji,Jj,Jk)
                  ENDDO
               ENDDO
            ENDDO

            DO Jk=1,Nzh
               DO Jj=1,Nyh
                  DO Ji=1,Nxh
                     DGhumid(Ji,Jj,Jk)=DGhumidf(Ji,Jj,Jk)
                  ENDDO
               ENDDO
            ENDDO

         ENDIF

         IF (LCLOUDU) THEN

            DO Jj=1,Nyr
               DO Ji=1,Nxr
                  DGhcloud(Ji,Jj)=DGhcloudf(Ji,Jj)
               ENDDO
            ENDDO

         ENDIF

         IF (LVDEPOCOMP) THEN

            DO Jj=1,Nydd
               DO Ji=1,Nxdd
                  DGtempsurf(Ji,Jj)=DGtempsurff(Ji,Jj)
                  DGpresurf(Ji,Jj)=DGpresurff(Ji,Jj)
                  DGsnow(Ji,Jj)=DGsnowf(Ji,Jj)
                  DGraysoli(Ji,Jj)=DGraysolif(Ji,Jj)
                  DGlhflux(Ji,Jj)=DGlhfluxf(Ji,Jj)
                  DGraysoli(Ji,Jj)=DGraysolif(Ji,Jj)
                  DGlhflux(Ji,Jj)=DGlhfluxf(Ji,Jj)
                  DGmodwind(Ji,Jj)=DGmodwindf(Ji,Jj)
                  DO Jk=1,Nlayer
                     DGswat(Ji,Jj,Jk)=DGswatf(Ji,Jj,Jk)
                  ENDDO
               ENDDO
            ENDDO


         ENDIF

C     Read new fields.
         CALL READMETEO(Nmeteof,
     s        DGUf,DGVf,DGWf,DGattenuationf,DGhumidf,DGtempf,
     s        DGrainf,DGhcloudf,DGpressf,DGtempsurff,DGpresurff,DGsnowf,
     $        DGraysolif,DGlhfluxf,DGswatf,DGmodwindf)
      ENDIF

C     If T > Tf.
      IF (LDIFF.AND.(iabs(NKz-NKzf).NE.1)) THEN
         
         DO Jk=1,Nz+1
            DO Jj=1,Ny
               DO Ji=1,Nx
                  DGKz(Ji,Jj,Jk)=DGKzf(Ji,Jj,Jk)
               ENDDO
            ENDDO
         ENDDO

         CALL READKZ(NKzf,DGKzf)
         
      ENDIF

      END
