C------------------------------------------------------------------------
C     
C     Copyright (C) 2001-2003 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE INITMETEO(
     s     Nmeteo,Nmeteof,ALFAmeteo,ALFAmeteof,
     s     NKz,NKzf,alpha_Kz,alpha_Kz_f,
     s     Ui,Vi,Wi,Kz,rain,hcloud,attenuation,humid,temp,press,
     $     tempsurf,presurf,snow,raysoli,lhflux,swat,modwind,
     s     Uf,Vf,Wf,Kzf,rainf,hcloudf,attenuationf,humidf,tempf,pressf,
     $     tempsurff,presurff,snowf,raysolif,lhfluxf,swatf,modwindf,
     s     DGUi,DGVi,DGWi,DGKz,DGattenuation,DGhumid,DGtemp,DGrain,
     $     DGhcloud,DGpress,DGtempsurf,DGpresurf,DGsnow,DGraysoli,
     s     DGlhflux,DGswat,DGmodwind,
     s     DGUf,DGVf,DGWf,DGKzf,DGattenuationf,DGhumidf,
     s     DGtempf,DGrainf,DGhcloudf,DGpressf,DGtempsurff,DGpresurff,
     $     DGsnowf,DGraysolif,DGlhfluxf,DGswatf,DGmodwindf)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This routine reads meteorological data for one timestep.
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     NMETEO, NMETEOF: interpolation is performed between iterations
C     # NMETEO and NMETEOF.
C     ALFAMETEO, ALFAMETEOF: interpolation coefficients.
C     NKZ, NKZF: interpolation is performed between iterations
C     # NKZ and NKZF (for vertical diffusion).
C     ALPHA_KZ, ALPHA_KZ_F: interpolation coefficients (for vertical
C     # diffusion).
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     -- OUTPUT VARIABLES
C     
C     UI, VI, WI: 3D wind fields at initial time along X, Y, Z.
C     KZ: 3D Kz field at initial time.
C     RAIN: 2D rain intensity field at initial time.
C     HCLOUD: 2D height of cloud basis at initial time.
C     ATTENUATION: 3D cloud attenuation field at initial time.
C     HUMID: 3D specific humidity field at initial time.
C     TEMP: 3D temperature field at initial time.
C     PRESS: 3D pressure field at initial time.
C     TEMPSURF: 2D surface temperature field at initial time.
C     PRESURF: 2D surface pressure field at initial time.
C     SNOW: 2D snow height field at initial time.
C     RAYSOLI: 2D surface solar irradiance field at initial time.
C     LHFLUX: 2D latent heat flux field at initial time.
C     SWAT: 3D soil water content field at initial time (3 level).
C     
C     The same variables are defined at final time of the timestep.
C     'f' is then put at the end of the name.
C     
C     In the same way, variables are defined with respect to the
C     timestep of input data.
C     
C     DGUI, DGVI, DGWI: 3D wind fields at initial time along X, Y, Z.
C     DGKZ: 3D Kz field at initial time.
C     DGRAIN: 2D rain intensity field at initial time.
C     DGHCLOUD: 2D height of cloud basis at initial time.
C     DGATTENUATION: 3D cloud attenuation field at initial time.
C     DGHUMID: 3D specific humidity field at initial time.
C     DGTEMP: 3D temperature field at initial time.
C     DGPRESS: 3D pressure field at initial time.
C     DGTEMPSURF: 2D surface temperature field at initial time.
C     DGPRESURF: 2D surface pressure field at initial time.
C     DGSNOW: 2D snow height field at initial time.
C     DGRAYSOLI: 2D surface solar irradiance field at initial time.
C     DGLHFLUX: 2D latent heat flux field at initial time.
C     DGSWAT: 3D soil water content field at initial time (3 level).
C     DGMODWIND: 2D wind module in first level.
C     
C     The same variables are defined at final time of the timestep.
C     'f' is then put at the end of the name.
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Jaouad Boutahar, CEREA, June 2001.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

C     -- INCLUDE FILES
C     PARADOM: parameters for all forced fields.
C     PARACHEM: parameters for sizes of 'chemical' arrays.
C     COMDATA: data on forced fields.

      INCLUDE 'PARADOM.INC'
      INCLUDE 'PARACHEM.INC'
      INCLUDE 'COMDATA.INC'

      DOUBLE PRECISION Ui(NX+1,NY,NZ),Uf(NX+1,NY,NZ)
      DOUBLE PRECISION Vi(NX,NY+1,NZ),Vf(NX,NY+1,NZ)
      DOUBLE PRECISION Wi(NX,NY,NZ+1),Wf(NX,NY,NZ+1)
      DOUBLE PRECISION Kz(NX,NY,NZ+1),Kzf(NX,NY,NZ+1)
      DOUBLE PRECISION attenuation(NXatt,NYatt,NZatt)
      DOUBLE PRECISION attenuationf(NXatt,NYatt,NZatt)
      DOUBLE PRECISION humid(NXh,NYh,NZh),humidf(NXh,NYh,NZh)
      DOUBLE PRECISION rain(NXr,NYr),rainf(NXr,NYr)
      DOUBLE PRECISION hcloud(NXr,NYr),hcloudf(NXr,NYr)
      DOUBLE PRECISION temp(NXt,NYt,NZt),tempf(NXt,NYt,NZt)
      DOUBLE PRECISION press(NXp,NYp,NZp),pressf(NXp,NYp,NZp)
      DOUBLE PRECISION tempsurf(NXdd,NYdd),tempsurff(NXdd,NYdd)
      DOUBLE PRECISION presurf(NXdd,NYdd),presurff(NXdd,NYdd)
      DOUBLE PRECISION snow(NXdd,NYdd),snowf(NXdd,NYdd)
      DOUBLE PRECISION raysoli(NXdd,NYdd),raysolif(NXdd,NYdd)
      DOUBLE PRECISION lhflux(NXdd,NYdd),lhfluxf(NXdd,NYdd)
      DOUBLE PRECISION swat(NXdd,NYdd,Nlayer),swatf(NXdd,NYdd,Nlayer)
      DOUBLE PRECISION modwind(NXdd,NYdd),modwindf(NXdd,NYdd)

      REAL DGUi(NX+1,NY,NZ),DGUf(NX+1,NY,NZ)
      REAL DGVi(NX,NY+1,NZ),DGVf(NX,NY+1,NZ)
      REAL DGWi(NX,NY,NZ+1),DGWf(NX,NY,NZ+1)
      REAL DGKz(NX,NY,NZ+1),DGKzf(NX,NY,NZ+1)
      REAL DGattenuation(NXatt,NYatt,NZatt)
      REAL DGattenuationf(NXatt,NYatt,NZatt)
      REAL DGhumid(NXh,NYh,NZh),DGhumidf(NXh,NYh,NZh)
      REAL DGtemp(NXt,NYt,NZt),DGtempf(NXt,NYt,NZt)
      REAL DGpress(NXp,NYp,NZp),DGpressf(NXp,NYp,NZp)
      REAL DGrain(NXr,NYr),DGrainf(NXr,NYr)
      REAL DGhcloud(NXr,NYr),DGhcloudf(NXr,NYr)
      REAL DGtempsurf(NXdd,NYdd),DGtempsurff(NXdd,NYdd)
      REAL DGpresurf(NXdd,NYdd),DGpresurff(NXdd,NYdd)
      REAL DGsnow(NXdd,NYdd),DGsnowf(NXdd,NYdd)
      REAL DGraysoli(NXdd,NYdd),DGraysolif(NXdd,NYdd)
      REAL DGlhflux(NXdd,NYdd),DGlhfluxf(NXdd,NYdd)
      REAL DGswat(NXdd,NYdd,Nlayer),DGswatf(NXdd,NYdd,Nlayer)
      REAL DGmodwind(NXdd,NYdd),DGmodwindf(NXdd,NYdd)

      INTEGER Nmeteo,Nmeteof,NKz,NKzf,Nlu,NluKz
      DOUBLE PRECISION ALFAmeteo,ALFAmeteof,alpha_Kz,alpha_Kz_f

      IF (LITER) THEN           ! First iteration.
         
         CALL READMETEO(Nmeteo,
     s        DGUi,DGVi,DGWi,DGattenuation,DGhumid,DGtemp,DGrain,
     s        DGhcloud,DGpress,DGtempsurf,DGpresurf,DGsnow,DGraysoli,
     $        DGlhflux,DGswat,DGmodwind)
         CALL READKZ(NKz,DGKz)

         IF (iabs(Nmeteo-Nmeteof).NE.1) THEN
            Nlu=(Nmeteo+Nmeteof)/2
         ELSE
            Nlu=Nmeteof
         ENDIF

         IF (iabs(NKz-NKzf).NE.1) THEN
            NluKz=(NKz+NKzf)/2
         ELSE
            NluKz=NKzf
         ENDIF

         CALL READMETEO(Nlu,
     s        DGUf,DGVf,DGWf,DGattenuationf,DGhumidf,DGtempf,
     s        DGrainf,DGhcloudf,DGpressf,DGtempsurff,DGpresurff,DGsnowf,
     $        DGraysolif,DGlhfluxf,DGswatf,DGmodwindf)
         CALL READKZ(NluKz,DGKzf)

         CALL INTERPOL(ALFAmeteo,alpha_Kz,
     s        Ui,Vi,Wi,Kz,rain,hcloud,attenuation,humid,temp,press,
     $        tempsurf,presurf,snow,raysoli,lhflux,swat,modwind,
     s        DGUi,DGVi,DGWi,DGKz,DGattenuation,DGhumid,DGtemp,DGrain,
     s        DGhcloud,DGpress,DGtempsurf,DGpresurf,DGsnow,DGraysoli,
     $        DGlhflux,DGswat,DGmodwind,
     s        DGUf,DGVf,DGWf,DGKzf,DGattenuationf,DGhumidf,DGtempf,
     s        DGrainf,DGhcloudf,DGpressf,DGtempsurff,DGpresurff,
     $        DGsnowf,DGraysolif,DGlhfluxf,DGswatf,DGmodwindf)

      ENDIF                     ! End of first iteration test.

      CALL BASCULE(Nmeteo,Nmeteof,NKz,NKzf,
     s     DGUi,DGVi,DGWi,DGKz,DGattenuation,DGhumid,DGtemp,DGrain,
     s     DGhcloud,DGpress,DGtempsurf,DGpresurf,DGsnow,DGraysoli,
     $     DGlhflux,DGswat,DGmodwind,
     s     DGUf,DGVf,DGWf,DGKzf,DGattenuationf,DGhumidf,DGtempf,DGrainf,
     s     DGhcloudf,DGpressf,DGtempsurff,DGpresurff,DGsnowf,DGraysolif,
     $     DGlhfluxf,DGswatf,DGmodwindf)

      IF (.NOT.LITER) THEN
         CALL SWITCH(Ui,Vi,Wi,Kz,rain,hcloud,attenuation,humid,temp,
     $        press,tempsurf,presurf,snow,raysoli,lhflux,swat,modwind,
     s        Uf,Vf,Wf,Kzf,rainf,hcloudf,attenuationf,humidf,tempf,
     $        pressf,tempsurff,presurff,snowf,raysolif,lhfluxf,swatf,
     s        modwindf)
      ENDIF

      CALL INTERPOL(ALFAmeteof,alpha_Kz_f,
     s     Uf,Vf,Wf,Kzf,rainf,hcloudf,attenuationf,humidf,tempf,pressf,
     $     tempsurff,presurff,snowf,raysolif,lhfluxf,swatf,modwindf,
     s     DGUi,DGVi,DGWi,DGKz,DGattenuation,DGhumid,DGtemp,DGrain,
     s     DGhcloud,DGpress,DGtempsurf,DGpresurf,DGsnow,DGraysoli,
     $     DGlhflux,DGswat,DGmodwind,
     s     DGUf,DGVf,DGWf,DGKzf,DGattenuationf,DGhumidf,DGtempf,DGrainf,
     s     DGhcloudf,DGpressf,DGtempsurff,DGpresurff,DGsnowf,DGraysolif,
     $     DGlhfluxf,DGswatf,DGmodwindf)

      END
