C------------------------------------------------------------------------
C     
C     Copyright (C) 2001-2003 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE INITPHOTOLYSIS(ts, tf, Cphotolysis_rates,
     $     Cphotolysis_rates_f)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This routine reads photolysis rates.
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     TS, TF: initial and final time of the time step ([seconds from
C     the 1st January of the current year]), UTC.
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     -- OUTPUT VARIABLES
C     
C     CPHOTOLYSIS_RATES, CPHOTOLYSIS_RATES_F: photolysis rates.
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Vivien Mallet, CEREA, August 2003.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

C     -- INCLUDE FILES
C     PARADOM: parameters for all forced fields.
C     PARACHEM: parameters for sizes of 'chemical' arrays.
C     COMDATA: data on forced fields.
C     COMDOM: global variables for the domain.
C     COMFILES: global variables for files names.

      INCLUDE 'PARADOM.INC'
      INCLUDE 'PARACHEM.INC'
      INCLUDE 'COMDATA.INC'
      INCLUDE 'COMDOM.INC'
      INCLUDE 'COMFILES.INC'

      INTEGER NUC,Ji,Jj,Jk,Jr,j,k,angle_index_i,angle_index_f

      DOUBLE PRECISION Ts, Tf

      INTEGER day_index_i, day_index_f, day_i, day_f, nb_days
      
      REAL rates_i(Nzphot, Nlatphot, Nanglephot)
      REAL rates_f(Nzphot, Nlatphot, Nanglephot)

      DOUBLE PRECISION alpha_z, alpha_y
      DOUBLE PRECISION one_alpha_z, one_alpha_y

      DOUBLE PRECISION angle_alpha_i, angle_alpha_f
      DOUBLE PRECISION one_angle_alpha_i, one_angle_alpha_f

      DOUBLE PRECISION time_angle_i, time_angle_f

      DOUBLE PRECISION Cphotolysis_rates(NRphot, Nx, Ny, Nz)
      DOUBLE PRECISION Cphotolysis_rates_f(NRphot, Nx, Ny, Nz)


      NUC = 4

C------------------------------------------------------------------------
C     1 - Time

C     Date.
      day_i = INT(ts / 3600.d0 / 24.d0)
      day_f = INT(tf / 3600.d0 / 24.d0)
C     Dates with which interpolation is performed.
      day_index_i = INT((day_i - Dttimephot0) / Dttimephot + 0.5d0) + 1
      day_index_f = INT((day_f - Dttimephot0) / Dttimephot + 0.5d0) + 1

C     Checks whether data is available.
      IF (day_index_f.GE.NECHTIMEPHOT) THEN
         write(*,*) 'Overstack for photolysis data!'
         write(*,*) 'day_index_f > NECHTIMEPHOT'
         write(*,*) day_index_f,' > ', NECHTIMEPHOT
         stop
      ENDIF

C------------------------------------------------------------------------
C     2 - Loop over photolysis reactions

C     For all photolysis reactions.
      DO Jr=1, Nrphot

C     2.1 - Reads data

         OPEN(NUC, FILE = file_photolysis_rates(Jr), access = 'direct',
     $        form = 'unformatted', recl = 4*Nanglephot*Nlatphot*Nzphot)

         READ(NUC, rec = day_index_i)
     s        (((rates_i(Ji, Jj, Jk), Ji=1,Nzphot),
     $        Jj=1,Nlatphot), Jk=1,Nanglephot)

         READ(NUC, rec = day_index_f)
     s        (((rates_f(Ji, Jj, Jk), Ji=1,Nzphot),
     $        Jj=1,Nlatphot), Jk=1,Nanglephot)

         CLOSE (NUC)

C     2.2 - Interpolation

C     Loop over z/altitude.
         DO Jk=1,Nz
            
C     Loop over y/latitude.
            DO Jj=1,Ny

C     Loop over x/longitude.
               DO Ji=1,Nx

C     Along z: index and interpolation coefficient.
                  k = 1
                  DO WHILE ( (k.lt.Nzphot)
     $                 .and. (Dzphot(k).lt.Dzz(Ji, Jj, Jk)) )
                     k = k + 1
                  ENDDO
                  IF (k>1) k = k - 1 ! Current altitude Dzz(Jk) is then
                                ! between Dzphot(k) and Dzphot(k+1);
                                ! except if Dzphot(Nzphot) < Dzz(Jk)
                                ! or if Dzphot(1) > Dzz(Jk).
                  alpha_z = ( Dzz(Ji, Jj, Jk) - Dzphot(k) )
     $                 / ( Dzphot(k+1) - Dzphot(k) )

C     Along y: index and interpolation coefficient.
                  j = INT((Dlat(Ji, Jj, Jk) - Dtlatphot0) / Dtlatphot)
     $                 + 1
                  alpha_y = ( Dlat(Ji, Jj, Jk) - Dtlatphot0
     $                 - (j-1) * Dtlatphot )
     $                 / Dtlatphot

C     Time angles.
                  time_angle_i = ts / 3600.d0 - 12.d0
     $                 + Dlon(Ji, Jj, Jk) / 15.d0
                  nb_days = int(time_angle_i/24.d0)
                  time_angle_i = dabs(time_angle_i - 24.d0 * nb_days)
                  if (time_angle_i.gt.12.d0)
     $                 time_angle_i = 24.d0 - time_angle_i

                  time_angle_f = tf / 3600.d0 - 12.d0
     $                 + Dlon(Ji, Jj, Jk) / 15.d0
                  nb_days = int(time_angle_f/24.d0)
                  time_angle_f = dabs(time_angle_f - 24.d0 * nb_days)
                  if (time_angle_f.gt.12.d0)
     $                 time_angle_f = 24.d0 - time_angle_f

C     For time angles: indices and interpolation coefficients.
                  angle_index_i = INT((time_angle_i - Dtanglephot0)
     $                 / Dtanglephot) + 1
                  angle_alpha_i = ( time_angle_i - Dtanglephot0
     $                 - (angle_index_i-1) * Dtanglephot )
     $                 / Dtanglephot

                  angle_index_f = INT((time_angle_f - Dtanglephot0)
     $                 / Dtanglephot) + 1
                  angle_alpha_f = ( time_angle_f - Dtanglephot0
     $                 - (angle_index_f-1) * Dtanglephot )
     $                 / Dtanglephot


C     Other weighting factors.
                  one_angle_alpha_i = 1.d0 - angle_alpha_i
                  one_angle_alpha_f = 1.d0 - angle_alpha_f
                  one_alpha_y = 1.d0 - alpha_y
                  one_alpha_z = 1.d0 - alpha_z

C     Interpolation.
                  IF (angle_index_i.ge.Nanglephot) THEN
                     Cphotolysis_rates(Jr, Ji, Jj, Jk) = 0.d0
                  ELSE
                     Cphotolysis_rates(Jr, Ji, Jj, Jk) =
     $                    one_alpha_z * one_alpha_y * one_angle_alpha_i
     $                    * DBLE(rates_i(k, j, angle_index_i))
     $                    +
     $                    alpha_z * one_alpha_y * one_angle_alpha_i
     $                    * DBLE(rates_i(k+1, j, angle_index_i))
     $                    +
     $                    one_alpha_z * alpha_y * one_angle_alpha_i
     $                    * DBLE(rates_i(k, j+1, angle_index_i))
     $                    +
     $                    alpha_z * alpha_y * one_angle_alpha_i
     $                    * DBLE(rates_i(k+1, j+1, angle_index_i))
     $                    +
     $                    one_alpha_z * one_alpha_y * angle_alpha_i
     $                    * DBLE(rates_i(k, j, angle_index_i+1))
     $                    +
     $                    alpha_z * one_alpha_y * angle_alpha_i
     $                    * DBLE(rates_i(k+1, j, angle_index_i+1))
     $                    +
     $                    one_alpha_z * alpha_y * angle_alpha_i
     $                    * DBLE(rates_i(k, j+1, angle_index_i+1))
     $                    +
     $                    alpha_z * alpha_y * angle_alpha_i
     $                    * DBLE(rates_i(k+1, j+1, angle_index_i+1))
                  ENDIF

                  IF (angle_index_f.ge.Nanglephot) THEN
                     Cphotolysis_rates_f(Jr, Ji, Jj, Jk) = 0.d0
                  ELSE
                     Cphotolysis_rates_f(Jr, Ji, Jj, Jk) =
     $                    one_alpha_z * one_alpha_y * one_angle_alpha_f
     $                    * DBLE(rates_f(k, j, angle_index_f))
     $                    +
     $                    alpha_z * one_alpha_y * one_angle_alpha_f
     $                    * DBLE(rates_f(k+1, j, angle_index_f))
     $                    +
     $                    one_alpha_z * alpha_y * one_angle_alpha_f
     $                    * DBLE(rates_f(k, j+1, angle_index_f))
     $                    +
     $                    alpha_z * alpha_y * one_angle_alpha_f
     $                    * DBLE(rates_f(k+1, j+1, angle_index_f))
     $                    +
     $                    one_alpha_z * one_alpha_y * angle_alpha_f
     $                    * DBLE(rates_f(k, j, angle_index_f+1))
     $                    +
     $                    alpha_z * one_alpha_y * angle_alpha_f
     $                    * DBLE(rates_f(k+1, j, angle_index_f+1))
     $                    +
     $                    one_alpha_z * alpha_y * angle_alpha_f
     $                    * DBLE(rates_f(k, j+1, angle_index_f+1))
     $                    +
     $                    alpha_z * alpha_y * angle_alpha_f
     $                    * DBLE(rates_f(k+1, j+1, angle_index_f+1))
                  ENDIF

               ENDDO            ! Loop over x/longitude.

            ENDDO               ! Loop over y/latitude.

         ENDDO                  ! Loop over z/altitude.
         
      ENDDO                     ! Loop over photolysis reactions.

      END
