C------------------------------------------------------------------------
C     
C     Copyright (C) 2001-2003 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE INITVDEPO(ts,tf,Nvdep,Nvdepf,ALFAvdep,ALFAvdepf,
     s     modwind,modwindf,rain,rainf,humid,humidf,temp,tempf,
     $     tempsurf,tempsurff,press,pressf,presurf,presurff,
     $     snow,snowf,raysoli,raysolif,lhflux,lhfluxf,swat,swatf,
     s     Vdep,Vdepf,DGVdep,DGVdepf)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This routine reads or computes deposition velocities.
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     NVDEP, NVDEPF: interpolation is performed between iterations NVDEP
C     # and NVDEPF.
C     ALFAVDEP, ALFAVDEPF: interpolation coefficients.
C     MODWIND: wind module.
C     RAIN: 2D rain intensity field at initial time.
C     HUMID: 3D specific humidity field at initial time.
C     TEMP: 3D temperature field at initial time.
C     TEMPSURF: 2D surface temperature field at initial time.
C     PRESS:
C     PRESURF: 2D surface pressure field at initial time.
C     SNOW: 2D snow height field at initial time.
C     RAYSOLI: 2D surface solar irradiance field at initial time.
C     LHFLUX: 2D latent heat flux field at initial time.
C     SWAT: 3D soil water content field at initial time (3 level).
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     -- OUTPUT VARIABLES
C     
C     VDEP, VDEPF: interpolated deposition-velocities.
C     DGVDEP, DGVDEPF: deposition velocities from input file.
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Jaouad Boutahar, CEREA, June 2001.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

C     -- INCLUDE FILES
C     PARADOM: parameters for all forced fields.
C     PARACHEM: parameters for sizes of 'chemical' arrays.
C     COMDATA: data on forced fields.
C     COMLUN: global variables for unit numbers.
C     COMFILES: global variables for files names.

      INCLUDE 'PARADOM.INC'
      INCLUDE 'PARACHEM.INC'
      INCLUDE 'COMDATA.INC'
      INCLUDE 'COMLUN.INC'
      INCLUDE 'COMFILES.INC'
      INCLUDE 'COMUSE.INC'

      DOUBLE PRECISION modwind(Nxdd,Nydd),modwindf(Nxdd,Nydd)
      DOUBLE PRECISION tempsurf(NXdd,NYdd),tempsurff(NXdd,NYdd)
      DOUBLE PRECISION presurf(NXdd,NYdd),presurff(NXdd,NYdd)
      DOUBLE PRECISION snow(NXdd,NYdd),snowf(NXdd,NYdd)
      DOUBLE PRECISION raysoli(NXdd,NYdd),raysolif(NXdd,NYdd)
      DOUBLE PRECISION lhflux(NXdd,NYdd),lhfluxf(NXdd,NYdd)
      DOUBLE PRECISION swat(NXdd,NYdd,Nlayer),swatf(NXdd,NYdd,Nlayer)
      DOUBLE PRECISION rain(NXr,NYr),rainf(NXr,NYr)
      DOUBLE PRECISION humid(NXh,NYh),humidf(NXh,NYh)
      DOUBLE PRECISION temp(NXt,NYt),tempf(NXt,NYt)
      DOUBLE PRECISION press(NXt,NYt),pressf(NXt,NYt)
      DOUBLE PRECISION ts,tf

      DOUBLE PRECISION Vdep(Nxdd,Nydd,Ndep)
      DOUBLE PRECISION Vdepf(Nxdd,Nydd,Ndep)

      REAL DGVdep(NXdd,NYdd,Ndep)
      REAL DGVdepf(NXdd,NYdd,Ndep)

      INTEGER Nvdep,Nvdepf
      DOUBLE PRECISION ALFAvdep,ALFAvdepf

      INTEGER Jsp,Ji,Jj

C-----------------------------------------------------------------------
C     1 - Compute Vdep

      IF (LITER) THEN           ! First iteration.

         IF (LVDEPOCOMP) THEN

C     Compute dry deposition velocity.
            CALL VDEPOT(ts,modwind,humid,temp,tempsurf,press,presurf,
     $           snow,raysoli,lhflux,swat,rain,Vdep)

         ELSE

C     Read dry deposition velocity.
            DO Jsp=1,Ndep

               OPEN(unit=NULVDEP,file=file_vdep(Jsp),
     s              access='direct',form='unformatted',recl=4*Nxdd*Nydd)

               READ(NULVDEP,rec=Nvdep)
     s              ((DGVdep(Ji,Jj,Jsp),Ji=1,Nxdd),Jj=1,Nydd)

               IF (iabs(Nvdep-Nvdepf).NE.1) THEN
                  READ(NULVDEP,rec=(Nvdep+Nvdepf)/2)
     s                 ((DGVdepf(Ji,Jj,Jsp),Ji=1,Nxdd),Jj=1,Nydd)
               ELSE
                  READ(NULVDEP,rec=Nvdepf)
     s                 ((DGVdepf(Ji,Jj,Jsp),Ji=1,Nxdd),Jj=1,Nydd)
               ENDIF

               CLOSE (NULVDEP)
            ENDDO

C     Interpolation for Vdep.
            DO Jsp=1,Ndep
               DO Jj=1,Nydd
                  DO Ji=1,Nxdd
                     Vdep(Ji,Jj,Jsp)=ALFAvdep*DBLE(DGVdep(Ji,Jj,Jsp))+
     s                    (1.D0-ALFAvdep)*DBLE(DGVdepf(Ji,Jj,Jsp))
                  ENDDO
               ENDDO
            ENDDO

         ENDIF                  ! End of compute or read.

      ENDIF                     ! End of first iteration test.

      IF ((.NOT.LVDEPOCOMP).AND.(iabs(Nvdep-Nvdepf).NE.1)) THEN

         DO Jsp=1,Ndep
            DO Jj=1,Nydd
               DO Ji=1,Nxdd
                  DGVdep(Ji,Jj,Jsp)=DGVdepf(Ji,Jj,Jsp)
               ENDDO
            ENDDO
         ENDDO

C     Read dry deposition velocity.
         DO Jsp=1,Ndep
            OPEN(unit=NULVDEP,file=file_vdep(Jsp),
     s           access='direct',form='unformatted',
     $           recl=4*(Nxdd)*(Nydd))
            
            READ(NULVDEP,rec=Nvdepf)
     s           ((DGVdepf(Ji,Jj,Jsp),Ji=1,Nxdd),Jj=1,Nydd)
            
            CLOSE (NULVDEP)
         ENDDO

      ENDIF

C     In both case (compute or read) operation is the same.
      DO Jsp=1,Ndep
         DO Jj=1,Nydd
            DO Ji=1,Nxdd
               IF (.NOT.LITER) Vdep(Ji,Jj,Jsp)=Vdepf(Ji,Jj,Jsp)
            ENDDO
         ENDDO
      ENDDO

C-----------------------------------------------------------------------
C     2 - Compute Vdepf

      IF (LVDEPOCOMP) THEN

C     Compute dry deposition.
         CALL VDEPOT(tf,modwindf,humidf,tempf,tempsurff,pressf,presurff,
     $        snowf,raysolif,lhfluxf,swatf,rainf,Vdepf)

      ELSE

C     Interpolation for Vdepf when dry deposition velocities are read.
         DO Jsp=1,Ndep
            DO Jj=1,Nydd
               DO Ji=1,Nxdd
                  Vdepf(Ji,Jj,Jsp)=ALFAvdepf*DBLE(DGVdep(Ji,Jj,Jsp))+
     s                 (1.D0-ALFAvdepf)*DBLE(DGVdepf(Ji,Jj,Jsp))
               ENDDO
            ENDDO
         ENDDO
         
      ENDIF

      END
