C------------------------------------------------------------------------
C     
C     Copyright (C) 2001-2003 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE INTERPOL(ALFAmeteo,alpha_Kz,
     s     Ui,Vi,Wi,Kz,rain,hcloud,attenuation,humid,temp,press,
     $     tempsurf,presurf,snow,raysoli,lhflux,swat,modwind,
     s     DGUi,DGVi,DGWi,DGKz,DGattenuation,DGhumid,DGtemp,DGrain,
     s     DGhcloud,DGpress,DGtempsurf,DGpresurf,DGsnow,DGraysoli,
     $     DGlhflux,DGswat,DGmodwind,
     s     DGUf,DGVf,DGWf,DGKzf,DGattenuationf,DGhumidf,DGtempf,DGrainf,
     s     DGhcloudf,DGpressf,DGtempsurff,DGpresurff,DGsnowf,DGraysolif,
     $     DGlhfluxf,DGswatf,DGmodwindf)


C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This routine linearly interpolates data.
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     ALFAMETEO: interpolation coefficient.
C     ALPHA_KZ: interpolation coefficient for vertical diffusion.
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     DGUI, DGVI, DGWI: 3D wind fields at initial time along X, Y, Z.
C     DGKZ: 3D Kz field at initial time.
C     DGRAIN: 2D rain intensity field at initial time.
C     DGHCLOUD: 2D height of cloud basis at initial time.
C     DGATTENUATION: 3D cloud attenuation field at initial time.
C     DGHUMID: 3D relative humidity field at initial time.
C     DGTEMP: 3D temperature field at initial time.
C     DGPRESS: 3D pressure field at initial time.
C     DGTEMPSURF: 2D surface temperature field at initial time.
C     DGPRESURF: 2D surface pressure field at initial time.
C     DGSNOW: 2D snow height field at initial time.
C     DGRAYSOLI: 2D surface solar irradiance field at initial time.
C     DGLHFLUX: 2D latent heat flux field at initial time.
C     DGSWAT: 3D soil water content field at initial time (3 level).
C     
C     The same variables are defined at final time of the timestep.
C     'f' is then put at the end of the name.
C     
C     -- OUTPUT VARIABLES
C     
C     UI, VI, WI: 3D interpolated wind-fields along X, Y, Z.
C     KZ: 3D interpolated Kz-field.
C     RAIN: 2D rain interpolated intensity-field.
C     HCLOUD: 2D height of cloud basis.
C     ATTENUATION: 3D cloud attenuation humidity-field.
C     HUMID: 3D interpolated relative humidity-field.
C     TEMP: 3D interpolated temperature-field.
C     PRESS: 3D interpolated pressure-field.
C     TEMPSURF: 2D surface temperature field at initial time.
C     PRESURF: 2D surface pressure field at initial time.
C     SNOW: 2D snow height field at initial time.
C     RAYSOLI: 2D surface solar irradiance field at initial time.
C     LHFLUX: 2D latent heat flux field at initial time.
C     SWAT: 3D soil water content field at initial time (3 level).
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Jaouad Boutahar, CEREA, June 2001.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

C     -- INCLUDE FILES
C     PARADOM: parameters for all forced fields.
C     COMDOM: global variables for mesh and forcing fields. 
C     COMUSE: flags and parameters for numerics.

      INCLUDE 'PARADOM.INC'
      INCLUDE 'COMDOM.INC'
      INCLUDE 'COMUSE.INC'

      DOUBLE PRECISION Ui(NX+1,NY,NZ),Vi(NX,NY+1,NZ)
      DOUBLE PRECISION Wi(NX,NY,NZ+1),Kz(NX,NY,NZ+1)
      DOUBLE PRECISION attenuation(NXatt,NYatt,NZatt)
      DOUBLE PRECISION humid(NXh,NYh,NZh),temp(NXt,NYt,NZt)
      DOUBLE PRECISION rain(NXr,NYr),press(NXp,NYp,NZp)
      DOUBLE PRECISION hcloud(NXr,NYr),tempsurf(NXdd,NYdd)
      DOUBLE PRECISION presurf(NXdd,NYdd)
      DOUBLE PRECISION snow(NXdd,NYdd),raysoli(NXdd,NYdd)
      DOUBLE PRECISION lhflux(NXdd,NYdd),swat(NXdd,NYdd,Nlayer)
      DOUBLE PRECISION modwind(NXdd,NYdd)

      REAL DGUi(NX+1,NY,NZ),DGUf(NX+1,NY,NZ)
      REAL DGVi(NX,NY+1,NZ),DGVf(NX,NY+1,NZ)
      REAL DGWi(NX,NY,NZ+1),DGWf(NX,NY,NZ+1)
      REAL DGKz(NX,NY,NZ+1),DGKzf(NX,NY,NZ+1)
      REAL DGattenuation(NXatt,NYatt,NZatt)
      REAL DGattenuationf(NXatt,NYatt,NZatt)
      REAL DGhumid(NXh,NYh,NZh),DGhumidf(NXh,NYh,NZh)
      REAL DGtemp(NXt,NYt,NZt),DGtempf(NXt,NYt,NZt)
      REAL DGrain(NXr,NYr),DGrainf(NXr,NYr)
      REAL DGhcloud(NXr,NYr),DGhcloudf(NXr,NYr)
      REAL DGpress(NXp,NYp,NZp),DGpressf(NXp,NYp,NZp)
      REAL DGtempsurf(NXdd,NYdd),DGtempsurff(NXdd,NYdd)
      REAL DGpresurf(NXdd,NYdd),DGpresurff(NXdd,NYdd)
      REAL DGsnow(NXdd,NYdd),DGsnowf(NXdd,NYdd)
      REAL DGraysoli(NXdd,NYdd),DGraysolif(NXdd,NYdd)
      REAL DGlhflux(NXdd,NYdd),DGlhfluxf(NXdd,NYdd)
      REAL DGswat(NXdd,NYdd,Nlayer),DGswatf(NXdd,NYdd,Nlayer)
      REAL DGmodwind(NXdd,NYdd),DGmodwindf(NXdd,NYdd)

      DOUBLE PRECISION ALFAmeteo,alpha_Kz
      INTEGER Ji,Jj,Jk


      IF (LADVEC) THEN 

         DO Jk=1,Nz
            DO Jj=1,Ny
               DO Ji=1,Nx+1
                  Ui(Ji,Jj,Jk)=ALFAmeteo*DBLE(DGUi(Ji,Jj,Jk))+
     s                 (1.D0-ALFAmeteo)*DBLE(DGUf(Ji,Jj,Jk))
               ENDDO
            ENDDO
         ENDDO

         DO Jk=1,Nz
            DO Jj=1,Ny+1
               DO Ji=1,Nx
                  Vi(Ji,Jj,Jk)=ALFAmeteo*DBLE(DGVi(Ji,Jj,Jk))+
     s                 (1.D0-ALFAmeteo)*DBLE(DGVf(Ji,Jj,Jk))
               ENDDO
            ENDDO
         ENDDO

         IF (LREAD_VERTICAL_WIND) THEN
            DO Jk=1,Nz+1
               DO Jj=1,Ny
                  DO Ji=1,Nx
                     Wi(Ji,Jj,Jk)=ALFAmeteo*DBLE(DGWi(Ji,Jj,Jk))+
     s                    (1.D0-ALFAmeteo)*DBLE(DGWf(Ji,Jj,Jk))
                  ENDDO
               ENDDO
            ENDDO
            
         ELSE                   ! The vertical wind is computed.

C     Wind divergence set to zero.
            
            DO Jj = 1, Ny
               DO Ji = 1, Nx
                  Wi(Ji, Jj, 1) = 0.d0
               ENDDO
            ENDDO
            
            DO Jk = 2, Nz+1
               DO Jj = 1, Ny
                  DO Ji = 1, Nx
                     Wi(Ji, Jj, Jk) = Wi(Ji, Jj, Jk-1) -
     $                    ( (Ui(Ji+1, Jj, Jk-1) - Ui(Ji, Jj, Jk-1))
     $                    / Dmx(Ji) * Dmz(Ji, Jj, Jk-1) +
     $                    (Vi(Ji, Jj+1, Jk-1) - Vi(Ji, Jj, Jk-1))
     $                    / Dmy(Jj) * Dmz(Ji, Jj, Jk-1) )
                  ENDDO
               ENDDO
            ENDDO

         ENDIF

      ENDIF

      IF (LDIFF) THEN 

         DO Jk=1,Nz+1
            DO Jj=1,Ny
               DO Ji=1,Nx
                  Kz(Ji,Jj,Jk)=alpha_Kz*DBLE(DGKz(Ji,Jj,Jk))+
     s                 (1.D0-alpha_Kz)*DBLE(DGKzf(Ji,Jj,Jk))
               ENDDO
            ENDDO
         ENDDO

      ENDIF

      IF (LCHIM) THEN 

         DO Jk=1,Nzatt
            DO Jj=1,Nyatt
               DO Ji=1,Nxatt
                  attenuation(Ji,Jj,Jk)=
     $                 ALFAmeteo*DBLE(DGattenuation(Ji,Jj,Jk))+
     s                 (1.D0-ALFAmeteo)*DBLE(DGattenuationf(Ji,Jj,Jk))
               ENDDO
            ENDDO
         ENDDO

         DO Jj=1,Nyr
            DO Ji=1,Nxr
               rain(Ji,Jj)=ALFAmeteo*DBLE(DGrain(Ji,Jj))+
     s              (1.D0-ALFAmeteo)*DBLE(DGrainf(Ji,Jj))
            ENDDO
         ENDDO

      ENDIF

      IF (LCHIM.OR.LVDEPOCOMP) THEN 

         DO Jk=1,Nzt
            DO Jj=1,Nyt
               DO Ji=1,Nxt
                  temp(Ji,Jj,Jk)=ALFAmeteo*DBLE(DGtemp(Ji,Jj,Jk))+
     s                 (1.D0-ALFAmeteo)*DBLE(DGtempf(Ji,Jj,Jk))
               ENDDO
            ENDDO
         ENDDO

         DO Jk=1,Nzh
            DO Jj=1,Nyh
               DO Ji=1,Nxh
                  humid(Ji,Jj,Jk)=ALFAmeteo*DBLE(DGhumid(Ji,Jj,Jk))+
     s                 (1.D0-ALFAmeteo)*DBLE(DGhumidf(Ji,Jj,Jk))
               ENDDO
            ENDDO
         ENDDO

         DO Jk=1,Nzp
            DO Jj=1,Nyp
               DO Ji=1,Nxp
                  press(Ji,Jj,Jk)=ALFAmeteo*DBLE(DGpress(Ji,Jj,Jk))+
     s                 (1.D0-ALFAmeteo)*DBLE(DGpressf(Ji,Jj,Jk))
               ENDDO
            ENDDO
         ENDDO

      ENDIF

      IF (LCLOUDU) THEN 

         DO Jj=1,Nyr
            DO Ji=1,Nxr
               hcloud(Ji,Jj)=ALFAmeteo*DBLE(DGhcloud(Ji,Jj))+
     s              (1.D0-ALFAmeteo)*DBLE(DGhcloudf(Ji,Jj))
            ENDDO
         ENDDO
      ELSE
         DO Jj=1,Nyr
            DO Ji=1,Nxr
               hcloud(Ji,Jj)=0.d0
            ENDDO
         ENDDO
      ENDIF

      IF (LVDEPOCOMP) THEN 

         DO Jj=1,Nydd
            DO Ji=1,Nxdd
               tempsurf(Ji,Jj)=ALFAmeteo*DBLE(DGtempsurf(Ji,Jj))+
     s              (1.D0-ALFAmeteo)*DBLE(DGtempsurff(Ji,Jj))
            ENDDO
         ENDDO

         DO Jj=1,Nydd
            DO Ji=1,Nxdd
               snow(Ji,Jj)=ALFAmeteo*DBLE(DGsnow(Ji,Jj))+
     s              (1.D0-ALFAmeteo)*DBLE(DGsnowf(Ji,Jj))
            ENDDO
         ENDDO

         DO Jj=1,Nydd
            DO Ji=1,Nxdd
               raysoli(Ji,Jj)=ALFAmeteo*DBLE(DGraysoli(Ji,Jj))+
     s              (1.D0-ALFAmeteo)*DBLE(DGraysolif(Ji,Jj))
            ENDDO
         ENDDO

         DO Jj=1,Nydd
            DO Ji=1,Nxdd
               lhflux(Ji,Jj)=ALFAmeteo*DBLE(DGlhflux(Ji,Jj))+
     s              (1.D0-ALFAmeteo)*DBLE(DGlhfluxf(Ji,Jj))
            ENDDO
         ENDDO

         DO Jj=1,Nydd
            DO Ji=1,Nxdd
               presurf(Ji,Jj)=ALFAmeteo*DBLE(DGpresurf(Ji,Jj))+
     s              (1.D0-ALFAmeteo)*DBLE(DGpresurff(Ji,Jj))
            ENDDO
         ENDDO

         DO Jk=1,Nlayer
            DO Jj=1,Nydd
               DO Ji=1,Nxdd
                  swat(Ji,Jj,Jk)=ALFAmeteo*DBLE(DGswat(Ji,Jj,Jk))+
     s                 (1.D0-ALFAmeteo)*DBLE(DGswatf(Ji,Jj,Jk))
               ENDDO
            ENDDO
         ENDDO

         DO Jj=1,Nydd
            DO Ji=1,Nxdd
               modwind(Ji,Jj)=ALFAmeteo*DBLE(DGmodwind(Ji,Jj))+
     s              (1.D0-ALFAmeteo)*DBLE(DGmodwindf(Ji,Jj))
            ENDDO
         ENDDO

      ENDIF

      END
