C------------------------------------------------------------------------
C     
C     Copyright (C) 2001-2003 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE SWITCH (Ui,Vi,Wi,Kz,rain,hcloud,attenuation,humid,temp,
     $     press,tempsurf,presurf,snow,raysoli,lhflux,swat,modwind,
     s     Uf,Vf,Wf,Kzf,rainf,hcloudf,attenuationf,humidf,tempf,pressf,
     $     tempsurff,presurff,snowf,raysolif,lhfluxf,swatf,modwindf)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This routine sets given meteorological fields.
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     UF, VF, WF: 3D wind fields along X, Y, Z.
C     KZF: 3D Kz field.
C     RAINF: 2D rain intensity field.
C     HCLOUDF: 2D height of cloud basis.
C     ATTENUATIONF: 3D cloud attenuation field.
C     HUMIDF: 3D relative humidity field.
C     TEMPF: 3D temperature field.
C     PRESSF: 3D pressure field.
C     TEMPSURFF: 2D surface temperature field.
C     PRESURFF: 2D surface pressure field.
C     SNOWF: 2D snow height field.
C     RAYSOLIF: 2D surface solar irradiance field.
C     LHFLUXF: 2D latent heat flux field.
C     SWATF: 3D soil water content field (3 levels).
C     MODWINDF: 2D wind module in first level.
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     -- OUTPUT VARIABLES
C     
C     U, V, W: 3D wind fields along X, Y, Z, set to UF, VF and WF.
C     KZ: 3D Kz field, set to KZF.
C     RAIN: 2D rain intensity field, set to RAINF.
C     HCLOUD: 2D height of cloud basis, set to HYCLOUDF.
C     ATTENUATION: 3D cloud attenuation field, set to ATTENUATIONF.
C     HUMID: 3D relative humidity field, set to HUMIDF.
C     TEMP: 3D temperature field, set to TEMPF.
C     PRESS: 3D pressure field, set to PRESSF.
C     TEMPSURF: 2D surface temperature field at initial time.
C     PRESURF: 2D surface pressure field at initial time.
C     SNOW: 2D snow height field at initial time.
C     RAYSOLI: 2D surface solar irradiance field at initial time.
C     LHFLUX: 2D latent heat flux field at initial time.
C     SWAT: 3D soil water content field at initial time (3 levels).
C     MODWIND: 2D wind module in first level.
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

C     -- INCLUDE FILES
C     PARADOM: parameters for all forced fields.
C     COMUSE: flags and parameters for numerics.

      INCLUDE 'PARADOM.INC'
      INCLUDE 'COMUSE.INC'

      DOUBLE PRECISION Ui(NX+1,NY,NZ),Uf(NX+1,NY,NZ)
      DOUBLE PRECISION Vi(NX,NY+1,NZ),Vf(NX,NY+1,NZ)
      DOUBLE PRECISION Wi(NX,NY,NZ+1),Wf(NX,NY,NZ+1)
      DOUBLE PRECISION Kz(NX,NY,NZ+1),Kzf(NX,NY,NZ+1)
      DOUBLE PRECISION attenuation(NXatt,NYatt,NZatt)
      DOUBLE PRECISION attenuationf(NXatt,NYatt,NZatt)
      DOUBLE PRECISION humid(NXh,NYh,NZh),humidf(NXh,NYh,NZh)
      DOUBLE PRECISION rain(NXr,NYr),rainf(NXr,NYr)
      DOUBLE PRECISION hcloud(NXr,NYr),hcloudf(NXr,NYr)
      DOUBLE PRECISION temp(NXt,NYt,NZt),tempf(NXt,NYt,NZt)
      DOUBLE PRECISION press(NXp,NYp,NZp),pressf(NXp,NYp,NZp)
      DOUBLE PRECISION tempsurf(NXdd,NYdd),tempsurff(NXdd,NYdd)
      DOUBLE PRECISION presurf(NXdd,NYdd),presurff(NXdd,NYdd)
      DOUBLE PRECISION snow(NXdd,NYdd),snowf(NXdd,NYdd)
      DOUBLE PRECISION raysoli(NXdd,NYdd),raysolif(NXdd,NYdd)
      DOUBLE PRECISION lhflux(NXdd,NYdd),lhfluxf(NXdd,NYdd)
      DOUBLE PRECISION swat(NXdd,NYdd,Nlayer),swatf(NXdd,NYdd,Nlayer)
      DOUBLE PRECISION modwind(NXdd,NYdd),modwindf(NXdd,NYdd)

      INTEGER Ji,Jj,Jk

      IF (LADVEC) THEN

         DO Jk=1,Nz
            DO Jj=1,Ny
               DO Ji=1,Nx+1
                  Ui(Ji,Jj,Jk)=Uf(Ji,Jj,Jk)
               ENDDO
            ENDDO
         ENDDO

         DO Jk=1,Nz
            DO Jj=1,Ny+1
               DO Ji=1,Nx
                  Vi(Ji,Jj,Jk)=Vf(Ji,Jj,Jk)
               ENDDO
            ENDDO
         ENDDO

         DO Jk=1,Nz+1
            DO Jj=1,Ny
               DO Ji=1,Nx
                  Wi(Ji,Jj,Jk)=Wf(Ji,Jj,Jk)
               ENDDO
            ENDDO
         ENDDO

      ENDIF

      IF (LDIFF) THEN

         DO Jk=1,Nz+1
            DO Jj=1,Ny
               DO Ji=1,Nx
                  Kz(Ji,Jj,Jk)=Kzf(Ji,Jj,Jk)
               ENDDO
            ENDDO
         ENDDO

      ENDIF

      IF (LCHIM) THEN

         DO Jk=1,Nzatt
            DO Jj=1,Nyatt
               DO Ji=1,Nxatt
                  attenuation(Ji,Jj,Jk)=attenuationf(Ji,Jj,Jk)
               ENDDO
            ENDDO
         ENDDO

         DO Jj=1,Nyr
            DO Ji=1,Nxr
               rain(Ji,Jj)=rainf(Ji,Jj)
            ENDDO
         ENDDO

      ENDIF

      IF (LCHIM.OR.LVDEPOCOMP) THEN

         DO Jk=1,Nzh
            DO Jj=1,Nyh
               DO Ji=1,Nxh
                  humid(Ji,Jj,Jk)=humidf(Ji,Jj,Jk)
               ENDDO
            ENDDO
         ENDDO

         DO Jk=1,Nzt
            DO Jj=1,Nyt
               DO Ji=1,Nxt
                  temp(Ji,Jj,Jk)=tempf(Ji,Jj,Jk)
               ENDDO
            ENDDO
         ENDDO

         DO Jk=1,Nzp
            DO Jj=1,Nyp
               DO Ji=1,Nxp
                  press(Ji,Jj,Jk)=pressf(Ji,Jj,Jk)
               ENDDO
            ENDDO
         ENDDO

      ENDIF

      IF (LCLOUDU) THEN

         DO Jj=1,Nyr
            DO Ji=1,Nxr
               hcloud(Ji,Jj)=hcloudf(Ji,Jj)
            ENDDO
         ENDDO

      ENDIF

      IF (LVDEPOCOMP) THEN

         DO Jj=1,Nydd
            DO Ji=1,Nxdd
               tempsurf(Ji,Jj)=tempsurff(Ji,Jj)
               presurf(Ji,Jj)=presurff(Ji,Jj)
               snow(Ji,Jj)=snowf(Ji,Jj)
               raysoli(Ji,Jj)=raysolif(Ji,Jj)
               lhflux(Ji,Jj)=lhfluxf(Ji,Jj)
               raysoli(Ji,Jj)=raysolif(Ji,Jj)
               lhflux(Ji,Jj)=lhfluxf(Ji,Jj)
               modwind(Ji,Jj)=modwindf(Ji,Jj)
               DO Jk=1,Nlayer
                  swat(Ji,Jj,Jk)=swatf(Ji,Jj,Jk)
               ENDDO
            ENDDO
         ENDDO

      ENDIF

      END

