C------------------------------------------------------------------------
C     
C     Copyright (C) 2001-2003 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE calc_conc(ts,
     s     Ui,Vi,Wi,DRkz,DLrain,DLhcloud,DLattenuation,DLhumid,DLtemp,
     s     DLpress,DLCforc2d,DLVdep,DLCsourc,DLCemis,
     s     DLCclx,DLCcly,DLCclz,DLCphotolysis_rates,DLscavg,
     s     Uf,Vf,Wf,DRkzf,DLrainf,DLhcloudf,DLattenuationf,DLhumidf,
     s     DLtempf,DLpressf,DLCforc2df,DLVdepf,DLCsourcf,DLCemisf,
     s     DLCclxf,DLCclyf,DLCclzf,DLCphotolysis_ratesf,DLscavgf,
     s     DLCG)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This routine computes one timestep of the CTM.
C     The splitting scheme is a first-order scheme with the sequential
C     order:
C     volumic source/advection/diffusion/chemistry.
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     TS: initial time (GMT, computed from January 1st, [s]).
C     UI, VI, WI: 3D wind fields at initial time along X, Y, Z ([m/s]).
C     DRKZ: 3D Kz field at initial time ([m^2/s]).
C     DLRAIN: 2D rain intensity field at initial time ([mm/h]).
C     DLHCLOUD: 2D height of cloud basis ([m])
C     DLATTENUATION: 3D cloud attenuation field at initial time.
C     DLHUMID: 3D specific humidity field at initial time ([-]).
C     DLTEMP: 3D temperature field at initial time ([K]).
C     DLPRESS: 3D pressure field at initial time ([Pa]).
C     DLCFORC: array of 2D forced chemical concentration fields at
C     # initial time ([\mu.g/m^3]).
C     DLVDEP: 2D dry deposition velocities at initial time ([m/s]).
C     DLCSOURC: array of chemical volumic emissions at initial time
C     # ([\mu.g/m^3/s]).
C     DLCEMIS: array of chemical surfacic emissions at initial time
C     # ([\mu.g/m^2/s]).
C     DLCCLX: array of 2D chemical boundary conditions in x-direction
C     # at initial time ([\mu.g/m^3]).
C     DLCCLY: array of 2D chemical boundary conditions in y-direction
C     # at initial time ([\mu.g/m^3]).
C     DLCCLZ: array of 2D chemical boundary conditions in z-direction
C     # at initial time ([\mu.g/m^3]).
C     DLCPHOTOLYSIS_RATES: photochemical kinetic rates
C     # at initial time ([s^{-1}]).
C     
C     The same variables are defined at final time of the timestep.
C     'f' is then put at the end of the name.
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     DLCG: array of 3D chemical concentrations ([\mu.g/m^3]). 
C     # Before entry, it is given at initial time of the timestep.
C     # On exit, it is computed at final time of the timestep.
C     
C     -- OUTPUT VARIABLES
C     
C     DLSCAVG: gas below-cloud scavenging ([s^{-1}])
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     2002/03/28: boundary conditions DLCCLX, DLCCLY, DLCCLZ
C     # (Karine Sartelet, CEREA)
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Jaouad Boutahar, CEREA, June 2001.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

C     -- INCLUDE FILES
C     PARADOM: parameters for all forced fields.
C     PARACHEM: parameters for sizes of 'chemical' arrays.
C     COMUSE: flags and parameters for numerics. 
C     COMCHEM: global variables for chemical species. 

      INCLUDE 'PARADOM.INC'
      INCLUDE 'PARACHEM.INC'
      INCLUDE 'COMUSE.INC'
      INCLUDE 'COMCHEM.INC'

      DOUBLE PRECISION Ui(NX+1,NY,NZ),Uf(NX+1,NY,NZ)  
      DOUBLE PRECISION Vi(NX,NY+1,NZ),Vf(NX,NY+1,NZ)
      DOUBLE PRECISION Wi(NX,NY,NZ+1),Wf(NX,NY,NZ+1)
      DOUBLE PRECISION DRkz(NX,NY,NZ+1),DRkzf(NX,NY,NZ+1)
      DOUBLE PRECISION DLtemp(NXt,NYt,NZt),DLtempf(NXt,NYt,NZt)
      DOUBLE PRECISION DLrain(NXr,NYr),DLrainf(NXr,NYr)
      DOUBLE PRECISION DLhcloud(NXr,NYr),DLhcloudf(NXr,NYr)
      DOUBLE PRECISION DLattenuation(NXatt,NYatt,NZatt)
      DOUBLE PRECISION DLattenuationf(NXatt,NYatt,NZatt)
      DOUBLE PRECISION DLhumid(NXh,NYh,NZh),DLhumidf(NXh,NYh,NZh)
      DOUBLE PRECISION DLpress(NXp,NYp,NZp),DLpressf(NXp,NYp,NZp)

      DOUBLE PRECISION DLCforc2d(Nxf,Nyf,NESPF)
      DOUBLE PRECISION DLCforc2df(Nxf,Nyf,NESPF)
      DOUBLE PRECISION DLCsourc(Nx,Ny,Nzs,Nsrc)
      DOUBLE PRECISION DLCsourcf(Nx,Ny,Nzs,Nsrc)
      DOUBLE PRECISION DLVdep(Nxdd,Nydd,Ndep),Cdepot(Nx,Ny)
      DOUBLE PRECISION DLVdepf(Nxdd,Nydd,Ndep),Cdepotf(Nx,Ny)
      DOUBLE PRECISION DLscavg(NXr,NYr,NZ,Nscav)
      DOUBLE PRECISION DLscavgf(NXr,NYr,NZ,Nscav)
      DOUBLE PRECISION DLCemis(Nx,Ny,Nemi),Cemis(Nx,Ny)
      DOUBLE PRECISION DLCemisf(Nx,Ny,Nemi),Cemisf(Nx,Ny)

      DOUBLE PRECISION DLCclx(2,Nyl,Nzl,Nspcl)
      DOUBLE PRECISION DLCclxf(2,Nyl,Nzl,Nspcl)
      DOUBLE PRECISION DLCcly(Nxl,2,Nzl,Nspcl)
      DOUBLE PRECISION DLCclyf(Nxl,2,Nzl,Nspcl)
      DOUBLE PRECISION DLCclz(Nxl,Nyl,Nspcl)
      DOUBLE PRECISION DLCclzf(Nxl,Nyl,Nspcl)
      DOUBLE PRECISION DLCphotolysis_rates(NRphot,Nx,Ny,Nz)
      DOUBLE PRECISION DLCphotolysis_ratesf(NRphot,Nx,Ny,Nz)

      DOUBLE PRECISION DLCG(Nx,Ny,Nz,NESP)
      DOUBLE PRECISION ZC(Nx,Ny,Nz)
      DOUBLE PRECISION Zclx(2,Nyl,Nzl),Zcly(Nxl,2,Nzl),Zclz(Nxl,Nyl)

      DOUBLE PRECISION ts,tf,Dt

      INTEGER i
      INTEGER In,Jn,Kn,Kl
      INTEGER Jemis,Jdepot,Jcl

      LOGICAL CLESP

      Dt=Dtstep

      IF(LADVEC)THEN

C------------------------------------------------------------------------
C     1 - Advection

         Jcl = 1
         
         DO i=1,Nesp

C     Treatment of boundary conditions
C     CLESP = TRUE (FALSE) if boundary conditions are given (not given).

            CLESP = .FALSE.

            If(LCBC.AND.(i.EQ.Nespcl(Jcl))) then

               CLESP = .TRUE.

               DO kn=1,NZ
                  DO jn=1,NY
                     DO Kl=1,2
                        Zclx(kl,jn,kn) = DLCclx(kl,jn,kn,Jcl)
                     ENDDO
                  ENDDO
               ENDDO

               DO kn=1,NZ
                  DO Kl=1,2
                     DO jn=1,NX
                        Zcly(jn,kl,kn) = DLCcly(jn,kl,kn,Jcl)
                     ENDDO
                  ENDDO
               ENDDO

               DO kn=1,NY
                  DO jn=1,NX
                     Zclz(jn,kn) = DLCclz(jn,kn,Jcl)
                  ENDDO
               ENDDO
               IF (Jcl.LT.Nspcl) Jcl = Jcl + 1

            Endif

C     1.1 - Integration of advection

            DO kn=1,NZ
               DO jn=1,NY
                  DO in=1,NX
                     ZC(in,jn,kn)=DLCG(in,jn,kn,i)
                  ENDDO
               ENDDO
            ENDDO

            CALL advection(Ui, Vi, Wi, CLESP, Zclx, Zcly, Zclz, dt, ZC)

            DO kn=1,NZ
               DO jn=1,NY
                  DO in=1,NX
                     DLCG(in,jn,kn,i)=ZC(in,jn,kn)
                  ENDDO
               ENDDO
            ENDDO

         ENDDO                  ! loop on species.

      ENDIF                     ! advection.

C------------------------------------------------------------------------
C     2 - Diffusion

      tf=ts+Dtstep 
      Jemis=1
      Jdepot=1

      IF(LDIFF)THEN 

         DO i=1,NESP

            DO kn=1,NZ
               DO jn=1,NY
                  DO in=1,NX
                     ZC(in,jn,kn)=DLCG(in,jn,kn,i)
                  ENDDO
               ENDDO
            ENDDO

C     2.1 - Diffusion along x

            CALL DIFF_X(ts,tf,ZC)

C     2.2 - Diffusion along y

            CALL DIFF_Y(ts,tf,ZC)

C     2.3 - Diffusion along z

C     Dry deposition velocities. 

            DO jn=1,NY
               DO in=1,NX
                  Cdepot(in,jn)=0.D0
                  Cdepotf(in,jn)=0.D0
               ENDDO
            ENDDO

            IF(LVDEPO)THEN

               IF(i.EQ.Nespd(Jdepot))THEN
                  
                  DO jn=1,NYdd
                     DO in=1,NXdd
                        Cdepot(in,jn)=DLVdep(in,jn,Jdepot)
                        Cdepotf(in,jn)=DLVdepf(in,jn,Jdepot)
                     ENDDO
                  ENDDO
                  
                  IF (Jdepot.LT.Ndep) Jdepot=Jdepot+1

               ENDIF            ! species.

            ENDIF               ! dry deposition.

C     Surfacic emission factors.

            DO jn=1,NY
               DO in=1,NX
                  Cemis(in,jn)=0.D0
                  Cemisf(in,jn)=0.D0
               ENDDO
            ENDDO

            IF(LEMISS)THEN

               IF(i.EQ.Nespem(Jemis))THEN

                  DO jn=1,NY
                     DO in=1,NX
                        Cemis(in,jn)=DLCemis(in,jn,Jemis)
                        Cemisf(in,jn)=DLCemisf(in,jn,Jemis)
                     ENDDO
                  ENDDO

                  IF (Jemis.LT.Nemi) Jemis=Jemis+1

               ENDIF            ! species.

            ENDIF               ! surfacic emissions.

C     Vertical diffusion.
            CALL DIFF_Z (ts,tf,DRKz,Cdepot,Cemis,
     s           DRKzf,Cdepotf,Cemisf,ZC)

            DO kn=1,NZ
               DO jn=1,NY
                  DO in=1,NX
                     DLCG(in,jn,kn,i)=ZC(in,jn,kn)
                  ENDDO
               ENDDO
            ENDDO

         ENDDO                  ! loop on species.

      ENDIF                     ! diffusion.

C------------------------------------------------------------------------
C     3 - Scavenging

      IF(LSCAV) THEN  

         tf=ts+Dtstep

         CALL SCAVENGING (ts,tf,DLscavg,DLscavgf,DLCG)

      ENDIF                     ! scavenging.

C------------------------------------------------------------------------
C     4 - Chemistry

      IF(LCHIM) THEN   

         tf=ts+Dtstep 

         CALL CHEM (ts,DLrain,DLhcloud,DLattenuation,DLhumid,DLtemp,
     s        DLpress,DLCsourc,DLCforc2d,DLCphotolysis_rates,
     s        tf,DLrainf,DLhcloudf,DLattenuationf,DLhumidf,DLtempf,
     s        DLpressf,DLCsourcf,DLCforc2df,DLCphotolysis_ratesf,
     s        DLCG)
      ENDIF                     ! chemistry.

      END
