C------------------------------------------------------------------------
C     
C     Copyright (C) 2001-2003 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE  calc_conc(ts,
     s     Ui,Vi,Wi,DRkz,DLrain,DLhcloud,DLattenuation,DLhumid,DLtemp,
     s     DLpress,DLCforc2d,DLVdep,DLCsourc,DLCemis,
     s     DLCclx,DLCcly,DLCclz,DLCphotolysis_rates,DLscavg,
     s     Uf,Vf,Wf,DRkzf,DLrainf,DLhcloudf,DLattenuationf,DLhumidf,
     $     DLtempf,DLpressf,
     s     DLCforc2df,DLVdepf,DLCsourcf,DLCemisf,
     s     DLCclxf,DLCclyf,DLCclzf,DLCphotolysis_ratesf,DLscavgf,
     s     DLCG)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This routine computes one timestep of the CTM.
C     The splitting scheme is a second-order scheme with the sequential
C     order:
C     volumic source(DT/2)/advection(DT/2)/chemistry(DT/2)
C     diffusion(DT)/advection(DT/2)chemistry(DT/2)/
C     advection(DT/2)/volumic source(DT/2).
C     
C     with symmetrization of advection and diffusion steps.
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     TS: initial time (GMT, computed from January 1st, [s]).
C     UI, VI, WI: 3D wind fields at initial time along X, Y, Z ([m/s]).
C     DRKZ: 3D Kz field at initial time ([m^2/s]).
C     DLRAIN: 2D rain intensity field at initial time ([mm/h]).
C     DLHCLOUD: 2D height of cloud basis ([m])
C     DLATTENUATION: 3D cloud attenuation field at initial time.
C     DLHUMID: 3D specific humidity field at initial time ([-]).
C     DLTEMP: 3D temperature field at initial time ([K]).
C     DLPRESS: 3D pressure field at initial time ([Pa]).
C     DLCFORC: array of 2D forced chemical concentration fields at
C     # initial time ([\mu.g/m^3]).
C     DLVDEP: 2D dry deposition velocities at initial time ([m/s]).
C     DLCSOURC: array of chemical volumic emissions at initial time
C     # ([\mu.g/m^3/s]).
C     DLCEMIS: array of chemical surfacic emissions at initial time
C     # ([\mu.g/m^2/s]).
C     DLCCLX: array of 2D chemical boundary conditions in x-direction
C     # at initial time ([\mu.g/m^3]).
C     DLCCLY: array of 2D chemical boundary conditions in y-direction
C     # at initial time ([\mu.g/m^3]).
C     DLCCLZ: array of 2D chemical boundary conditions in z-direction
C     # at initial time ([\mu.g/m^3]).
C     DLCPHOTOLYSIS_RATES: photochemical kinetic rates
C     # at initial time ([s^{-1}]).
C     
C     The same variables are defined at final time of the timestep.
C     'f' is then put at the end of the name.
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     DLCG: array of 3D chemical concentrations ([\mu.g/m^3]). 
C     # Before entry, it is given at initial time of the timestep.
C     # On exit, it is computed at final time of the timestep.
C     
C     -- OUTPUT VARIABLES
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     2002/03/28: boundary conditions DLCCLX, DLCCLY, DLCCLZ
C     # (Karine Sartelet, CEREA)
C     2002/03/18 : bug Emission/Deposition (Denis Qulo, CEREA). 
C     2002/04/16: bug time management (ts, tf) (Karine Sartelet).
C     2002/04/25: bug emission (jemis) (Karine Sartelet)
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Jaouad Boutahar, CEREA, June 2001.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

C     -- INCLUDE FILES
C     PARADOM: parameters for all forced fields.
C     PARACHEM: parameters for sizes of 'chemical' arrays.
C     COMUSE: flags and parameters for numerics. 
C     COMCHEM: global variables for chemical species. 

      INCLUDE 'PARADOM.INC'
      INCLUDE 'PARACHEM.INC'
      INCLUDE 'COMUSE.INC'
      INCLUDE 'COMCHEM.INC'

      DOUBLE PRECISION Ui(NX+1,NY,NZ),Uf(NX+1,NY,NZ),Um(NX+1,NY,NZ)
      DOUBLE PRECISION Vi(NX,NY+1,NZ),Vf(NX,NY+1,NZ),Vm(NX,NY+1,NZ)
      DOUBLE PRECISION Wi(NX,NY,NZ+1),Wf(NX,NY,NZ+1),Wm(NX,NY,NZ+1)
      DOUBLE PRECISION DRkz(NX,NY,NZ+1),DRkzf(NX,NY,NZ+1)
      DOUBLE PRECISION DLtemp(NXt,NYt,NZt),DLtempf(NXt,NYt,NZt)
      DOUBLE PRECISION DLtempm(NXt,NYt,NZt)
      DOUBLE PRECISION DLrain(NXr,NYr),DLrainf(NXr,NYr)
      DOUBLE PRECISION DLrainm(NXr,NYr)
      DOUBLE PRECISION DLhcloud(NXr,NYr),DLhcloudf(NXr,NYr)
      DOUBLE PRECISION DLhcloudm(NXr,NYr)
      DOUBLE PRECISION DLattenuation(NXatt,NYatt,NZatt)
      DOUBLE PRECISION DLattenuationf(NXatt,NYatt,NZatt)
      DOUBLE PRECISION DLattenuationm(NXatt,NYatt,NZatt)
      DOUBLE PRECISION DLhumid(NXh,NYh,NZh),DLhumidf(NXh,NYh,NZh)
      DOUBLE PRECISION DLhumidm(NXh,NYh,NZh)
      DOUBLE PRECISION DLpress(NXp,NYp,NZp),DLpressf(NXp,NYp,NZp)
      DOUBLE PRECISION DLpressm(NXp,NYp,NZp)

      DOUBLE PRECISION DLCforc2d(Nxf,Nyf,NESPF)
      DOUBLE PRECISION DLCforc2df(Nxf,Nyf,NESPF)
      DOUBLE PRECISION DLCforc2dm(Nxf,Nyf,NESPF)
      DOUBLE PRECISION DLCsourc(Nx,Ny,Nzs,Nsrc)
      DOUBLE PRECISION DLCsourcf(Nx,Ny,Nzs,Nsrc)
      DOUBLE PRECISION DLCsourcm(Nx,Ny,Nzs,Nsrc)
      DOUBLE PRECISION DLVdep(Nxdd,Nydd,Ndep),Cdepot(Nx,Ny)
      DOUBLE PRECISION DLVdepf(Nxdd,Nydd,Ndep),Cdepotf(Nx,Ny)
      DOUBLE PRECISION DLVdepm(Nx,Ny,Ndep)
      DOUBLE PRECISION DLscavg(NXr,NYr,NZ,Nscav)
      DOUBLE PRECISION DLscavgf(NXr,NYr,NZ,Nscav)
      DOUBLE PRECISION DLscavgm(NXr,NYr,NZ,Nscav)
      DOUBLE PRECISION DLCemis(Nx,Ny,Nemi),Cemis(Nx,Ny)
      DOUBLE PRECISION DLCemisf(Nx,Ny,Nemi),Cemisf(Nx,Ny)
      DOUBLE PRECISION DLCemism(Nx,Ny,Nemi)

      DOUBLE PRECISION DLCclx(2,Nyl,Nzl,Nspcl)
      DOUBLE PRECISION DLCclxf(2,Nyl,Nzl,Nspcl)
      DOUBLE PRECISION DLCclxm(2,Nyl,Nzl,Nspcl)
      DOUBLE PRECISION DLCcly(Nxl,2,Nzl,Nspcl)
      DOUBLE PRECISION DLCclyf(Nxl,2,Nzl,Nspcl)
      DOUBLE PRECISION DLCclym(Nxl,2,Nzl,Nspcl)
      DOUBLE PRECISION DLCclz(Nxl,Nyl,Nspcl)
      DOUBLE PRECISION DLCclzf(Nxl,Nyl,Nspcl)
      DOUBLE PRECISION DLCclzm(Nxl,Nyl,Nspcl)
      DOUBLE PRECISION DLCphotolysis_rates(NRphot,Nx,Ny,Nz)
      DOUBLE PRECISION DLCphotolysis_ratesm(NRphot,Nx,Ny,Nz)
      DOUBLE PRECISION DLCphotolysis_ratesf(NRphot,Nx,Ny,Nz)

      DOUBLE PRECISION DLCG(Nx,Ny,Nz,NESP),ZC(Nx,Ny,Nz)
      DOUBLE PRECISION Zclx(2,Nyl,Nzl),Zcly(Nxl,2,Nzl),Zclz(Nxl,Nyl)

      DOUBLE PRECISION ts,ts2,tf2,tf,Dt
      INTEGER i,In,Jn,Kn,kl
      INTEGER Jemis,Jdepot,Jcl

      LOGICAL CLESP

      Dt = Dtstep/2.d0
      ts2 = ts + Dtstep/2.d0
      tf2 = ts + Dtstep/2.d0
      tf = ts + Dtstep

C------------------------------------------------------------------------
C     Computation of forcing fields at time TS2 (middle point).

      CALL linterpol(Ui,Vi,Wi,Uf,Vf,Wf,Um,Vm,Wm,
     s     DLrain,DLhcloud,DLattenuation,DLhumid,DLtemp,DLpress,
     s     DLrainf,DLhcloudf,DLattenuationf,DLhumidf,DLtempf,DLpressf,
     s     DLrainm,DLhcloudm,DLattenuationm,DLhumidm,DLtempm,DLpressm,
     s     DLCsourc,DLCsourcf,DLVdep,DLVdepf,
     $     DLscavg,DLscavgf,DLCemis,DLCemisf,
     s     DLCsourcm,DLVdepm,DLscavgm,DLCemism,
     s     DLCforc2d,DLCforc2df,DLCforc2dm,
     s     DLCclx,DLCclxf,DLCclxm,
     s     DLCcly,DLCclyf,DLCclym,
     s     DLCclz,DLCclzf,DLCclzm,
     $     DLCphotolysis_rates,DLCphotolysis_ratesf,
     $     DLCphotolysis_ratesm)


C------------------------------------------------------------------------
C     1 - Advection (from TS to TS+DT/2)

      IF(LADVEC)THEN

         Jcl = 1

         DO i=1,NESP

C     Treatment of boundary conditions
C     CLESP = TRUE (FALSE) if boundary conditions are given (not given).

            CLESP = .FALSE.

            If(LCBC.AND.(i.EQ.Nespcl(Jcl))) then
               CLESP = .TRUE.
               DO kn=1,NZ
                  DO jn=1,NY
                     Do kl = 1,2
                        Zclx(kl,jn,kn) = DLCclx(kl,jn,kn,Jcl)
                     Enddo
                  ENDDO
               ENDDO
               DO kn=1,NZ
                  Do kl=1,2
                     DO jn=1,NX
                        Zcly(jn,kl,kn) = DLCcly(jn,kl,kn,Jcl)
                     Enddo
                  ENDDO
               ENDDO
               DO kn=1,NY
                  DO jn=1,NX
                     Zclz(jn,kn) = DLCclz(jn,kn,Jcl)
                  ENDDO
               ENDDO
               IF (Jcl.LT.Nspcl) Jcl = Jcl + 1

            Endif

C     1.1 - Integration of advection

            DO kn=1,NZ
               DO jn=1,NY
                  DO in=1,NX
                     ZC(in,jn,kn)=DLCG(in,jn,kn,i)
                  ENDDO
               ENDDO
            ENDDO

            CALL advection(Ui, Vi, Wi, CLESP, Zclx, Zcly, Zclz, dt, ZC)

            DO kn=1,NZ
               DO jn=1,NY
                  DO in=1,NX
                     DLCG(in,jn,kn,i)=ZC(in,jn,kn)
                  ENDDO
               ENDDO
            ENDDO

         ENDDO                  ! loop on species

      ENDIF                     ! advection


C------------------------------------------------------------------------
C     2 - Chemistry (from TS to TS+DT/2)

      IF(LCHIM)THEN

         CALL CHEM (ts,DLrain,DLhcloud,DLattenuation,DLhumid,DLtemp,
     s        DLpress,DLCsourc,DLCforc2d,DLCphotolysis_rates,
     s        tf2,DLrainm,DLhcloudm,DLattenuationm,DLhumidm,DLtempm,
     s        DLpressm,DLCsourcm,DLCforc2dm,DLCphotolysis_ratesm,
     s        DLCG)

      ENDIF


C------------------------------------------------------------------------
C     3 - Scavneging (from TS to TS+DT/2)

      IF(LSCAV)THEN

         CALL SCAVENGING (ts,tf2,DLscavg,DLscavgm,DLCG)

      ENDIF


C------------------------------------------------------------------------
C     4 - Diffusion (from TS to TS+DT): 
C     X(DT/2)Y(DT/2)Z(DT)Y(DT/2)X(DT/2)

      Jemis=1
      Jdepot=1

      DO i=1,NESP  

         DO kn=1,NZ
            DO jn=1,NY
               DO in=1,NX
                  ZC(in,jn,kn)=DLCG(in,jn,kn,i)
               ENDDO
            ENDDO
         ENDDO

         IF(LDIFF)THEN

C     4.1 - Diffusion along X (from TS to TS+DT/2)

            CALL DIFF_X(ts,tf2,ZC)
            

C     4.2 - Diffusion along Y (from TS to TS+DT/2)

            CALL DIFF_Y(ts,tf2,ZC)

C     4.3 - Diffusion along Z (from TS to TS+DT)

C     Dry deposition velocities.

            DO jn=1,NY
               DO in=1,NX
                  Cdepot(in,jn)=0.D0
                  Cdepotf(in,jn)=0.D0
               ENDDO
            ENDDO

            IF(LVDEPO)THEN

               IF(i.EQ.Nespd(Jdepot))THEN

                  DO jn=1,NYdd
                     DO in=1,NXdd
                        Cdepot(in,jn)=DLVdep(in,jn,Jdepot)
                        Cdepotf(in,jn)=DLVdepf(in,jn,Jdepot)
                     ENDDO
                  ENDDO
                  IF (Jdepot.LT.Ndep) Jdepot=Jdepot+1
               ENDIF            ! species

            ENDIF               ! dry deposition   

C     Surfacic emission factors.

            DO jn=1,NY
               DO in=1,NX
                  Cemis(in,jn)=0.D0
                  Cemisf(in,jn)=0.D0
               ENDDO
            ENDDO

            IF(LEMISS)THEN

               IF(i.EQ.Nespem(Jemis))THEN

                  DO jn=1,NY
                     DO in=1,NX
                        Cemis(in,jn)=DLCemis(in,jn,Jemis)
                        Cemisf(in,jn)=DLCemisf(in,jn,Jemis)
                     ENDDO
                  ENDDO

                  IF (Jemis.LT.Nemi) Jemis=Jemis+1

               ENDIF            ! species.

            ENDIF               ! surfacic emissions.

            CALL DIFF_Z (ts,tf,DRkz,Cdepot,Cemis,
     s           DRKzf,Cdepotf,Cemisf,ZC)

C     4.4 - Diffusion along Y (from TS+DT/2 to TS+DT)

            CALL DIFF_Y(ts2,tf,ZC)

C     4.5 - Diffusion along X (from TS+DT/2 to TS+DT)
            
            CALL DIFF_X(ts2,tf,ZC)

         ENDIF                  ! diffusion.

         DO kn=1,NZ
            DO jn=1,NY
               DO in=1,NX
                  DLCG(in,jn,kn,i)=ZC(in,jn,kn)
               ENDDO
            ENDDO
         ENDDO

      ENDDO                     ! loop on species.

C------------------------------------------------------------------------
C     5 - Scavenging (from TS+DT/2 to TS+DT)

      IF(LSCAV)THEN

         CALL SCAVENGING (ts2,tf,DLscavg,DLscavgm,DLCG)

      ENDIF

C------------------------------------------------------------------------
C     6 - Chemistry (from TS+DT/2 to TS+DT)

      IF(LCHIM)THEN

         CALL CHEM (ts2,DLrainm,DLhcloudm,DLattenuationm,DLhumidm,
     s        DLtempm,DLpressm,DLCsourcm,DLCforc2dm,
     $        DLCphotolysis_ratesm,
     s        tf,DLrainf,DLhcloudf,DLattenuationf,DLhumidf,DLtempf,
     s        DLpressf,DLCsourcf,DLCforc2df,DLCphotolysis_ratesf,
     s        DLCG)

      ENDIF
C     
C------------------------------------------------------------------------
C     7 - Advection (from TS+DT/2 to TS+DT) in reverse order

      IF(LADVEC)THEN

         Jcl = 1

         DO i=1,NESP

C     Treatment of boundary conditions
C     CLESP = TRUE (FALSE) if boundary conditions are given (not given).

            CLESP = .FALSE.

            If(LCBC.AND.(i.EQ.Nespcl(Jcl))) then

               CLESP = .TRUE.

               DO kn=1,NZ
                  DO jn=1,NY
                     Do kl=1,2
                        Zclx(kl,jn,kn) = DLCclxm(kl,jn,kn,Jcl)
                     Enddo
                  ENDDO
               ENDDO
               DO kn=1,NZ
                  Do kl=1,2
                     DO jn=1,NX
                        Zcly(jn,kl,kn) = DLCclym(jn,kl,kn,Jcl)
                     Enddo
                  ENDDO
               ENDDO
               DO kn=1,NY
                  DO jn=1,NX
                     Zclz(jn,kn) = DLCclzm(jn,kn,Jcl)
                  ENDDO
               ENDDO
               IF (Jcl.LT.Nspcl) Jcl = Jcl + 1

            Endif
            
C     7.1 - Integration of advection

            DO kn=1,NZ
               DO jn=1,NY
                  DO in=1,NX
                     ZC(in,jn,kn)=DLCG(in,jn,kn,i)
                  ENDDO
               ENDDO
            ENDDO

            CALL advection(Um, Vm, Wm, CLESP, Zclx, Zcly, Zclz, dt, ZC)
            
            DO kn=1,NZ
               DO jn=1,NY
                  DO in=1,NX
                     DLCG(in,jn,kn,i)=ZC(in,jn,kn)
                  ENDDO
               ENDDO
            ENDDO

         ENDDO

      ENDIF

      END
