C------------------------------------------------------------------------
C     
C     Copyright (C) 2001-2003 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE  linterpol(Ui,Vi,Wi,Uf,Vf,Wf,Um,Vm,Wm,
     s     DLrain,DLhcloud,DLattenuation,DLhumid,DLtemp,DLpress,
     s     DLrainf,DLhcloudf,DLattenuationf,DLhumidf,DLtempf,DLpressf,
     s     DLrainm,DLhcloudm,DLattenuationm,DLhumidm,DLtempm,DLpressm,
     s     DLCsourc,DLCsourcf,DLVdep,DLVdepf,
     $     DLscavg,DLscavgf,DLCemis,DLCemisf,
     s     DLCsourcm,DLVdepm,DLscavgm,DLCemism,
     s     DLCforc2d,DLCforc2df,DLCforc2dm,
     s     DLCclx,DLCclxf,DLCclxm,
     s     DLCcly,DLCclyf,DLCclym,
     s     DLCclz,DLCclzf,DLCclzm,
     $     DLCphotolysis_rates,DLCphotolysis_ratesf,
     $     DLCphotolysis_ratesm)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This routine computes by linear interpolation the forcing fields
C     at middle point for a second-order splitting scheme. 
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     TS: initial time (GMT, computed from January 1st, [s]).
C     UI, VI, WI: 3D wind fields at initial time along X, Y, Z ([m/s]).
C     DRKZ: 3D Kz field at initial time ([m^2/s]).
C     DLRAIN: 2D rain intensity field at initial time ([mm/h]).
C     DLHCLOUD: 2D height of cloud basis ([m])
C     DLATTENUATION: 3D cloud attenuation field at initial time.
C     DLHUMID: 3D relative humidity field at initial time ([%]).
C     DLTEMP: 3D temperature field at initial time ([K]).
C     DLPRESS: 3D pressure field at initial time ([Pa]).
C     DLCFORC: array of 2D forced chemical concentration fields at
C     # initial time ([\mu.g/m^3]).
C     DLVDEP: 2D dry deposition velocities at initial time ([m/s]).
C     DLCSOURC: array of chemical volumic emissions at initial time
C     # ([\mu.g/m^3/s]).
C     DLCEMIS: array of chemical surfacic emissions at initial time
C     # ([\mu.g/m^2/s]).
C     DLCCLX: array of 2D chemical boundary conditions in x-direction
C     # at initial time ([\mu.g/m^3]).
C     DLCCLY: array of 2D chemical boundary conditions in y-direction
C     # at initial time ([\mu.g/m^3]).
C     DLCCLZ: array of 2D chemical boundary conditions in z-direction
C     # at initial time ([\mu.g/m^3]).
C     DLCPHOTOLYSIS_RATES: photochemical kinetic rates
C     # at initial time ([s^{-1}]).
C     
C     The same variables are defined at final time of the timestep.
C     'f' is then put at the end of the name.
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     -- OUTPUT VARIABLES
C     
C     The same variables are defined at middle point of the timestep.
C     'm' is then put at the end of the name.
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Jaouad Boutahar, CEREA, June 2001.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

C     -- INCLUDE FILES
C     PARADOM: parameters for all forced fields.
C     PARACHEM: parameters for sizes of 'chemical' arrays.

      INCLUDE 'PARADOM.INC'
      INCLUDE 'PARACHEM.INC'

      DOUBLE PRECISION Ui(NX+1,NY,NZ),Uf(NX+1,NY,NZ)
      DOUBLE PRECISION Um(NX+1,NY,NZ)               
      DOUBLE PRECISION Vi(NX,NY+1,NZ),Vf(NX,NY+1,NZ)
      DOUBLE PRECISION Vm(NX,NY+1,NZ)               
      DOUBLE PRECISION Wi(NX,NY,NZ+1),Wf(NX,NY,NZ+1)
      DOUBLE PRECISION Wm(NX,NY,NZ+1)               
      DOUBLE PRECISION DLtemp(NXt,NYt,NZt),DLtempf(NXt,NYt,NZt)
      DOUBLE PRECISION DLtempm(NXt,NYt,NZt)
      DOUBLE PRECISION DLrain(NXr,NYr),DLrainf(NXr,NYr)
      DOUBLE PRECISION DLrainm(NXr,NYr)
      DOUBLE PRECISION DLhcloud(NXr,NYr),DLhcloudf(NXr,NYr)
      DOUBLE PRECISION DLhcloudm(NXr,NYr)
      DOUBLE PRECISION DLattenuation(NXatt,NYatt,NZatt)
      DOUBLE PRECISION DLattenuationf(NXatt,NYatt,NZatt)
      DOUBLE PRECISION DLattenuationm(NXatt,NYatt,NZatt)
      DOUBLE PRECISION DLhumid(NXh,NYh,NZh),DLhumidf(NXh,NYh,NZh)
      DOUBLE PRECISION DLhumidm(NXh,NYh,NZh)
      DOUBLE PRECISION DLpress(NXp,NYp,NZp),DLpressf(NXp,NYp,NZp)
      DOUBLE PRECISION DLpressm(NXp,NYp,NZp)

      DOUBLE PRECISION DLCforc2d(Nxf,Nyf,NESPF)
      DOUBLE PRECISION DLCforc2df(Nxf,Nyf,NESPF)
      DOUBLE PRECISION DLCforc2dm(Nxf,Nyf,NESPF)
      DOUBLE PRECISION DLCsourc(Nx,Ny,Nzs,Nsrc)
      DOUBLE PRECISION DLCsourcf(Nx,Ny,Nzs,Nsrc)
      DOUBLE PRECISION DLCsourcm(Nx,Ny,Nzs,Nsrc) 
      DOUBLE PRECISION DLVdep(Nx,Ny,Ndep),DLVdepf(Nx,Ny,Ndep)
      DOUBLE PRECISION DLVdepm(Nx,Ny,Ndep)                  
      DOUBLE PRECISION DLscavg(Nxr,Nyr,Nz,Nscav)
      DOUBLE PRECISION DLscavgf(Nxr,Nyr,Nz,Nscav)
      DOUBLE PRECISION DLscavgm(Nxr,Nyr,Nz,Nscav)
      DOUBLE PRECISION DLCemis(Nx,Ny,Nemi),DLCemisf(Nx,Ny,Nemi)
      DOUBLE PRECISION DLCemism(Nx,Ny,Nemi)

      DOUBLE PRECISION DLCclx(2,Nyl,Nzl,Nspcl)
      DOUBLE PRECISION DLCclxf(2,Nyl,Nzl,Nspcl)
      DOUBLE PRECISION DLCclxm(2,Nyl,Nzl,Nspcl)
      DOUBLE PRECISION DLCcly(Nxl,2,Nzl,Nspcl)
      DOUBLE PRECISION DLCclyf(Nxl,2,Nzl,Nspcl)
      DOUBLE PRECISION DLCclym(Nxl,2,Nzl,Nspcl)
      DOUBLE PRECISION DLCclz(Nxl,Nyl,Nspcl)
      DOUBLE PRECISION DLCclzf(Nxl,Nyl,Nspcl)
      DOUBLE PRECISION DLCclzm(Nxl,Nyl,Nspcl)

      DOUBLE PRECISION DLCphotolysis_rates(NRphot,Nx,Ny,Nz)
      DOUBLE PRECISION DLCphotolysis_ratesm(NRphot,Nx,Ny,Nz)
      DOUBLE PRECISION DLCphotolysis_ratesf(NRphot,Nx,Ny,Nz)

      INTEGER Ji,Jj,Jk,Jsp,Jl,Jr

C     Advection along X

      DO Jk=1,Nz
         DO Jj=1,Ny
            DO Ji=1,Nx+1
               Um(Ji,Jj,Jk)=(Ui(Ji,Jj,Jk)+Uf(Ji,Jj,Jk))/2.D0
            ENDDO
         ENDDO
      ENDDO

C     Advection along Y

      DO Jk=1,Nz
         DO Jj=1,Ny+1
            DO Ji=1,Nx
               Vm(Ji,Jj,Jk)=(Vi(Ji,Jj,Jk)+Vf(Ji,Jj,Jk))/2.D0
            ENDDO
         ENDDO
      ENDDO

C     Advection along Z

      DO Jk=1,Nz+1
         DO Jj=1,Ny
            DO Ji=1,Nx
               Wm(Ji,Jj,Jk)=(Wi(Ji,Jj,Jk)+Wf(Ji,Jj,Jk))/2.D0
            ENDDO
         ENDDO
      ENDDO

C     Rain intensity

      DO Jj=1,Nyr
         DO Ji=1,Nxr
            DLrainm(Ji,Jj)=(DLrain(Ji,Jj)+DLrainf(Ji,Jj))/2.D0
         ENDDO
      ENDDO

C     Height of cloud basis

      DO Jj=1,Nyr
         DO Ji=1,Nxr
            DLhcloudm(Ji,Jj)=(DLhcloud(Ji,Jj)+DLhcloudf(Ji,Jj))/2.D0
         ENDDO
      ENDDO

C     Temperature

      DO Jk=1,Nzt
         DO Jj=1,Nyt
            DO Ji=1,Nxt
               DLtempm(Ji,Jj,Jk)=(DLtemp(Ji,Jj,Jk)+DLtempf(Ji,Jj,Jk))
     s              /2.D0
            ENDDO
         ENDDO
      ENDDO

C     Cloud attenuation.

      DO Jk=1,Nzatt
         DO Jj=1,Nyatt
            DO Ji=1,Nxatt
               DLattenuationm(Ji,Jj,Jk)=(DLattenuation(Ji,Jj,Jk)
     $              +DLattenuationf(Ji,Jj,Jk))/2.D0
            ENDDO
         ENDDO
      ENDDO

C     Relative humidity

      DO Jk=1,Nzh
         DO Jj=1,Nyh
            DO Ji=1,Nxh
               DLhumidm(Ji,Jj,Jk)=(DLhumid(Ji,Jj,Jk)+DLhumidf(Ji,Jj,Jk))
     s              /2.D0
            ENDDO
         ENDDO
      ENDDO

C     Pressure

      DO Jk=1,Nzp
         DO Jj=1,Nyp
            DO Ji=1,Nxp
               DLpressm(Ji,Jj,Jk)=(DLpress(Ji,Jj,Jk)+DLpressf(Ji,Jj,Jk))
     s              /2.D0
            ENDDO
         ENDDO
      ENDDO

C     Volumic emissions

      DO Jsp=1,Nsrc
         DO Jk=1,Nzs
            DO Jj=1,Ny
               DO Ji=1,Nx
                  DLCsourcm(Ji,Jj,Jk,Jsp)=(DLCsourc(Ji,Jj,Jk,Jsp)+
     s                 DLCsourcf(Ji,Jj,Jk,Jsp))/2.D0
               ENDDO
            ENDDO
         ENDDO
      ENDDO

C     Dry deposition velocity

      DO Jsp=1,Ndep
         DO Jj=1,Ny
            DO Ji=1,Nx
               DLVdepm(Ji,Jj,Jsp)=(DLVdep(Ji,Jj,Jsp)
     s              +DLVdepf(Ji,Jj,Jsp))/2.D0
            ENDDO
         ENDDO
      ENDDO

C     Below cloud scavenging coefficient

      DO Jsp=1,Nscav
         DO Jk=1,Nz
            DO Jj=1,Nyr
               DO Ji=1,Nxr
                  DLScavgm(Ji,Jj,Jk,Jsp)=(DLScavg(Ji,Jj,Jk,Jsp)
     s                 +DLScavgf(Ji,Jj,Jk,Jsp))/2.D0
               ENDDO
            ENDDO
         ENDDO
      ENDDO

C     Surfacic emissions

      DO Jsp=1,Nemi
         DO Jj=1,Ny
            DO Ji=1,Nx
               DLCemism(Ji,Jj,Jsp)=(DLCemis(Ji,Jj,Jsp)+
     s              DLCemisf(Ji,Jj,Jsp))/2.D0
            ENDDO
         ENDDO
      ENDDO

C     Forced chemical concentrations

      DO Jsp=1,NESPF
         DO Jj=1,Nyf
            DO Ji=1,Nxf
               DLCforc2dm(Ji,Jj,Jsp)=(DLCforc2d(Ji,Jj,Jsp)
     s              +DLCforc2df(Ji,Jj,Jsp))/2.D0
            ENDDO
         ENDDO
      ENDDO
      
C     Lateral boundary conditions for advection

      DO Jsp=1,Nspcl
         DO Jj=1,Nzl
            DO Ji=1,Nyl
               DO Jl=1,2
                  DLCclxm(Jl,Ji,Jj,Jsp)=(DLCclx(Jl,Ji,Jj,Jsp)+
     s                 DLCclxf(Jl,Ji,Jj,Jsp))/2.D0
               ENDDO
            ENDDO

            DO Ji=1,Nxl
               DO Jl=1,2
                  DLCclym(Ji,Jl,Jj,Jsp)=(DLCcly(Ji,Jl,Jj,Jsp)+
     s                 DLCclyf(Ji,Jl,Jj,Jsp))/2.D0
               ENDDO
            ENDDO

         ENDDO

         DO Jj=1,Nyl
            DO Ji=1,Nxl
               DLCclzm(Ji,Jj,Jsp)=(DLCclz(Ji,Jj,Jsp)+
     s              DLCclzf(Ji,Jj,Jsp))/2.D0  
            ENDDO
         ENDDO
      ENDDO

C     Photolytic parameters

      DO Jk=1,Nz
         DO Jj=1,Ny
            DO Ji=1,Nx
               DO Jr=1,Nrphot
                  DLCphotolysis_ratesm(Jr, Ji, Jj, Jk)=
     $                 ( DLCphotolysis_rates(Jr, Ji, Jj, Jk)
     s                 + DLCphotolysis_ratesf(Jr, Ji, Jj, Jk) ) / 2.D0
               ENDDO
            ENDDO
         ENDDO
      ENDDO
      
      END
