// Copyright (C) 2003-2004 CEREA
//     Author: Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of a simulation system for air quality.
// 
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


#include <iostream>
#include <vector>
#include <cmath>

#define SELDONDATA_DEBUG_LEVEL_4

#include "AtmoData.hxx"

using namespace std;
using namespace SeldonData;
using namespace AtmoData;


int main(int argc, char** argv)
{
  
  TRY;

  cout << endl;

  string main_config_file("bio.cfg"), sec_config_file("");

  if (argc != 2 && argc != 3 && argc != 4)
    {
      string mesg  = "Usage:\n";
      mesg += string("  ") + argv[0] + " [main configuration file] [secondary config file] [date]\n";
      mesg += string("  ") + argv[0] + " [main configuration file] [date]\n\n";
      mesg += string("  ") + argv[0] + " [date]\n\n";
      mesg += "Arguments:\n";
      mesg += "  [secondary configuration file] (optional): secondary configuration file.\n";
      mesg += "  [main configuration file] (optional): main configuration file. Default: bio.cfg\n";
      mesg += "  [date]: date.\n";
      cout << mesg << endl;
      return 1;
    }

  if (argc == 4)
    sec_config_file = argv[2];
  if (argc == 3 || argc == 4)
    main_config_file = argv[1];

  // Configuration files.
  if (!exists(main_config_file))
    throw string("Unable to find configuration file \"")
      + main_config_file + "\".";
  ConfigStreams config(main_config_file);
  if (exists(sec_config_file))
    config.AddFile(sec_config_file);


  ////////////////////////
  // FIRST DECLARATIONS //
  ////////////////////////

  typedef float real;

  // Input domain.
  int Nt, Nz, Ny, Nx;
  real Delta_t, Delta_y, Delta_x;
  real t_min, y_min, x_min;

  config.FindFromBeginning("[domain]");

  config.PeekValue("Nt", Nt);
  config.PeekValue("Nz", Nz);
  config.PeekValue("Ny", Ny);
  config.PeekValue("Nx", Nx);
  config.PeekValue("Delta_t", Delta_t);
  config.PeekValue("Delta_y", Delta_y);
  config.PeekValue("Delta_x", Delta_x);
  config.PeekValue("t_min", t_min);
  config.PeekValue("y_min", y_min);
  config.PeekValue("x_min", x_min);

  // Dates.
  int date_meteo_int;
  config.PeekValue("Date", date_meteo_int);
  Date date_meteo(date_meteo_int);
  Date date(convert<int>(argv[argc - 1]));
  int record = date.GetNumberOfDays() - date_meteo.GetNumberOfDays();

  // Files.
  string dir_out, surface_temperature_file,
    PAR_file, LUC_file, land_data_file;
  real Delta_t_bio;
  int Nc, Nt_bio;

  config.FindFromBeginning("[biogenic_files]");

  config.PeekValue("SurfaceTemperature", surface_temperature_file);
  config.PeekValue("PAR", PAR_file);
  // An extra step available for PAR?
  bool extra_step = (int(file_size(PAR_file) / (Nt * Nx * Ny * sizeof(float)))
		     > record + 1);

  config.PeekValue("LUC", LUC_file);
  // Number of land use categories.
  Nc = int(file_size(LUC_file)) / sizeof(float) / (Ny * Nx);
  config.PeekValue("land_data", land_data_file);
  config.PeekValue("directory_out", dir_out);

  config.FindFromBeginning("[biogenic]");

  config.PeekValue("Delta_t", Delta_t_bio);
  Nt_bio = int(real(Nt) * Delta_t / Delta_t_bio);

  bool save_rates;
  config.PeekValue("Rates", save_rates);


  /////////////////////////
  // METEOROLOGICAL DATA //
  /////////////////////////

  cout << "Reading and interpolating meteorological data..."; cout.flush();

  // Input fields.
  RegularGrid<real> GridT(t_min, Delta_t, Nt);
  RegularGrid<real> GridT_cumulated(t_min - Delta_t / 2.0, Delta_t,
				    extra_step ? Nt + 1 : Nt);
  RegularGrid<real> GridZ(Nz);
  RegularGrid<real> GridY(y_min, Delta_y, Ny);
  RegularGrid<real> GridX(x_min, Delta_x, Nx);
  RegularGrid<real> GridC(Nc);
  Data<real, 3> SurfaceTemperature(GridT, GridY, GridX);
  Data<real, 3> PAR(GridT_cumulated, GridY, GridX);
  Data<real, 3> LUC(GridC, GridY, GridX);

  FormatBinary<float> Meteo;
  Meteo.ReadRecord(surface_temperature_file, record, SurfaceTemperature);
  Meteo.ReadSteps(PAR_file, record * Nt, PAR);
  Meteo.Read(LUC_file, LUC);

  // Output fields.
  RegularGrid<real> GridT_out(t_min, Delta_t_bio, Nt_bio);
  Data<real, 3> SurfaceTemperature_out(GridT_out, GridY, GridX);
  Data<real, 3> PAR_out(GridT_out, GridY, GridX);

  // Temporal linear interpolation.
  LinearInterpolationDimension(SurfaceTemperature, SurfaceTemperature_out, 0);
  LinearInterpolationDimension(PAR, PAR_out, 0);
  PAR_out.ThresholdMin(0.);

  cout << "  done." << endl;
  cout << endl;


  ///////////////////
  // EMISSION DATA //
  ///////////////////

  cout << "Reading biogenic data..."; cout.flush();

  ifstream LandStream(land_data_file.c_str());
  string line;
  Data<float, 1> Density(Nc), EF_isoprene(Nc), EF_monoterpene(Nc), EF_NO(Nc);

  FormatFormattedText LandData("<c 0 54><e><e><e><e>");

  LandData.Read(land_data_file, "1", Density);
  LandData.Read(land_data_file, "2", EF_isoprene);
  LandData.Read(land_data_file, "3", EF_monoterpene);
  LandData.Read(land_data_file, "4", EF_NO);

  EF_isoprene.Mlt(1./3600.);
  EF_monoterpene.Mlt(1./3600.);
  EF_NO.Mlt(1.e-3);

  cout << " done." << endl;


  ///////////////
  // EMISSIONS //
  ///////////////

  cout << "Computing biogenic emissions..."; cout.flush();

  Data<float, 3, real> Isoprene(GridT_out, GridY, GridX);
  Data<float, 3, real> Terpene(GridT_out, GridY, GridX);
  Data<float, 3, real> NO(GridT_out, GridY, GridX);

  // Emissions without environmental correction factors.
  Data<float, 2, real> IsopreneRates(GridY, GridX);
  Data<float, 2, real> TerpeneRates(GridY, GridX);
  Data<float, 2, real> NORates(GridY, GridX);

  Isoprene.SetZero();
  Terpene.SetZero();
  NO.SetZero();

  IsopreneRates.SetZero();
  TerpeneRates.SetZero();
  NORates.SetZero();

  ComputeBiogenicRates(LUC, Density, EF_isoprene, EF_monoterpene, EF_NO,
		       IsopreneRates, TerpeneRates, NORates);
  ComputeBiogenicEmissions(SurfaceTemperature_out, PAR_out, LUC, Density,
			   EF_isoprene, EF_monoterpene, EF_NO, Isoprene,
			   Terpene, NO);

  cout << " done." << endl;

  cout << "Writing emissions..."; cout.flush();
  FormatBinary<float> EmissionsFormat;
  EmissionsFormat.Append(Isoprene, dir_out + "Isoprene.bin");
  EmissionsFormat.Append(Terpene, dir_out + "Terpene.bin");
  EmissionsFormat.Append(NO, dir_out + "NO.bin");
  if (save_rates)
    {
      EmissionsFormat.Write(IsopreneRates, dir_out + "IsopreneRates.bin");
      EmissionsFormat.Write(TerpeneRates, dir_out + "TerpeneRates.bin");
      EmissionsFormat.Write(NORates, dir_out + "NORates.bin");
    }
  cout << " done." << endl;


  //////////////
  // ANALYSIS //
  //////////////

  cout << "\n-- Analysis --\n" << endl;

  DISP(Isoprene.GetMax());
  DISP(Isoprene.Mean());
  cout << endl;
  DISP(Terpene.GetMax());
  DISP(Terpene.Mean());
  cout << endl;
  DISP(NO.GetMax());
  DISP(NO.Mean());

  cout << endl;

  END;

  return 0;

}
