// Copyright (C) 2003-2004 CEREA
//     Authors: Chi-Sian Soh and Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of a simulation system for air quality.
// 
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


//////////////
// INCLUDES //

#include <cmath>
#include <iostream>
#include <algorithm>
#include <string>
using namespace std;

#define SELDONDATA_DEBUG_LEVEL_4

#include "SeldonData.hxx"
using namespace SeldonData;

#include "AtmoData.hxx"
using namespace AtmoData;

// INCLUDES //
//////////////


template<class T>
void Cut(T& x)
{
  x = max(T(0), x);
  x = min(T(1e+30), x);
}

// Sets infinite resistance values to 1e+30.
template<class T>
void Infinity(T& x)
{
  if (x==T(9999))
    x = T(1e+30);
}


int main(int argc, char** argv)
{
  TRY;
  
  cout << endl;



  string main_config_file("dep.cfg"), sec_config_file("");

  if (argc != 2 && argc != 3 && argc != 4)
    {
      string mesg  = "Usage:\n";
      mesg += string("  ") + argv[0] + " [main configuration file] [secondary config file] [date]\n";
      mesg += string("  ") + argv[0] + " [main configuration file] [date]\n";
      mesg += string("  ") + argv[0] + " [date]\n\n";
      mesg += "Arguments:\n";
      mesg += "  [main configuration file] (optional): main configuration file. Default: dep.cfg\n";
      mesg += "  [secondary configuration file] (optional): secondary configuration file.\n";
      mesg += "  [date]: date in format YYYYMMDD.\n";
      cout << mesg << endl;
      return 1;
    }

  if (argc == 4)
    sec_config_file = argv[2];
  if (argc == 3 || argc == 4)
    main_config_file = argv[1];

  // Configuration files.
  if (!exists(main_config_file))
    throw string("Unable to find configuration file \"")
      + main_config_file + "\".";
  ConfigStreams configuration(main_config_file);
  if (exists(sec_config_file))
    configuration.AddFile(sec_config_file);


  ////////////////////////
  // FIRST DECLARATIONS //
  ////////////////////////

  typedef float real;

  // Units: SI.
  const real pi = 3.14159265358979323846264;
  const real Karman = 0.4;
  const real nu = 0.15;
  const real D_H2O = 0.25;
  const real Pr = 0.74;
  const real M_vap = 18.015e-3;
  const real M_air = 28.97e-3;

  int h, i, j, k, l, s;

  // Input and output directories.
  string dir_in, dir_out;
  // Input dimensions for meteological data.
  int Nt_in, Nz_in, Ny_in, Nx_in;
  // Input-data steps.
  real Delta_t_in, Delta_y_in, Delta_x_in;
  real t_min_in, y_min_in, x_min_in;
  // Polair Inputs
  string SurfaceTemperature_file, SurfaceRichardson_file, SolarRadiation_file,
    WindModule_file, U_file, V_file, PARdiff_file, PARdir_file,
    SpecificHumidity_file, SurfacePressure_file, FrictionVelocity_file,
    CanopyWetness_file, Rain_file, RoughnessHeight_file;
  // Starting date.
  string starting_date_str;

  // Output dimensions.
  int Nt_out, Nz_out, Ny_out, Nx_out;
  // Output steps.
  real Delta_t_out, Delta_y_out, Delta_x_out;
  real t_min_out, y_min_out, x_min_out;
  // Output altitudes.
  string levels;

  // Type of landuse classification.
  string landuse_type;
  // Landuse file.
  string LUC_file;
  // Number of LUC categories.
  int Nc;
  // Land Data Files
  string midsummer, autumn, late_autumn, snow, spring;

  // Number of species.
  int Ns_out;
  // Species data.
  string species_data;

  // Options.
  string option_Ra, option_Rb, option_Rc;
  bool option_save_resistances, option_Roughness;


  /////////////////////////
  // CONFIGURATION FILES //
  /////////////////////////

  cout << "Reading configuration files..."; cout.flush(); 
 
  configuration.Find("[domain]");

  // Dates.
  int date_meteo_int;
  configuration.PeekValue("Date", date_meteo_int);
  Date date_meteo(date_meteo_int);
  Date date(convert<int>(argv[argc - 1]));
  int record = date.GetNumberOfDays() - date_meteo.GetNumberOfDays();

  configuration.PeekValue("Nt", Nt_in);
  configuration.PeekValue("Nz", Nz_in);
  configuration.PeekValue("Ny", Ny_in);
  configuration.PeekValue("Nx", Nx_in);

  configuration.PeekValue("Delta_t", Delta_t_in);
  configuration.PeekValue("Delta_y", Delta_y_in);
  configuration.PeekValue("Delta_x", Delta_x_in);

  configuration.PeekValue("t_min", t_min_in);
  configuration.PeekValue("y_min", y_min_in);
  configuration.PeekValue("x_min", x_min_in);
  configuration.PeekValue("vertical_levels", levels);

  Nt_out = Nt_in;
  Nz_out = Nz_in;
  Ny_out = Ny_in;
  Nx_out = Nx_in;
  Delta_t_out = Delta_t_in;
  Delta_y_out = Delta_y_in;
  Delta_x_out = Delta_x_in;
  t_min_out = t_min_in;
  y_min_out = y_min_in;
  x_min_out = x_min_in;

  // Input/output files.
  configuration.FindFromBeginning("[Files]");

  configuration.PeekValue("directory_out", dir_out);
  
  configuration.PeekValue("SurfaceTemperature", SurfaceTemperature_file);
  configuration.PeekValue("SurfaceRichardson", SurfaceRichardson_file);
  configuration.PeekValue("SolarRadiation", SolarRadiation_file);
  // An extra step available for accumulated data?
  bool extra_step = (int(file_size(SolarRadiation_file)
			 / (Nt_in * Nx_in * Ny_in * sizeof(float)))
		     > record + 1);

  configuration.PeekValue("WindModule", WindModule_file);
  configuration.PeekValue("ZonalWind", U_file);
  configuration.PeekValue("MeridionalWind", V_file);
  configuration.PeekValue("PARdiff", PARdiff_file);
  configuration.PeekValue("PARdir", PARdir_file);
  configuration.PeekValue("SpecificHumidity", SpecificHumidity_file);
  configuration.PeekValue("SurfacePressure", SurfacePressure_file);
  configuration.PeekValue("FrictionVelocity", FrictionVelocity_file);
  configuration.PeekValue("CanopyWetness", CanopyWetness_file);
  configuration.PeekValue("Rain", Rain_file);
  configuration.PeekValue("RoughnessHeight", RoughnessHeight_file);

  // LUC.
  configuration.FindFromBeginning("[LUC]");

  configuration.PeekValue("Type", landuse_type);

  // Species.
  configuration.FindFromBeginning("[Species]");

  configuration.PeekValue("Ns", Ns_out);
  configuration.PeekValue("Data", species_data);

  // Options.
  configuration.Find("[Options]");

  configuration.PeekValue("CellRoughness", option_Roughness);
  configuration.PeekValue("Ra", option_Ra);
  configuration.PeekValue("Rb", option_Rb);
  configuration.PeekValue("Rc", option_Rc);
  configuration.PeekValue("save_resistances", option_save_resistances);

  // Reads the configuration file associated with the landuse type.
  ConfigStream classification(landuse_type);

  classification.PeekValue("Nc", Nc);
    
  classification.PeekValue("Midsummer", midsummer);
  classification.PeekValue("Autumn", autumn);
  classification.PeekValue("Late_autumn", late_autumn);
  classification.PeekValue("Snow", snow);
  classification.PeekValue("Spring", spring);

  classification.PeekValue("File", LUC_file);

  cout << " done." << endl;
  cout << endl;


  ///////////
  // GRIDS //
  ///////////

  cout << "Memory allocation for data fields..."; cout.flush();

  // Input settings.

  // Input grids.
  RegularGrid<real> GridT_in(t_min_in, Delta_t_in, Nt_in);
  RegularGrid<real> GridT_cumulated_in(t_min_in - Delta_t_in / 2.0,
				       Delta_t_in,
				       extra_step ? Nt_in + 1 : Nt_in);
  RegularGrid<real> GridZ_in(Nz_in);
  RegularGrid<real> GridY_in(y_min_in, Delta_y_in, Ny_in);
  RegularGrid<real> GridX_in(x_min_in, Delta_x_in, Nx_in);
  RegularGrid<real> GridY_interf_in(y_min_in - Delta_y_in / 2.,
				    Delta_y_in, Ny_in + 1);
  RegularGrid<real> GridX_interf_in(x_min_in - Delta_x_in / 2.,
				    Delta_x_in, Nx_in + 1);
  RegularGrid<real> GridC(Nc);
  
  // Output settings.

  // Output grids.
  RegularGrid<real> GridT_out(t_min_out, Delta_t_out, Nt_out);
  RegularGrid<real> GridZ_out(Nz_out);
  RegularGrid<real> GridZ_interf_out(Nz_out + 1);
  RegularGrid<real> GridY_out(y_min_out, Delta_y_out, Ny_out);
  RegularGrid<real> GridX_out(x_min_out, Delta_x_out, Nx_out);

  // Species.
  RegularGrid<real> GridS(Ns_out);

  // Reads output altitudes.
  FormatText Heights_out;
  Heights_out.Read(levels.c_str(), GridZ_interf_out);
  // Sets values at nodes.
  for (k=0; k<Nz_out; k++)
    GridZ_in(k) = GridZ_out(k) = ( GridZ_interf_out(k)
                                   + GridZ_interf_out(k+1) ) / 2.0;


  //////////
  // DATA //
  //////////

  // Input fields.
  Data<real, 3> LandUse(GridC, GridY_in, GridX_in);
  Data<real, 2> RoughnessHeight_cell(GridY_in, GridX_in);

  Data<real, 3> SurfaceRichardson(GridT_in, GridY_in, GridX_in);
  Data<real, 4> ZonalWind(GridT_in, GridZ_in, GridY_in, GridX_interf_in);
  Data<real, 4> MeridionalWind(GridT_in, GridZ_in, GridY_interf_in, GridX_in);
  Data<real, 4> WindModule(GridT_in, GridZ_in, GridY_in, GridX_in);
  Data<real, 3> SolarRadiation(GridT_cumulated_in, GridY_in, GridX_in);
  Data<real, 3> SurfaceTemperature(GridT_in, GridY_in, GridX_in);
  Data<real, 3> PARdiff(GridT_cumulated_in, GridY_in, GridX_in);
  Data<real, 3> PARdir(GridT_cumulated_in, GridY_in, GridX_in);
  Data<real, 4> SpecificHumidity(GridT_in, GridZ_in, GridY_in, GridX_in);     
  Data<real, 3> SurfacePressure(GridT_in, GridY_in, GridX_in);
  Data<real, 3> FrictionVelocity(GridT_in, GridY_in, GridX_in);
  Data<real, 3> CanopyWetness(GridT_in, GridY_in, GridX_in);
  Data<real, 3> Rain(GridT_cumulated_in, GridY_in, GridX_in);
 
  Data<string, 1, real> SpeciesNames(GridS);
  Data<real, 1> MolarMass(GridS);
  Data<real, 1> Henry(GridS);
  Data<real, 1> Diffusivity(GridS);
  Data<real, 1> Reactivity(GridS);
  Data<real, 1> Alpha(GridS);
  Data<real, 1> Beta(GridS);
  Data<real, 1> Rm(GridS);
  
  Data<real, 1> LeafLength(GridC);
  Data<real, 1> rst_min(GridC);
  Data<real, 1> brs(GridC);
  Data<real, 1> T_min(GridC);
  Data<real, 1> T_max(GridC);
  Data<real, 1> T_opt(GridC);
  Data<real, 1> b_vpd(GridC);
  Data<real, 1> psi_c1(GridC);
  Data<real, 1> psi_c2(GridC);
  Data<real, 1> H(GridC);
  Data<real, 1> LAI(GridC);
  Data<real, 1> RoughnessHeight(GridC);
  Data<real, 1> ZeroDisplacementHeight(GridC);
  Data<real, 1> Rg_SO2(GridC);
  Data<real, 1> Rg_O3(GridC);
  Data<real, 1> Rc_srf_SO2(GridC);
  Data<real, 1> Rc_srf_O3(GridC);
  Data<real, 1> R_ac0(GridC);
  Data<real, 1> Ri(GridC);
  Data<real, 1> Rlu(GridC);
  Data<real, 1> WithVegetation(GridC);
  Data<real, 1> TallVegetation(GridC);
    
  // Output fields.
  Data<real, 3> Velocities(GridT_out, GridY_out, GridX_out);

  Data<real, 4> DRa(GridT_out, GridY_out, GridX_out, GridC);
  Data<real, 4> DRb(GridT_out, GridY_out, GridX_out, GridC);
  Data<real, 4> DRc(GridT_out, GridY_out, GridX_out, GridC);

  Data<real, 3> SurfaceRelativeHumidity(GridT_in, GridY_in, GridX_in);

  cout << " done." << endl;
  cout << endl;


  /////////////////
  // READS INPUT //
  /////////////////

  cout << "Extracting input data..."; cout.flush();

  FormatText InputParam;
  
  // Species data.
  ifstream SpeciesData(species_data.c_str());
  for (i = 0; i < Ns_out; i++)
    SpeciesData >> SpeciesNames(i);
  InputParam.Read(SpeciesData, MolarMass);
  InputParam.Read(SpeciesData, Henry);
  InputParam.Read(SpeciesData, Diffusivity);
  InputParam.Read(SpeciesData, Reactivity);
  InputParam.Read(SpeciesData, Alpha);
  InputParam.Read(SpeciesData, Beta);
  InputParam.Read(SpeciesData, Rm);
  SpeciesData.close();

  // Land use resistances and roughness heights.
  ifstream LandData;
  int MM = date.GetMonth();
  if ( MM == 5 || MM == 6 || MM == 7 || MM == 8 )
    LandData.open(midsummer.c_str());
  else if ( MM == 9 || MM == 10)
    LandData.open(autumn.c_str());
  else if ( MM == 11 || MM == 12 || MM == 1 || MM == 2 )
    LandData.open(late_autumn.c_str());
  else if ( MM == 3 || MM == 4 )
    LandData.open(spring.c_str());
  else
    LandData.open(snow.c_str());

  InputParam.Read(LandData, LeafLength);
  InputParam.Read(LandData, rst_min);
  InputParam.Read(LandData, brs);
  InputParam.Read(LandData, T_min);
  InputParam.Read(LandData, T_max);
  InputParam.Read(LandData, T_opt);
  InputParam.Read(LandData, b_vpd);
  InputParam.Read(LandData, psi_c1);
  InputParam.Read(LandData, psi_c2);
  InputParam.Read(LandData, H);
  InputParam.Read(LandData, LAI);
  InputParam.Read(LandData, RoughnessHeight);
  InputParam.Read(LandData, ZeroDisplacementHeight);
  InputParam.Read(LandData, Rg_SO2);
  InputParam.Read(LandData, Rg_O3);
  InputParam.Read(LandData, Rc_srf_SO2);
  InputParam.Read(LandData, Rc_srf_O3);
  InputParam.Read(LandData, R_ac0);
  InputParam.Read(LandData, Ri);
  InputParam.Read(LandData, Rlu);
  InputParam.Read(LandData, WithVegetation);
  InputParam.Read(LandData, TallVegetation);
  LandData.close();

  LeafLength.Apply(Infinity<real>);
  rst_min.Apply(Infinity<real>);
  brs.Apply(Infinity<real>);
  T_min.Apply(Infinity<real>);
  T_max.Apply(Infinity<real>);
  T_opt.Apply(Infinity<real>);
  b_vpd.Apply(Infinity<real>);
  psi_c1.Apply(Infinity<real>);
  psi_c2.Apply(Infinity<real>);
  H.Apply(Infinity<real>);
  LAI.Apply(Infinity<real>);
  Rg_SO2.Apply(Infinity<real>);
  Rg_O3.Apply(Infinity<real>);
  Rc_srf_SO2.Apply(Infinity<real>);
  Rc_srf_O3.Apply(Infinity<real>);
  R_ac0.Apply(Infinity<real>);
  Ri.Apply(Infinity<real>);
  Rlu.Apply(Infinity<real>);
  
  // Polair inputs.
  FormatBinary<float> InputPolair;

  InputPolair.Read(LUC_file, LandUse);
  InputPolair.Read(RoughnessHeight_file, RoughnessHeight_cell);

  InputPolair.ReadRecord(SurfaceTemperature_file, record, SurfaceTemperature);
  InputPolair.ReadRecord(SurfaceRichardson_file, record, SurfaceRichardson);
  InputPolair.ReadSteps(SolarRadiation_file, record * Nt_in, SolarRadiation);
  InputPolair.ReadRecord(WindModule_file, record, WindModule);
  InputPolair.ReadSteps(PARdiff_file, record * Nt_in, PARdiff);
  InputPolair.ReadSteps(PARdir_file, record * Nt_in, PARdir);
  InputPolair.ReadRecord(SpecificHumidity_file, record, SpecificHumidity);
  InputPolair.ReadRecord(SurfacePressure_file, record, SurfacePressure);
  InputPolair.ReadRecord(FrictionVelocity_file, record, FrictionVelocity);
  InputPolair.ReadRecord(CanopyWetness_file, record, CanopyWetness);
  InputPolair.ReadSteps(Rain_file, record * Nt_in, Rain);
  InputPolair.ReadRecord(U_file, record, ZonalWind);
  InputPolair.ReadRecord(V_file, record, MeridionalWind);

  cout << " done." << endl;
  cout << endl;
  

  /////////////////////////
  // METEOROLOGICAL DATA //
  /////////////////////////
  
  cout << "Computing meteorological fields..."; cout.flush();
  
  SurfaceTemperature.Add(-273.15);

  // Calculation of relative humidity.
  real P_sat_vap, P_vap;
  for (h=0; h<Nt_out; h++)
    for (j=0; j<Ny_out; j++)
      for (i=0; i<Nx_out; i++)
        {
          P_sat_vap = 611.2 * exp(17.67 * SurfaceTemperature(h, j, i)
                                  / (SurfaceTemperature(h, j, i) + 243.50));
          P_vap = SpecificHumidity(h, 0, j, i) * SurfacePressure(h, j, i)
            / ( SpecificHumidity(h, 0, j, i) +
		(1 - SpecificHumidity(h, 0, j, i)) * (M_vap / M_air) );
          SurfaceRelativeHumidity(h, j, i) = P_vap / P_sat_vap;
        }
  SurfaceRelativeHumidity.Threshold(0.0, 1.0);

  CanopyWetness.SetZero();

  // PAR to W.m^-2.
  PARdiff.Mlt(1./4.6);
  PARdir.Mlt(1./4.6);

  cout << " done." << endl;
  cout << endl;

  
  ///////////////////////////
  // DEPOSOTION VELOCITIES //
  ///////////////////////////
  
  cout << "Computing deposition velocities..."; cout.flush();
  cout << endl;
  
  real Rsx, Rmx, Rlux, Rdc, Rclx, Rgx;
  real r0, r1, wind(0.), friction_wind, tmp;
  real Gs, f_T, f_D, f_psi, Wst(0.), bt, D, e_sat, e_amb, psi;
  real lon, lat, ut, alpha, theta;
  real rst_sun, rst_shade, PAR_shade, PAR_sun, F_sun, F_shade, R_dir, R_diff;
  real Rcut, Rg, Rst, Rns, Rac;
  real stab, rich, Cd(0.), Ra, Rb, Rc(0.);
  string surface_condition;
  real Sc;
  real zszo, zszot, zot, al_zszo, al_zszot, a2, alt, alu, c, drib;
  real fm(0.), fh(0.);
  real b1 = 5., c1 = 5., d1 = 5.;
  real roughness_height;
   
  for (s=0; s<Ns_out; s++)
    {
      Velocities.SetZero();
      Sc = nu / Diffusivity(s);
      for (h=0; h<Nt_out; h++)
	for (j=0; j<Ny_out; j++)
	  for (i=0; i<Nx_out; i++)
	    {
	      
	      // Surface condition.
	      if ( CanopyWetness(h, j, i) <= 0.1 && 
		   SurfaceRelativeHumidity(h, j, i) < 0.8)
		surface_condition = "dry";
	      else if ( CanopyWetness(h, j, i) <= 0.1 &&
			SurfaceRelativeHumidity(h, j, i) >= 0.9 )
		surface_condition = "high humidity";
	      else if ( CanopyWetness(h, j, i) > 0.8 )
		surface_condition = "dew";
	      else if ( CanopyWetness(h, j, i) > 0.8 && Rain(h, j, i) > 0.01 )
		surface_condition = "rain";
	      else
		surface_condition = "dry";

	      // Loop for the different LUCs.
	      for (l=0; l<Nc; l++)
		{		
		  if (option_Roughness)
		    roughness_height = RoughnessHeight_cell(j, i);
		  else
		    roughness_height = RoughnessHeight(l);
		  

		  /////////////
		  // Ra & Rb //
		  /////////////
		  
		  if (option_Ra == "diag" || option_Rb == "diag")
		    {
		      // Stability function.
		      r0 = log(25. / roughness_height);
		      r0 = Karman * Karman / (r0 * r0);
		      r1 = 9.4 * 7.4 * r0 * sqrt(25. / roughness_height);
		      rich = SurfaceRichardson(h, j, i);
		      if (rich < 0.0)
			stab = 1. - 9.4 * rich / (1. + r1 * sqrt(-rich));
		      else
			stab = 1. / (pow(1. + 4.7 * rich, 2.0));
		      // Drag coefficient Cd.
		      Cd = r0 * stab;
		      // Wind module.
		      wind = max(WindModule(h, 0, j, i), 0.1);
		    }
		  
		  
		  ////////
		  // Ra //
		  ////////
		  
		  if (option_Ra != "diag")
		    {
		      zszo = (GridZ_in(0) + roughness_height)
			/ roughness_height;
		      zot = roughness_height / 10;
		      zszot = (GridZ_in(0) + zot) / zot;
		      al_zszo = log(zszo);
		      al_zszot = log(zszot);
		      alu = Karman / al_zszo;
		      alt = Karman / al_zszot;
		      
		      if (SurfaceRichardson(h, j, i) > 0)
			{
			  drib = sqrt(1. + d1 * SurfaceRichardson(h, j, i));
			  if (option_Ra == "fh")
			    fh = 1. / (1. + 3. * b1
				       * SurfaceRichardson(h, j, i) * drib);
			  else
			    fm = 1. / (1. + 2. * b1
				       * SurfaceRichardson(h, j, i) / drib);
			}
		      else
			{
			  c = alu * alt * b1 * c1 * sqrt(1. - 1./zszot)
			    * pow(double(pow(double(zszot), double(1./3.))
					 - 1.), double(3./2.));
			  drib = 1. + 3. * c
			    * sqrt(-SurfaceRichardson(h, j, i));
			  if (option_Ra == "fh")
			    fh = 1. - 3. * b1 * SurfaceRichardson(h, j, i)
			      / drib;
			  else
			    fm = 1. - 2. * b1 * SurfaceRichardson(h,j, i)
			      / drib;
			}
		      
		      if (option_Ra == "fh")
			Ra = 1. / (alu * alt * WindModule(h, 0, j, i) * fh);
		      else
			{
			  a2 = (Karman * Karman) / (al_zszo * al_zszo);
			  Ra = 1. / (a2 * WindModule(h, 0, j, i) * fm);
			}
		    }
		  else // old parameterization.
		    // Aerodynamic resistance.
		    Ra = 1. / (wind * Cd);
		  
		  
		  ////////
		  // Rb //
		  ////////
		  
		  if (option_Rb == "friction")
		    {
		      if (WithVegetation(l) == 1)
			Rb = 2. / (Karman * FrictionVelocity(h, j, i))
			  * pow(double(Sc/Pr), double(2./3.));
		      else
		        Rb = 1. / (Karman * FrictionVelocity(h, j, i))
			  * pow(double(Sc/Pr), double(2./3.));
		    }
		  else
		    {
		      // Friction wind.
		      friction_wind = wind * sqrt(Cd);
		      // 
		      tmp = sqrt(MolarMass(s)/18.);
		      tmp = 2. * float(pow(double((nu / (D_H2O * Pr)) * tmp),
					   double(2./3.))) / Karman;
		      // Quasilaminary sublayer resistance.
		      Rb = tmp / friction_wind;
		    }
		  
		  
		  ////////
		  // Rc //
		  ////////
		  
		  if (option_Rc == "zhang")
		    {
		      /*** Fraction of stomatal blocking, Wst ***/
		      if ( surface_condition == "dry" ||
			   surface_condition == "high humidity" )
			Wst = 0;
		      else if ( SolarRadiation(h, j, i) <= 200 )
			Wst = 0.;
		      else if ( SolarRadiation(h, j, i) > 200 &&
				SolarRadiation(h, j, i) <= 600 )
			Wst = (SolarRadiation(h, j, i) - 200.) / 800.;
		      else if ( SolarRadiation(h, j, i) > 600 )
			Wst = 0.5;
		      
		      
		      /*** Stomatal Resistance, Rst ***/
		      
		      if (rst_min(l) == real(1e+30) || brs(l) == real(1e+30)
			  || T_min(l) == real(1e+30) || T_max(l) == real(1e+30)
			  || T_opt(l) == real(1e+30) || b_vpd(l) == real(1e+30)
			  || psi_c1(l) == real(1e+30) || psi_c2(l) == real(1e+30)
			  || LAI(l) == real(1e+30))
			Rst = 1e+30;
		      else if (SolarRadiation(h, j, i) == 0)
			Rst = 1e+30;
		      else if (SurfaceTemperature(h, j,i) < T_min(l)
			       || SurfaceTemperature(h, j, i) > T_max(l))
			Rst = 1e+30;
		      else
			{
			  // Unstressed canopy stomatal conductance.
			  lon = x_min_out + i * Delta_x_out;
			  lat = y_min_out + j * Delta_y_out;
			  ut = t_min_in + h * Delta_t_in
			    - real(int(t_min_in + h * Delta_t_in) / 24) * 24.;
			  theta = ZenithAngle(lon, lat, date.GetDate(), ut)
			    / 180. * pi;
			  alpha = pi / 3.;
			  
			  if (theta > pi / 2.)
			    {
			      F_sun = 0.;
			      F_shade = 0.;
			    }
			  else
			    {
			      F_sun = 2. * cos(theta)
				* (1. -  exp(-0.5 * LAI(l) / cos(theta)));
			      F_shade = LAI(l) - F_sun;
			    }
			  
			  // R_dir and R_diff.
			  R_diff = PARdiff(h, j, i);
			  R_dir = PARdir(h, j, i);
			  
			  if (LAI(l) < 2.5 || SolarRadiation(h, j, i) < 200.)
			    {
			      PAR_shade = R_diff
				* exp(-0.5 * pow(double(LAI(l)), 0.7))
				+ 0.07 * R_dir * (1.1 - 0.1 * LAI(l))
				* exp(-cos(theta));
			      PAR_sun = R_dir * cos(alpha) / cos(theta)
				+ PAR_shade;
			    }
			  else 
			    {
			      PAR_shade = R_diff
				* exp(-0.5 * pow(double(LAI(l)), 0.8))
				+ 0.07 * R_dir * (1.1 - 0.1 * LAI(l))
				* exp(-cos(theta));
			      PAR_sun = pow(double(R_dir), 0.8) * cos(alpha) 
				/ cos(theta) + PAR_shade;
			    }
			  rst_sun = rst_min(l) * (1. + brs(l) / PAR_sun);
			  rst_shade = rst_min(l) * (1. + brs(l) / PAR_shade);
			  
			  Gs = F_sun / rst_sun + F_shade / rst_shade;
			  
			  // Conductance-reducing effects of air temperature.
			  bt = (T_max(l) - T_opt(l)) / (T_opt(l) - T_min(l));
			  f_T = (SurfaceTemperature(h, j, i) - T_min(l)) /
			    (T_opt(l) - T_min(l))
			    * pow((T_max(l) - SurfaceTemperature(h, j, i)) /
				  (T_max(l) - T_opt(l)), bt);
			  
			  // Conductance-reducing effects
			  // of water vapour deficit.
			  e_amb = SpecificHumidity(h, 0, j, i)
			    * SurfacePressure(h, j, i)
			    / (0.622 * (1. - SpecificHumidity(h, 0, j, i))
			       + SpecificHumidity(h, 0, j, i));
			  e_sat = 610.78 * exp(17.2694
					       * SurfaceTemperature(h, j, i)
					       / (SurfaceTemperature(h, j, i)
						  + 237.29));
			  D = (e_sat - e_amb) / 1000.;
			  f_D = 1. - b_vpd(l) * D;
			  
			  // Conductance-reducing effects of water stress.
			  if (TallVegetation(l) == l)
			    psi = (-0.72 - 0.0013 * SolarRadiation(h, j, i))
			      * 102.;
			  else
			    psi = (-0.395 - 0.043
				   * SurfaceTemperature(h, j, i)) * 102.;
			  
			  if (psi > psi_c1(l))
			    f_psi = 1.;
			  else
			    f_psi = (psi - psi_c2(l))
			      / (psi_c1(l) - psi_c2(l));
			  
			  Rst = 1. / (Gs * f_T * f_D * f_psi
				      * Diffusivity(s)/D_H2O);
			}
		      Cut(Rst);
		      
		      
		      /*** Ground resistance Rg ***/
		      
		      if (SpeciesNames(s)=="O3")
			Rg = Rg_O3(l);
		      else if (SpeciesNames(s)=="SO2")
			Rg = Rg_SO2(l);
		      else
			Rg = 1. / (Alpha(s) / Rg_SO2(l) + Beta(s) / Rg_O3(l));
		      if (SurfaceTemperature(h, j, i) < -1.)
			Rg = Rg * exp(0.2
				      * (-1. - SurfaceTemperature(h, j, i)));
		      Cut(Rg);
		      
		      
		      /*** Local leaf cuticular resistance Rcut ***/
		      
		      if (SpeciesNames(s)=="O3")
			Rcut = Rc_srf_O3(l);
		      else if (SpeciesNames(s)=="SO2")
			Rcut = Rc_srf_SO2(l);
		      else
			Rcut = 1. / (Alpha(s) / Rc_srf_SO2(l) + Beta(s)
				     / Rc_srf_O3(l));
		      if (SurfaceTemperature(h, j, i) < -1)
			Rcut = Rcut
			  * exp(0.2 * (-1 - SurfaceTemperature(h, j, i)));
		      Cut(Rcut);
		      
		      
		      /*** In canopy aerodymanic resistance Rac ***/
		      
		      Rac = R_ac0(l) * pow(double(LAI(l)), 0.25)
			/ pow(double(FrictionVelocity(h, j, i)), 2.);
		      Cut(Rac);
		      
		      
		      /*** Rc ***/
		      
		      Rns = 1. / (1./(Rac + Rg) + 1./Rcut);

		      Rc = 1. / ((1. - Wst)/(Rst + Rm(s)) + 1./Rns);
		      
		    }
		  else if (option_Rc == "wesely")
		    {
		      /*** Stomatal resistance ***/
		      if ( (SurfaceTemperature(h, j, i)<0.01) ||
			   (SurfaceTemperature(h, j, i)>39.99))
			Rsx = 1e+30;
		      else
			Rsx = Ri(l) * (1. + 40000.
				       /float(pow(double(SolarRadiation(h, j, i)
							 + 0.1), 2.0)))
			  * (400. / (SurfaceTemperature(h, j, i)
				     * (40. - SurfaceTemperature(h, j, i))));
		      Rsx = Rsx * D_H2O / Diffusivity(s);
		      
		      /*** Mesophyll resistance ***/
		      Rmx = 1. / (Henry(s)/3000. + 100.*Reactivity(s));
		      
		      /*** Cuticle resistance ***/
		      Rlux = Rlu(l) / (1.e-5 * Henry(s) + Reactivity(s));
		      
		      /*** Buoyant-convection resistance ***/
		      Rdc = 100. * (1. + 1000. /
				    (SolarRadiation(h, j, i) + 10.));
		      
		      /*** Rclx ***/
		      if (SpeciesNames(s)=="O3")
			Rclx = Rc_srf_O3(l);
		      else if (SpeciesNames(s)=="SO2")
			Rclx = Rc_srf_SO2(l);
		      else
			Rclx = 1. / (Alpha(s) / Rc_srf_SO2(l)
				     + Beta(s) / Rc_srf_O3(l));
		      
		      /*** Rgx ***/
		      if (SpeciesNames(s)=="O3")
			Rgx = Rg_O3(l);
		      else if (SpeciesNames(s)=="SO2")
			Rgx = Rg_SO2(l);
		      else
			Rgx = 1. / (Alpha(s) / Rg_SO2(l)
				    + Beta(s) / Rg_O3(l));
		      
		      /*** Rc ***/
		      Cut(Rsx); Cut(Rmx); Cut(Rlux);
		      Cut(Rdc); Cut(Rclx); Cut(Rgx);

		      Rc = 1. / (1./(Rsx + Rmx) + 1./Rlux
				 + 1./(Rdc + Rclx) + 1./(R_ac0(l) + Rgx));
		    }
		  
		  
		  if (Rc > 9999)
		    Rc = 9999.;
		  else if (Rc < 10)
		    Rc = 10.;
		  
		  
		  /////////////////////////
		  // DEPOSITION VELOCITY //
		  /////////////////////////
		  
		  Velocities(h, j, i) += LandUse(l, j, i) / (Rc + Rb + Ra);
		  
		  DRa(h, j, i, l) = Ra;
		  DRb(h, j, i, l) = Rb;
		  DRc(h, j, i, l) = Rc;
		}
	    }
      
      
      // Writes output files.
      FormatBinary<float> PolairOut;
      PolairOut.Append(Velocities, dir_out + SpeciesNames(s) + ".bin");
      cout << "Done for " << SpeciesNames(s) << endl;
      
      if (option_save_resistances)
        {
          PolairOut.Append(DRa, dir_out + "Ra.bin");
          PolairOut.Append(DRb, dir_out + "Rb.bin");
          PolairOut.Append(DRc, dir_out + "Rc.bin");
        }
     
    }
  
  cout << " done." << endl;
  
  cout << endl;
  
  END;
  
  return 0;
  
}
