// Copyright (C) 2003-2004 CEREA
//     Authors: Denis Qulo and Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of a simulation system for air quality.
// 
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


/////////////////////////
// EMEPCOUNTRYEMISSION //

template <class T>
class EmepCountryEmission
{
public:
  T emission_;
  int country_;
public:
  EmepCountryEmission(T emission, int country):
    emission_(emission), country_(country) {}
};

// EMEPCOUNTRYEMISSION //
/////////////////////////


//////////////
// TIMEZONE //

class TimeZone
{
public:
  vector<int> countries_;
  vector<int> local_times_;
  TimeZone(int N): countries_(N), local_times_(N)
  {
    for (vector<int>::iterator iter = countries_.begin();
	 iter != countries_.end(); ++iter)
      {
	*iter = iter - countries_.begin();
	local_times_.at(iter - countries_.begin()) = 1;
      }
  }
  void Init(string file_name)
  {
    int country, time;
    ExtStream file_stream(file_name);
    while (!file_stream.IsEmpty())
      {
	file_stream.GetElement();
	file_stream.GetElement(country);
	file_stream.GetElement(time);
	if (country < int(local_times_.size()))
	  local_times_.at(country) = time;
	file_stream.GetLine();
      }
  }
  int operator () (int i) const
  {
    return local_times_.at(i);
  }
};

// TIMEZONE //
//////////////


////////////////////////
// GETTEMPORALFACTORS //

//! Reads monthly profiles for a given month or daily profiles of a week.
template <class real>
void GetTemporalFactors(string input_file, Data<real, 3>& Factors)
{
  
  string line;
  int s, Ncountry;

  Factors.Fill(1.0);
  
  ConfigStream input_stream(input_file);
  if (!input_stream.is_open())
    throw string("Error in GetTemporalFactors: \"") + input_file
      + "\" cannot be opened.";
  
  while (input_stream.GetLine(line))
    {
      vector<real> factors;
      split(line, factors);

      if (int(factors.size()) != 2 + Factors.GetLength(2))
	throw string("Error in GetTemporalFactors: \"") + input_file
	  + "\" badly formatted.";

      Ncountry = int(factors[0]);
      s = int(factors[1]);

      if (s - 1 < Factors.GetLength(1))
	for(int i = 0; i < Factors.GetLength(2); i++)
	  Factors(Ncountry, s - 1, i) = factors[i + 2];
    }
  
}
  
// GETTEMPORALFACTORS //
////////////////////////


///////////////////////////
// SPECIATIONAGGREGATION //

//! Computes speciation/aggregation factors.
template <class real>
void SpeciationAggregation(vector<string>& Sp_emis_names,
			   vector<string>& Sp_model_names,
			   string aggregation_matrix,
			   string speciation_directory,
			   Data<real, 3>& Species_factor)
{

  int Nsectors = Species_factor[2].GetLength();
  int Nsp_emis = Species_factor[1].GetLength();
  int Nsp_model = Species_factor[0].GetLength();
  
  string Sp_real_names;
  vector<real> molecular_weights_model(Nsp_model);
  vector<real> React_model(Nsp_model);
  
  string input_file, line, stmp;
  string species_names;
  vector<real> vtmp;
  
  /** Reads model species names, molecular weight and reactivity ***/

  ExtStream aggregation_stream(aggregation_matrix);
  if (!aggregation_stream.is_open())
    throw string(" File ") + input_file + " doesn't exist.";

  // Species names.
  aggregation_stream.SkipElements(3);
  aggregation_stream.GetLine(line);
  split(line, Sp_model_names);
  // Molecular weights.
  aggregation_stream.SkipElements(3);
  aggregation_stream.GetLine(line);
  split(line, molecular_weights_model);
  // Reactivities.
  aggregation_stream.SkipElements(3);
  aggregation_stream.GetLine(line);
  split(line, React_model);

  if ( int(Sp_model_names.size()) != Nsp_model
       || int(molecular_weights_model.size()) != Nsp_model
       || int(React_model.size()) != Nsp_model )
    throw "Aggregation file badly formatted.";
  
  /*** Reads speciation ***/

  RegularGrid<real> GridSectors(Species_factor[2]);
  Data<real, 1> Speciation_coeff(GridSectors);

  real Aggregation_coeff;
  real React_real;
  real molecular_weights_real;

  const real M_NO2 = 46.0;
  const real int_OH(1.0);

  Species_factor.Fill(0.0);

  for (int l=0; l<Nsp_emis; l++)
    {
      input_file = speciation_directory + Sp_emis_names[l] + ".dat";
      ExtStream speciation_stream(input_file);
      if (!speciation_stream.is_open())
	throw string("File \"") + input_file + "\" doesn't exist.";

      while (!speciation_stream.IsEmpty())
	{
	  Sp_real_names = speciation_stream.GetElement();
	  for (int k = 0; k < Nsectors; k++)
	    speciation_stream >> Speciation_coeff(k);
	  
	  ExtStream aggregation_stream(aggregation_matrix);
	  aggregation_stream.SkipLines(3);
	  stmp = "";
	  while (!aggregation_stream.IsEmpty() && Sp_real_names != stmp)
	    {
	      aggregation_stream >> stmp;
	      line = aggregation_stream.GetLine();
	    }
	  if (aggregation_stream.IsEmpty() && Sp_real_names != stmp)
	    throw string("Species ") + Sp_real_names + " not found.";
	  
	  split(line, vtmp);
	  molecular_weights_real = vtmp[0];
	  React_real = vtmp[1];

	  // NOX is given in NO2 equivalent units.
	  if (Sp_emis_names[l] == "NOX")
	    for (int k=0; k<Nsectors; k++)
	      Speciation_coeff(k) *= molecular_weights_real / M_NO2;

	  for (int m=0; m<Nsp_model; m++)
	    { 
	      Aggregation_coeff = vtmp[m + 2];
	      for (int k = 0; k < Nsectors; k++)
		{
		  Species_factor(m, l, k) += Speciation_coeff(k) * 0.01
		    * Aggregation_coeff
		    * molecular_weights_model[m] / molecular_weights_real
		    * (1. - exp(-int_OH*React_real) )
		    / (1. - exp(-int_OH*React_model[m]) );
		}
	    }
	  if (aggregation_stream.bad())
	    throw "Aggregation file is badly formatted";
	}
    }

}

// SPECIATIONAGGREGATION //
///////////////////////////


//////////////////////////
// VERTICALDISTRIBUTION //

//! Computes vertical distribution factors.
template <class real>
void ComputeVerticalDistribution(Data<real, 2>& vertical_distribution_in,
				 RegularGrid<real>& GridZ_interf_in,
				 RegularGrid<real>& GridZ_interf_out,
				 Data<real, 1>& Ground_part,
				 Data<real, 2>& vertical_distribution_out)
{
  
  int k, s;

  int Nsectors = vertical_distribution_in[0].GetLength();

  // Vertical Grids.
  int Nz_in = vertical_distribution_in[1].GetLength();
  int Nz_out = vertical_distribution_out[1].GetLength();

  RegularGrid<real> DeltaZ_in(Nz_in);
  RegularGrid<real> DeltaZ_out(Nz_out);

  string input_file, line;

  /*** Reads vertical distribution heights ***/
  
  // Sets values at nodes.
  for (k=0; k<Nz_in; k++)
    DeltaZ_in(k) = GridZ_interf_in(k+1) - GridZ_interf_in(k);
  for (k=0; k<Nz_out; k++)
    DeltaZ_out(k) = GridZ_interf_out(k+1) - GridZ_interf_out(k);

  vertical_distribution_out.Fill(0);

  int k_in = 0;
  for (k = 0; k < Nz_out && k_in < Nz_in; k++)
    {
      if (GridZ_interf_out(k+1) <= GridZ_interf_in(k_in+1))
	for (s=0; s<Nsectors; s++)
	  vertical_distribution_out(s, k) =
	    vertical_distribution_in(s, k_in) / DeltaZ_in(k_in)
	    * DeltaZ_out(k);
      else
	{
	  for (s=0; s<Nsectors; s++)
	    vertical_distribution_out(s, k) = 
	      vertical_distribution_in(s, k_in) / DeltaZ_in(k_in)
	      * ( GridZ_interf_in(k_in+1) - GridZ_interf_out(k));
	  k_in++;
	  while (k_in < Nz_in && GridZ_interf_out(k+1) > GridZ_interf_in(k_in+1))
	    {
	      for (s=0; s<Nsectors; s++)
		vertical_distribution_out(s, k) +=
		  vertical_distribution_in(s, k_in);
	      k_in++;
	    }
	  if (k_in < Nz_in)
	    for (s=0; s<Nsectors; s++)
	      vertical_distribution_out(s, k) += 
		vertical_distribution_in(s, k_in) / DeltaZ_in(k_in)
		* ( GridZ_interf_out(k+1) - GridZ_interf_in(k_in));
	}
    }
}

//! Computes vertical distribution factors.
template <class real>
void DividesByHeights(RegularGrid<real>& GridZ_interf_out,
		      Data<real, 1>& Ground_part,
		      Data<real, 2>& vertical_distribution_out)
{
  
  int k, s;

  int Nsectors = vertical_distribution_out[0].GetLength();

  // Vertical Grids.
  int Nz_out = vertical_distribution_out[1].GetLength();

  RegularGrid<real> DeltaZ_out(Nz_out);

  // Sets values at nodes.
  for (k=0; k<Nz_out; k++)
    DeltaZ_out(k) = GridZ_interf_out(k+1) - GridZ_interf_out(k);

  Ground_part.Mlt(0.01);
  vertical_distribution_out.Mlt(0.01);

  for (s=0; s<Nsectors; s++)
    for (k=0; k<Nz_out; k++)
      vertical_distribution_out(s, k) /= DeltaZ_out(k);
}

// VERTICALDISTRIBUTION //
//////////////////////////


////////////////////////
// GRIDCORREPONDENCES //

//! Correspondence between Emep and Polair grids.
template <class real>
void GridCorrepondences(const Data<int, 2, real>& LUC,
			Data<int, 2, real>& Nurb_emep,
			Data<int, 2, real>& Nwat_emep,
			Data<int, 2, real>& Nfor_emep,
			Data<int, 2, real>& Noth_emep,
			Data<int, 2, real>& Ntot_polair)
{

  const real pi = 3.14159265358979323846264;

  const real xpol(8.);
  const real ypol(110.);
  const real M = 6370./50. * ( 1. + sin(pi/3.) );

  /*** Coordinates ***/

  int Nx_luc = LUC[1].GetLength();
  real x_min_center_luc = LUC[1](0);
  real delta_x_luc = (LUC[1](Nx_luc-1) - LUC[1](0) )/ real(Nx_luc-1);
  real x_min_luc = x_min_center_luc - delta_x_luc/2.0;

  int Ny_luc = LUC[0].GetLength();
  real y_min_center_luc = LUC[0](0);
  real delta_y_luc = (LUC[0](Ny_luc-1) - LUC[0](0)) / real(Ny_luc-1);
  real y_min_luc = y_min_center_luc - delta_y_luc/2.0;

  int Nx_emep = Nurb_emep[1].GetLength();
  int Ny_emep = Nurb_emep[0].GetLength();

  int Nx = Ntot_polair[1].GetLength();
  real x_min_center = Ntot_polair[1](0);
  real Delta_x = (Ntot_polair[1](Nx-1) - Ntot_polair[1](0)) / real(Nx-1); 
  real x_min = x_min_center - Delta_x/2.0;

  int Ny = Ntot_polair[0].GetLength();
  real y_min_center = Ntot_polair[0](0);
  real Delta_y = (Ntot_polair[0](Ny-1) - Ntot_polair[0](0)) / real(Ny-1);
  real y_min = y_min_center - Delta_y/2.0;

  /*** Computes output data ***/

  Ntot_polair.Fill(0);
  
  Nurb_emep.Fill(0);
  Nwat_emep.Fill(0);
  Nfor_emep.Fill(0);
  Noth_emep.Fill(0);
    
  for (int i=0; i<Nx_luc; i++)
    for (int j=0; j<Ny_luc; j++)
      {
	real lon = (x_min_center_luc + i * delta_x_luc) * pi/180.; // rad
	real lat = (y_min_center_luc + j * delta_y_luc) * pi/180.; // rad
	
	int i_emep = int( xpol + M * tan(pi/4. - lat/2.)
			  * sin(lon + 32. * pi/180.) - 0.5 );
	int j_emep = int( ypol - M * tan(pi/4. - lat/2.)
			  * cos(lon + 32. * pi/180.) - 0.5 );

	if (i_emep >= 0 && i_emep < Nx_emep && j_emep >= 0 && j_emep < Ny_emep)
	  if (LUC(j, i) == 13)
	    Nurb_emep(j_emep, i_emep) += 1;
	  else if (LUC(j, i)==0)
	    Nwat_emep(j_emep, i_emep) += 1;
	  else if (LUC(j, i) >= 1 && LUC(j, i) <= 6)
	    Nfor_emep(j_emep, i_emep) += 1;
	  else if (LUC(j, i) >= 7 && LUC(j, i) <= 12)
	    Noth_emep(j_emep, i_emep) += 1;
	  else
	    throw "Error in GridCorrepondences: LUC index out of range.";
	
	lon = x_min_luc + i*delta_x_luc; // deg
	lat = y_min_luc + j*delta_y_luc; // deg
	
	int i_polair = int((lon - x_min) / Delta_x); 
	int j_polair = int((lat - y_min) / Delta_y); 
	
	if (i_polair >= 0 && i_polair < Nx && j_polair >= 0 && j_polair < Ny)
	  Ntot_polair(j_polair, i_polair) += 1;
      }
}

// GRIDCORREPONDENCES //
////////////////////////


//////////////////////
// READMODIFIEDEMEP //

//! Reads Emep modified files.
template <class real>
void ReadModifiedEmep(Date date, vector<string>& Sp_emis_names, string input_directory,
		      Data<real, 3>& MonthlyFactors, Data<real, 3>& DailyFactors,
		      Data<list<EmepCountryEmission<real> >, 4, real>& Emis_land,
		      Data<list<EmepCountryEmission<real> >, 4, real>& Emis_water)
{

  int Ncountry, i, j;
  string line;

  int Nsp_emis = Emis_land[0].GetLength();
  int Nsectors = Emis_land[3].GetLength();
  RegularGrid<real> GridSectors(Emis_land[3]);
  Data<real, 1> Emis_emep(GridSectors);

  // Find the day of the week.
  int day = date.GetWeekDay();

  for(int l = 0; l < Nsp_emis; l++)
    {
      string input_file = input_directory + Sp_emis_names[l] + ".dat";
      ExtStream EmepEmisStream(input_file);
      if (!EmepEmisStream.is_open())
	throw string("File ") + input_file + " doesn't exist.";

      while ( has_element(EmepEmisStream) )
	{
	  EmepEmisStream >> Ncountry >> i >> j;
	  for(int s = 0; s < Nsectors; s++)
	    EmepEmisStream >> Emis_emep(s);
	  getline(EmepEmisStream, line);
	  
	  // In water.
	  if ( Ncountry >= 30 && Ncountry <= 35 )
	    for (int s = 0; s < Nsectors; s++)
	      Emis_water(l, j-1, i-1, s).push_back(EmepCountryEmission<real>(Emis_emep(s)
									     * MonthlyFactors(Ncountry, s, date.GetMonth()-1)
									     * DailyFactors(Ncountry, s, day),
									     Ncountry));
	  // In land.
	  else
	    for (int s = 0; s < Nsectors; s++)
	      Emis_land(l, j-1, i-1, s).push_back(EmepCountryEmission<real>(Emis_emep(s)
									    * MonthlyFactors(Ncountry, s, date.GetMonth()-1)
									    * DailyFactors(Ncountry, s, day),
									    Ncountry));
	}
      if (EmepEmisStream.bad())
	throw string("EMEP emission file \"") + input_file
	  + "\" is badly formatted.";
    }
}

// READMODIFIEDEMEP //
//////////////////////


//////////////////
// EMEPTOPOLAIR //

template <class real>
void EmepToPolair(const Data<int, 2, real>& LUC,
		  const Data<real, 2>& HourlyFactors,
		  const TimeZone& local_time,
		  const Data<real, 3>& Species_factor,
		  const Data<real, 1>& Ground_part,
		  const real Ratio_urb,
		  const real Ratio_for,
		  const real Ratio_oth,
		  const Data<int, 2, real>& Nurb_emep,
		  const Data<int, 2, real>& Nwat_emep,
		  const Data<int, 2, real>& Nfor_emep,
		  const Data<int, 2, real>& Noth_emep,
		  const Data<int, 2, real>& Ntot_polair,
		  Data<list<EmepCountryEmission<real> >, 4, real>& Emis_land,
		  Data<list<EmepCountryEmission<real> >, 4, real>& Emis_water,
		  Data<list<EmepCountryEmission<real> >, 4, real>& Emis_out)
{
  int l, i, j, s;
  typename list<EmepCountryEmission<real> >::iterator iter;
  typename list<EmepCountryEmission<real> >::iterator iter_out;

  const real pi = 3.14159265358979323846264;
  const real Earth_radius = 6370997.;

  const real xpol(8.);
  const real ypol(110.);
  const real M = 6370./50. * ( 1. + sin(pi/3.) );

  int Nsectors = Species_factor.GetLength(2);
  int Nsp_emis = Species_factor.GetLength(1);

  int Nx_luc = LUC[1].GetLength();
  real x_min_center_luc = LUC[1](0);
  real delta_x_luc = (LUC[1](Nx_luc-1) - LUC[1](0)) / real(Nx_luc-1);
  real x_min_luc = x_min_center_luc - delta_x_luc/2.0;

  int Ny_luc = LUC[0].GetLength();
  real y_min_center_luc = LUC[0](0);
  real delta_y_luc = (LUC[0](Ny_luc-1) - LUC[0](0)) / real(Ny_luc-1);
  real y_min_luc = y_min_center_luc - delta_y_luc/2.0;

  int Nx_emep = Nurb_emep[1].GetLength();
  int Ny_emep = Nurb_emep[0].GetLength();

  int Nx = Ntot_polair[1].GetLength();
  real x_min_center = Ntot_polair[1](0);
  real Delta_x = (Ntot_polair[1](Nx-1) - Ntot_polair[1](0)) / real(Nx-1); 
  real x_min = x_min_center - Delta_x/2.0;

  int Ny = Ntot_polair[0].GetLength();
  real y_min_center = Ntot_polair[0](0);
  real Delta_y = (Ntot_polair[0](Ny-1) - Ntot_polair[0](0)) / real(Ny-1);
  real y_min = y_min_center - Delta_y/2.0;

  real Ratio(0.0);
  
  for (i=0; i<Nx_luc; i++)
    for (j=0; j<Ny_luc; j++)
      {
	real lon = (x_min_center_luc + i * delta_x_luc) * pi/180.; // rad
	real lat = (y_min_center_luc + j * delta_y_luc) * pi/180.; // rad
	
	int i_emep = int(xpol + M * tan(pi/4. - lat/2.)
			 * sin(lon + 32. * pi/180.) - 0.5 );
	int j_emep = int(ypol - M * tan(pi/4. - lat/2.)
			 * cos(lon + 32. * pi/180.) - 0.5 );

	lon = x_min_luc + i * delta_x_luc; // deg
	lat = y_min_luc + j * delta_y_luc; // deg
	
	int i_polair = int ((lon - x_min) / Delta_x);
	int j_polair = int ((lat - y_min) / Delta_y);
	
	if ( i_emep >= 0 && i_emep < Nx_emep && j_emep >= 0 && j_emep < Ny_emep
	     && i_polair >= 0 && i_polair < Nx && j_polair >= 0 && j_polair < Ny )
	  {
	    if (LUC(j, i) == 0)
	      {
		Ratio = 1. / real(Nwat_emep(j_emep, i_emep));
		for (l = 0; l < Nsp_emis; l++)
		  for (s = 0; s < Nsectors; s++)
		    for (iter = Emis_water(l, j_emep, i_emep, s).begin();
			 iter != Emis_water(l, j_emep, i_emep, s).end(); ++iter)
		      {
			iter_out = Emis_out(l, s, j_polair, i_polair).begin();
			while (iter_out != Emis_out(l, s, j_polair, i_polair).end()
			       && iter_out->country_ != iter->country_)
			  ++iter_out;
			if (iter_out != Emis_out(l, s, j_polair, i_polair).end())
			  iter_out->emission_ += iter->emission_ * Ratio;
			else
			  Emis_out(l, s, j_polair, i_polair).push_back(EmepCountryEmission<real>(iter->emission_ * Ratio, iter->country_));
		      }
	      }
	    else
	      {
		if (LUC(j, i) == 13)
		  Ratio = Ratio_urb
		    / real( Ratio_urb * Nurb_emep(j_emep, i_emep)
			    + Ratio_for * Nfor_emep(j_emep, i_emep)
			    + Ratio_oth * Noth_emep(j_emep, i_emep) );
		
		if ( LUC(j, i) >= 1 && LUC(j, i) <= 6 )
		  Ratio = Ratio_for
		    / real( Ratio_urb * Nurb_emep(j_emep, i_emep)
			    + Ratio_for * Nfor_emep(j_emep, i_emep)
			    + Ratio_oth * Noth_emep(j_emep, i_emep) );
		
		if ( LUC(j, i) >= 7 && LUC(j, i) <= 12 )
		  Ratio = Ratio_oth
		    / real( Ratio_urb * Nurb_emep(j_emep, i_emep)
			    + Ratio_for * Nfor_emep(j_emep, i_emep)
			    + Ratio_oth * Noth_emep(j_emep, i_emep) );
		
		for (l = 0; l < Nsp_emis; l++)
		  for (s = 0; s < Nsectors; s++)
		    for (iter = Emis_land(l, j_emep, i_emep, s).begin();
			 iter != Emis_land(l, j_emep, i_emep, s).end(); ++iter)
		      {
			iter_out = Emis_out(l, s, j_polair, i_polair).begin();
			while (iter_out != Emis_out(l, s, j_polair, i_polair).end()
			       && iter_out->country_ != iter->country_)
			  ++iter_out;
			if (iter_out != Emis_out(l, s, j_polair, i_polair).end())
			  iter_out->emission_ += iter->emission_ * Ratio;
			else
			  Emis_out(l, s, j_polair, i_polair).push_back(EmepCountryEmission<real>(iter->emission_ * Ratio, iter->country_));
		      }
	      }
	  }
      }

  const real ratio_pi = pi / 180.;
  const real factor = ratio_pi * ratio_pi * Earth_radius * Earth_radius
    * delta_x_luc * delta_y_luc;;
  real surface;

  // Divides by the surface. Emissions are then given in Tons/m^2/an.
  for (i = 0; i < Nx; i++)
    for (j = 0; j < Ny; j++)
      {
	surface = Ntot_polair(j, i) * factor * cos(Ntot_polair[0](j) * ratio_pi);
	for (l = 0; l < Nsp_emis; l++)
	  for (s = 0 ; s < Nsectors; s++)
	    for (iter = Emis_out(l, s, j, i).begin();
		 iter != Emis_out(l, s, j, i).end(); ++iter)
	      iter->emission_ /= surface;
      }
  
}

// EMEPTOPOLAIR //
//////////////////
