// Copyright (C) 2003-2004 CEREA
//     Author: Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of a simulation system for air quality.
// 
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


// This program reads EMEP total annual emissions and put it on Polair Grid.

#include <iostream>
#include <vector>
#include <list>
#include <cmath>

#define SELDONDATA_DEBUG_LEVEL_2

#include "SeldonData.hxx"

using namespace SeldonData;

#include "Emissions.cxx"

int main(int argc, char* argv[])
{
  
  TRY;

  cout << endl;

  typedef float real;

  string configuration_file("emissions.cfg"), sec_config_file("");

  if (argc != 2 && argc != 3 && argc != 4)
    {
      string mesg  = "Usage:\n";
      mesg += string("  ") + argv[0] + " [main configuration file] [secondary configuration file] [date]\n";
      mesg += string("  ") + argv[0] + " [main configuration file] [date]\n";
      mesg += string("  ") + argv[0] + " [date]\n\n";
      mesg += "Arguments:\n";
      mesg += "  [main configuration file] (optional): main configuration file. Default: emissions.cfg\n";
      mesg += "  [secondary configuration file] (optional): secondary configuration file.\n";
      mesg += "  [date]: date in format YYYYMMDD.\n";
      cout << mesg << endl;
      return 1;
    }

  if (argc == 4)
    sec_config_file = argv[2];
  if (argc == 3 || argc == 4)
    configuration_file = argv[1];

  if (!exists(configuration_file))
    throw string("Unable to find configuration file \"")
      + configuration_file + "\".";

  Date date(convert<int>(argv[argc - 1]));

  cout << "Date: " << date.GetDate() << endl << endl;

  
  ////////////////////////
  // FIRST DECLARATIONS //
  ////////////////////////

  int h, i, j, k, l, m, s;

  // Configuration file.
  ConfigStreams configuration(configuration_file);
  if (exists(sec_config_file))
    configuration.AddFile(sec_config_file);

  /*** Output ***/
  
  int Nt = 24;
  int Nx, Ny, Nz;
  real Delta_x, Delta_y;
  real x_min, y_min;
  string levels;

  configuration.FindFromBeginning("[domain]");

  configuration.PeekValue("Nx", Nx);
  configuration.PeekValue("Ny", Ny);
  configuration.PeekValue("Nz", Nz);
  configuration.PeekValue("Delta_x", Delta_x);
  configuration.PeekValue("Delta_y", Delta_y);
  configuration.PeekValue("x_min", x_min);
  configuration.PeekValue("y_min", y_min);
  configuration.PeekValue("vertical_levels", levels);

  RegularGrid<real> GridY(y_min, Delta_y, Ny);
  RegularGrid<real> GridX(x_min, Delta_x, Nx);
  RegularGrid<real> GridZ(Nz), GridZ_interf(Nz + 1);
  FormatText Levels;
  Levels.Read(levels, GridZ_interf);
  RegularGrid<real> GridT(Nt);
  RegularGrid<real> GridWeek(7);
  RegularGrid<real> GridMonth(12);

  /*** EMEP ***/

  int Nx_emep, Ny_emep, Ncountries, Nsp_emis, Nsectors;
  string EMEP_directory, hourly_factors_file,
    weekdays_factors_file, monthly_factors_file,
    time_zones_file;
  real Ratio_urb, Ratio_for, Ratio_oth;

  configuration.FindFromBeginning("[EMEP]");
 
  configuration.PeekValue("input_directory", EMEP_directory);

  configuration.PeekValue("Nx_emep", Nx_emep);
  configuration.PeekValue("Ny_emep", Ny_emep);
  configuration.PeekValue("Ncountries", Ncountries);
  configuration.PeekValue("Nsectors", Nsectors);

  configuration.PeekValue("hourly_factors", hourly_factors_file);
  configuration.PeekValue("weekdays_factors", weekdays_factors_file);
  configuration.PeekValue("monthly_factors", monthly_factors_file);
  configuration.PeekValue("time_zones", time_zones_file);

  configuration.PeekValue("urban_ratio", Ratio_urb);
  configuration.PeekValue("forest_ratio", Ratio_for);
  configuration.PeekValue("other_ratio", Ratio_oth);

  string vertical_distribution_file;
  configuration.PeekValue("Polair_vertical_distribution", vertical_distribution_file);

  // Species names.
  vector<string> Sp_emis_names;

  configuration.Find("Species");
  split(configuration.GetLine(), Sp_emis_names);

  Nsp_emis = Sp_emis_names.size();  
  
  RegularGrid<real> GridY_emep(Ny_emep);
  RegularGrid<real> GridX_emep(Nx_emep);
  RegularGrid<real> Grid_countries(Ncountries);
  RegularGrid<real> GridSectors(Nsectors);
  RegularGrid<real> GridSp_emis(Nsp_emis);

  /*** LUC ***/

  int Nx_luc, Ny_luc;
  real x_min_luc, y_min_luc;
  real Delta_x_luc, Delta_y_luc;
  string LUC_file;

  configuration.FindFromBeginning("[LUC]");
 
  configuration.PeekValue("Nx", Nx_luc);
  configuration.PeekValue("Ny", Ny_luc);
  configuration.PeekValue("x_min", x_min_luc);
  configuration.PeekValue("y_min", y_min_luc);
  configuration.PeekValue("Delta_x", Delta_x_luc);
  configuration.PeekValue("Delta_y", Delta_y_luc);

  configuration.PeekValue("File", LUC_file);

  RegularGrid<real> GridX_luc(x_min_luc, Delta_x_luc, Nx_luc);
  RegularGrid<real> GridY_luc(y_min_luc, Delta_y_luc, Ny_luc);
  
  /*** Species ***/

  int Nsp_model;
  string aggregation_file, speciation_directory;

  configuration.FindFromBeginning("[Species]");

  configuration.PeekValue("N", Nsp_model);
  configuration.PeekValue("aggregation", aggregation_file);
  configuration.PeekValue("speciation_directory", speciation_directory);

  RegularGrid<real> GridSp_model(Nsp_model);

  /*** Files ***/

  string Dir_out_emis, Dir_out_src;

  configuration.FindFromBeginning("[Files]");

  configuration.PeekValue("Directory_out_srf_emis", Dir_out_emis);
  configuration.PeekValue("Directory_out_vol_emis", Dir_out_src);


  //////////
  // DATA //
  //////////

  Data<list<EmepCountryEmission<real> >, 4, real> Emis_land(GridSp_emis, GridY_emep, GridX_emep, GridSectors);
  Data<list<EmepCountryEmission<real> >, 4, real> Emis_water(GridSp_emis, GridY_emep, GridX_emep, GridSectors);
  
  Data<int, 2, real> Ntot_polair(GridY, GridX);
  
  Data<int, 2, real> Nurb_emep(GridY_emep, GridX_emep);
  Data<int, 2, real> Nwat_emep(GridY_emep, GridX_emep);
  Data<int, 2, real> Nfor_emep(GridY_emep, GridX_emep);
  Data<int, 2, real> Noth_emep(GridY_emep, GridX_emep);

  Data<int, 2, real> LUC(GridY_luc, GridX_luc);
  Data<list<EmepCountryEmission<real> >, 4, real> Emis_out(GridSp_emis, GridSectors, GridY, GridX);

  Data<real, 4> Emissions_out(GridSp_model, GridT, GridY, GridX);
  Data<real, 3> Species_factor(GridSp_model, GridSp_emis, GridSectors);
  Data<real, 3> MonthlyFactors(Grid_countries, GridSectors, GridMonth);
  Data<real, 3> DailyFactors(Grid_countries, GridSectors, GridWeek);
  Data<real, 2> HourlyFactors(GridSectors, GridT);

  Data<real, 1> Ground_part(GridSectors);
  Data<real, 2> vertical_distribution_out(GridSectors, GridZ);
  Data<real, 5> Sources_out(GridSp_model, GridT, GridZ, GridY, GridX);

  vector<string> Sp_model_names(Nsp_model);  

  // LUC.
  FormatBinary<int> FormatLUC;
  FormatLUC.Read(LUC_file, LUC);
  

  //////////////////////
  // READS INPUT DATA //
  //////////////////////

  cout << "Reading monthly factors..."; cout.flush();
  GetTemporalFactors(monthly_factors_file, MonthlyFactors);
  cout << " done." << endl;
  
  cout << "Reading daily factors..."; cout.flush();
  GetTemporalFactors(weekdays_factors_file, DailyFactors);
  cout << " done." << endl;

  cout << "Grid correspondences EMEP / LUC / POLAIR..."; cout.flush();
  GridCorrepondences(LUC, Nurb_emep, Nwat_emep,
		     Nfor_emep, Noth_emep, Ntot_polair);
  cout << " done." << endl;

  cout << "Reading aggregation/speciation..."; cout.flush();
  SpeciationAggregation(Sp_emis_names, Sp_model_names,
			aggregation_file, speciation_directory,
			Species_factor);
  cout << " done" << endl;

  cout << "Reading vertical distribution..."; cout.flush();
  FormatFormattedText distribution_format(string("<e><e ") + to_str(Nz) + ">");
  distribution_format.Read(vertical_distribution_file, "0", Ground_part);
  distribution_format.Read(vertical_distribution_file, "1", vertical_distribution_out);
  DividesByHeights(GridZ_interf, Ground_part, vertical_distribution_out);
  // maximum height.
  int k_max = Nz;
  bool zero(true);
  while (zero && k_max > 0)
    {
      k_max--;
      for (s=0; s<Nsectors; s++)
	zero = vertical_distribution_out(s, k_max) == 0. && zero;
    }
  k_max++;
  cout << " done." << endl;

  /*** Reads hourly profiles ***/

  cout << "Reading hourly factors..."; cout.flush();
  FormatFormattedText HourlyFactors_format(string("<e><e ")
					   + to_str(HourlyFactors.GetLength(1))
					   + ">");
  HourlyFactors_format.Read(hourly_factors_file, "1", HourlyFactors);
  // Local times associated with countries.
  TimeZone local_time(Ncountries);
  local_time.Init(time_zones_file);
  cout << " done." << endl;

  /*** Reads input Emissions ***/
  
  cout << "Reading input emissions..."; cout.flush();
  ReadModifiedEmep(date.GetDate(), Sp_emis_names, EMEP_directory,
		   MonthlyFactors, DailyFactors, 
		   Emis_land, Emis_water);
  cout << " done." << endl;


  ////////////////////////
  // COMPUTES EMISSIONS //
  ////////////////////////

  cout << "Computes emissions on Polair grid..."; cout.flush();
  EmepToPolair(LUC, HourlyFactors, local_time, Species_factor, Ground_part,
	       Ratio_urb, Ratio_for, Ratio_oth, Nurb_emep, Nwat_emep,
	       Nfor_emep, Noth_emep, Ntot_polair, Emis_land, Emis_water,
	       Emis_out);
  cout << " done." << endl;

  cout << "Computes final emissions..."; cout.flush();

  // Winter/summer time.
  int inc, time_index;
  // Searches for the last sunday in March.
  Date winter_limit;
  winter_limit.SetYear(date.GetYear()); winter_limit.SetMonth(3); winter_limit.SetDay(31);
  while (winter_limit.GetWeekDay() != 6)
    winter_limit.AddDays(-1);
  // Searches for the last sunday in October.
  Date summer_limit;
  summer_limit.SetYear(date.GetYear()); summer_limit.SetMonth(10); summer_limit.SetDay(31);
  while (summer_limit.GetWeekDay() != 6)
    summer_limit.AddDays(-1);

  if (date.GetNumberOfDays() < winter_limit.GetNumberOfDays()
      || date.GetNumberOfDays() > summer_limit.GetNumberOfDays())
    inc = 0;
  else
    inc = 1;

  real factor, emission;

  list<EmepCountryEmission<real> >::iterator iter;
  // Surface emissions.
  Emissions_out.Fill(0.0);
  for (l = 0; l < Nsp_emis; l++)
    for (j = 0; j < Ny; j++)
      for (i = 0; i < Nx; i++)
	for (s = 0; s < Nsectors; s++)
	  for (iter = Emis_out(l, s, j, i).begin();
	       iter != Emis_out(l, s, j, i).end(); ++iter)
	    {
	      emission = iter->emission_;
	      for (m = 0; m < Nsp_model; m++)
		{
		  factor = Species_factor(m, l, s) * emission;
		  time_index = inc + local_time(iter->country_);
		  for (h = 0; h < Nt; h++)
		    Emissions_out(m, h, j, i) += factor
		      * Ground_part(s)
		      * HourlyFactors(s, time_index++ % 24);
		}
	    }
  // From Tons/m^2/an to microg/m^2/s.
  Emissions_out.Mlt(1.e12 / (365. * real(Nt) * 3600.));
  
  Sources_out.Fill(0.0);
  for (l = 0; l < Nsp_emis; l++)
    for (j = 0; j < Ny; j++)
      for (i = 0; i < Nx; i++)
	for (s = 0; s < Nsectors; s++)
	  for (iter = Emis_out(l, s, j, i).begin();
	       iter != Emis_out(l, s, j, i).end(); ++iter)
	    {
	      emission = iter->emission_;
	      for (m = 0; m < Nsp_model; m++)
		{
		  factor = Species_factor(m, l, s) * emission;
		  for (k = 0; k < k_max; k++)
		    {
		      time_index = inc + local_time(iter->country_);
		      for (h = 0; h < Nt; h++)
			Sources_out(m, h, k, j, i) += factor
			  * vertical_distribution_out(s, k)
			  * HourlyFactors(s, time_index++ % 24);
		    }
		}
	    }
  // From Tons/m^2/an/hauteur to microg/m^3/s.
  Sources_out.Mlt(1.e12 / (3600. * 365. * real(Nt)) );

  cout << " done." << endl;


  /////////////////
  // WRITES DATA //
  /////////////////

  cout << "Writing output emissions..."; cout.flush();

  FormatBinary<float> Output;
  for (m = 0; m < Nsp_model; m++)
    {
      Array<real, 3> Emis_extract = Emissions_out.GetArray()
      	(m, Range::all(), Range::all(), Range::all());
      Output.Append(Emis_extract, Dir_out_emis + Sp_model_names[m] + ".bin");
    }

  for (m=0; m<Nsp_model; m++)
    {
      Data<real, 4> Sources_extract(Nt, k_max, Ny, Nx);
      Sources_extract.SubData(Sources_out,
			      m, Range::all(), Range(0, k_max-1),
			      Range::all(), Range::all());
      Output.Append(Sources_extract, Dir_out_src + Sp_model_names[m] + ".bin");
    }
  
  cout << " done." << endl;

  cout << endl;

  END;
  
  return 0;
  
}
