// Copyright (C) 2003-2004 CEREA
//     Author: Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of a simulation system for air quality.
// 
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


//////////////
// INCLUDES //

#include <cmath>
#include <iostream>
#include <algorithm>
using namespace std;

#define SELDONDATA_DEBUG_LEVEL_4
#define SELDONDATA_WITH_GRIB

#include "AtmoData.hxx"
using namespace AtmoData;

// INCLUDES //
//////////////


int main(int argc, char** argv)
{

  TRY;
  
  cout << endl;

  string configuration_file("Kz.cfg"), sec_config_file("");

  if (argc != 2 && argc != 3 && argc != 4)
    {
      string mesg  = "Usage:\n";
      mesg += string("  ") + argv[0] + " [main configuration file] [secondary configuration file] [date]\n";
      mesg += string("  ") + argv[0] + " [main configuration file] [date]\n";
      mesg += string("  ") + argv[0] + " [date]\n\n";
      mesg += "Arguments:\n";
      mesg += "  [main configuration file] (optional): main configuration file. Default: Kz.cfg\n";
      mesg += "  [secondary configuration file] (optional): secondary configuration file.\n";
      mesg += "  [date]: date in format YYYYMMDD.\n";
      cout << mesg << endl;
      return 1;
    }

  if (argc == 4)
    sec_config_file = argv[2];
  if (argc == 3 || argc == 4)
    configuration_file = argv[1];

  if (!exists(configuration_file))
    throw string("Unable to find configuration file \"")
      + configuration_file + "\".";


  ////////////////////////
  // FIRST DECLARATIONS //
  ////////////////////////

  typedef float real;

  int h, i, j, k;


  /////////////////////////
  // CONFIGURATION FILES //
  /////////////////////////

  cout << "Reading configuration files..."; cout.flush(); 
 
  ConfigStreams configuration(configuration_file);
  if (exists(sec_config_file))
    configuration.AddFile(sec_config_file);

  // Domain.
  int Nt, Nz, Ny, Nx;
  real Delta_t, Delta_y, Delta_x;
  real t_min, y_min, x_min;
  string vertical_levels;

  configuration.FindFromBeginning("[domain]");

  configuration.PeekValue("Nt", Nt);
  configuration.PeekValue("Nx", Nx);
  configuration.PeekValue("Ny", Ny);
  configuration.PeekValue("Nz", Nz);
  configuration.PeekValue("Delta_t", Delta_t);
  configuration.PeekValue("Delta_y", Delta_y);
  configuration.PeekValue("Delta_x", Delta_x);
  configuration.PeekValue("t_min", t_min);
  configuration.PeekValue("y_min", y_min);
  configuration.PeekValue("x_min", x_min);
  configuration.PeekValue("vertical_levels", vertical_levels);
  
  // Dates.
  int date_meteo_int;
  configuration.PeekValue("Date", date_meteo_int);
  Date date_meteo(date_meteo_int);
  Date date(convert<int>(argv[argc - 1]));
  int record = date.GetNumberOfDays() - date_meteo.GetNumberOfDays();

  // Vertical diffusion.
  real Kz_min, Kz_max, Delta_t_Kz;
  bool apply_vert;

  configuration.FindFromBeginning("[Kz]");

  configuration.PeekValue("Delta_t", Delta_t_Kz);
  configuration.PeekValue("min", Kz_min);
  configuration.PeekValue("max", Kz_max);
  configuration.PeekValue("apply_vert", apply_vert);

  // Files.
  string directory_in, file_out;

  configuration.FindFromBeginning("[Kz]");

  configuration.PeekValue("directory_in", directory_in);
  configuration.PeekValue("Louis_file", file_out);

  cout << " done." << endl;


  ///////////
  // GRIDS //
  ///////////

  cout << "Memory allocation for data fields..."; cout.flush();

  // Grids.
  RegularGrid<real> GridT(t_min, Delta_t, Nt);
  RegularGrid<real> GridT_Kz(t_min, Delta_t_Kz, int(24./Delta_t_Kz));
  RegularGrid<real> GridZ(Nz);
  RegularGrid<real> GridY(y_min, Delta_y, Ny);
  RegularGrid<real> GridX(x_min, Delta_x, Nx);

  // Data may be provided on in_interfaces.
  RegularGrid<real> GridZ_interf(Nz + 1);
  RegularGrid<real> GridY_interf(y_min - Delta_y / 2., Delta_y, Ny + 1);
  RegularGrid<real> GridX_interf(x_min - Delta_x / 2., Delta_x, Nx + 1);

  // Reads output altitudes.
  FormatText Heights;
  Heights.Read(vertical_levels, GridZ_interf);
  // Sets values at nodes.
  for (k=0; k<Nz; k++)
    GridZ(k) = ( GridZ_interf(k) + GridZ_interf(k+1) ) / 2.0;


  //////////
  // DATA //
  //////////

  // Input fields.

  Data<real, 4> Temperature(GridT, GridZ, GridY, GridX);
  Data<real, 4> PotentialTemperature(GridT, GridZ, GridY, GridX);
  Data<real, 4> Pressure(GridT, GridZ, GridY, GridX);

  Data<real, 3> ConvectiveRain(GridT, GridY, GridX);

  Data<real, 4> MeridionalWind(GridT, GridZ, GridY_interf, GridX);
  Data<real, 4> ZonalWind(GridT, GridZ, GridY, GridX_interf);
  Data<real, 4> VerticalWind(GridT, GridZ_interf, GridY, GridX);

  Data<real, 4> Temperature_Kz(GridT_Kz, GridZ, GridY, GridX);
  Data<real, 4> PotentialTemperature_Kz(GridT_Kz, GridZ, GridY, GridX);
  Data<real, 4> Pressure_Kz(GridT_Kz, GridZ, GridY, GridX);

  Data<real, 3> ConvectiveRain_Kz(GridT_Kz, GridY, GridX);

  Data<real, 4> MeridionalWind_Kz(GridT_Kz, GridZ, GridY_interf, GridX);
  Data<real, 4> ZonalWind_Kz(GridT_Kz, GridZ, GridY, GridX_interf);
  Data<real, 4> VerticalWind_Kz(GridT_Kz, GridZ_interf, GridY, GridX);

  Data<real, 4> Kz(GridT_Kz, GridZ_interf, GridY, GridX);

  cout << " done." << endl;


  /////////////////
  // READS INPUT //
  /////////////////

  FormatBinary<float> PolairFormat;

  cout << "Extracting data..."; cout.flush();

  PolairFormat.ReadRecord(directory_in + "Temperature.bin",
			  record, Temperature);
  PolairFormat.ReadRecord(directory_in + "Pressure.bin", record, Pressure);
  PolairFormat.ReadRecord(directory_in + "MeridionalWind.bin",
			  record, MeridionalWind);
  PolairFormat.ReadRecord(directory_in + "ZonalWind.bin", record, ZonalWind);

  PolairFormat.ReadRecord(directory_in + "ConvectiveRain.bin",
			  record, ConvectiveRain);

  cout << " done." << endl;


  /////////////////////////
  // WINDS AND DIFFUSION //
  /////////////////////////

  // Vertical diffusion (Louis formula, 1979).
  cout << "Computing Kz..."; cout.flush();

  if (Delta_t != Delta_t_Kz)
    {
      LinearInterpolationDimension(Temperature, Temperature_Kz, 0);
      LinearInterpolationDimension(Pressure, Pressure_Kz, 0);
      LinearInterpolationDimension(ZonalWind, ZonalWind_Kz, 0);
      LinearInterpolationDimension(MeridionalWind, MeridionalWind_Kz, 0);
      LinearInterpolationDimension(ConvectiveRain, ConvectiveRain_Kz, 0);
    }
  else
    {
      Temperature_Kz.GetArray() = Temperature.GetArray();
      Pressure_Kz.GetArray() = Pressure.GetArray();
      ZonalWind_Kz.GetArray() = ZonalWind.GetArray();
      MeridionalWind_Kz.GetArray() = MeridionalWind.GetArray();
      ConvectiveRain_Kz.GetArray() = ConvectiveRain.GetArray();
    }

  VerticalWind_Kz.SetZero();

  ComputePotentialTemperature(Temperature_Kz, Pressure_Kz,
			      PotentialTemperature_Kz);

  ComputeLouisKz(ZonalWind_Kz, MeridionalWind_Kz,
		 PotentialTemperature_Kz, Kz);

  real Kz_max_loc;
  int imax(0);

  for (h=0; h<Kz.GetLength(0); h++)
    for (j=0; j<Ny; j++)
      for (i=0; i<Nx; i++)
	{
	  Kz_max_loc = 0.0;
	  for (k=0; k<Nz+1; k++)
	    if (Kz(h, k, j, i) >= Kz_max_loc)
	      {
		Kz_max_loc = Kz(h, k, j, i);
		imax = k;
	      }
	  if (ConvectiveRain_Kz(h, j, i) > 1./6.)
	    for (k=imax; k<Nz+1; k++)
	      Kz(h, k, j, i) = Kz_max_loc;
	  else
	    for (k=imax; k<Nz+1; k++)
	      Kz(h, k, j, i) = (1. - ConvectiveRain_Kz(h, j, i) * 6.)
		* Kz(h, k, j, i)
		+ 6. * ConvectiveRain_Kz(h, j, i) * Kz_max_loc;
	}

  if (!apply_vert)
    for (h=0; h<Kz.GetLength(0); h++)
      for (j=0; j<Ny; j++)
	for (i=0; i<Nx; i++)
	  {
	    if (Kz(h, 1, j, i) < Kz_min)
	      Kz(h, 1, j, i) = Kz_min;
	  }
  else
    Kz.ThresholdMin(Kz_min);

  Kz.ThresholdMax(Kz_max);
 
  cout << " done." << endl;
  

  ////////////////////////
  // WRITES OUTPUT DATA //
  ////////////////////////

  cout << "Writing data..."; cout.flush();
  PolairFormat.Append(Kz, file_out);
  cout << " done." << endl;

  cout << endl;

  END;

  return 0;

}
