// Copyright (C) 2003-2004 CEREA
//     Author: Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of a simulation system for air quality.
// 
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


#include <cmath>
#include <cstring>
#include <iostream>
using namespace std;

#define SELDONDATA_DEBUG_LEVEL_4

#include "AtmoData.hxx"
using namespace AtmoData;

#define _cls cls_

extern "C"
{
  void _cls(float*, float*, float*, float*, float*, 
	    float*, int*, float*, float*, float*, float*,
	    float*, float*, float*, float*, float*,
	    float*);
}


int main(int argc, char** argv)
{

  TRY;
  
  cout << endl;

  string configuration_file("Kz_TM.cfg"), sec_config_file("");

  if (argc != 2 && argc != 3 && argc != 4)
    {
      string mesg  = "Usage:\n";
      mesg += string("  ") + argv[0] + " [main configuration file] [secondary config file] [date]\n";
      mesg += string("  ") + argv[0] + " [main configuration file] [date]\n";
      mesg += string("  ") + argv[0] + " [date]\n\n";
      mesg += "Arguments:\n";
      mesg += "  [main configuration file] (optional): main configuration file. Default: Kz_TM.cfg\n";
      mesg += "  [secondary configuration file] (optional): secondary configuration file.\n";
      mesg += "  [date]: date in format YYYYMMDD.\n";
      cout << mesg << endl;
      return 1;
    }

  if (argc == 4)
    sec_config_file = argv[2];
  if (argc == 3 || argc == 4)
    configuration_file = argv[1];

  if (!exists(configuration_file))
    throw string("Unable to find configuration file \"")
      + configuration_file + "\".";


  ////////////////////////
  // FIRST DECLARATIONS //
  ////////////////////////

  // Warning: 'real' must be 'float' because of Fortran functions
  // that are called.
  typedef float real;

  int h, i, j, k;

  // Constants.
  const real Ka = 0.40;
  const real g = 9.81;


  /////////////////////////
  // CONFIGURATION FILES //
  /////////////////////////

  cout << "Reading configuration files..."; cout.flush(); 
 
  ConfigStreams configuration(configuration_file);
  if (exists(sec_config_file))
    configuration.AddFile(sec_config_file);

  // Output domain.
  int Nt, Nz, Ny, Nx;
  real Delta_t, Delta_y, Delta_x;
  real t_min, y_min, x_min;
  string vertical_levels;

  configuration.FindFromBeginning("[domain]");

  configuration.PeekValue("Nt", Nt);
  configuration.PeekValue("Nx", Nx);
  configuration.PeekValue("Ny", Ny);
  configuration.PeekValue("Nz", Nz);
  configuration.PeekValue("Delta_t", Delta_t);
  configuration.PeekValue("Delta_y", Delta_y);
  configuration.PeekValue("Delta_x", Delta_x);
  configuration.PeekValue("t_min", t_min);
  configuration.PeekValue("y_min", y_min);
  configuration.PeekValue("x_min", x_min);
  configuration.PeekValue("vertical_levels", vertical_levels);
  
  // Dates.
  int date_meteo_int;
  configuration.PeekValue("Date", date_meteo_int);
  Date date_meteo(date_meteo_int);
  Date date(convert<int>(argv[argc - 1]));
  int record = date.GetNumberOfDays() - date_meteo.GetNumberOfDays();

  // Vertical diffusion.
  real Kz_min, Kz_max;
  bool apply_vert;

  configuration.FindFromBeginning("[Kz]");

  configuration.PeekValue("min", Kz_min);
  configuration.PeekValue("max", Kz_max);
  configuration.PeekValue("apply_vert", apply_vert);

  // Troen & Mahrt coefficients.
  double p;
  float C;
  // Ratio between the SBL and the PBL.
  float SBL = 0.1;
  // Critical Richardson number.
  float Ric;

  configuration.PeekValue("p", p);
  configuration.PeekValue("C", C);
  configuration.PeekValue("SBL", SBL);
  configuration.PeekValue("Ric", Ric);
  
  bool fluxes_diagnosed;
  int BL_diag;
  configuration.PeekValue("fluxes_diagnosed", fluxes_diagnosed);
  configuration.PeekValue("BL_diag", BL_diag);

  string dir_in, dir_out, file_in;

  configuration.PeekValue("directory_in", dir_in);
  configuration.PeekValue("directory_out", dir_out);
  configuration.PeekValue("Louis_file", file_in);

  string LUC_file, roughness_file;
  int Nc, sea_index;

  configuration.PeekValue("LUC_file", LUC_file);
  Nc = int(file_size(LUC_file)) / sizeof(float) / (Ny * Nx);
  configuration.PeekValue("sea_index", sea_index);

  configuration.PeekValue("roughness_file", roughness_file);

  cout << " done." << endl;


  ///////////
  // GRIDS //
  ///////////

  cout << "Memory allocation for data fields..."; cout.flush();

  RegularGrid<real> GridT(t_min, Delta_t, Nt);
  // Vertical levels depend on t, z, y and x.
  RegularGrid<real> GridZ(Nz);
  RegularGrid<real> GridY(y_min, Delta_y, Ny);
  RegularGrid<real> GridX(x_min, Delta_x, Nx);
  // On in_interfaces.
  RegularGrid<real> GridZ_interf(Nz + 1);
  RegularGrid<real> GridY_interf(y_min - Delta_y / 2., Delta_y, Ny + 1);
  RegularGrid<real> GridX_interf(x_min - Delta_x / 2., Delta_x, Nx + 1);
  RegularGrid<real> GridC(Nc);
  
  // Reads vertical-heights.
  FormatText Heights;
  Heights.Read(vertical_levels, GridZ_interf);
  // Sets values at nodes.
  for (k=0; k<Nz; k++)
    GridZ(k) = ( GridZ_interf(k) + GridZ_interf(k+1) ) / 2.0;


  //////////
  // DATA //
  //////////

  // Input fields.
  Data<real, 3> LUC(GridC, GridY, GridX);
  Data<real, 4> Temperature(GridT, GridZ, GridY, GridX);
  Data<real, 3> SurfaceTemperature(GridT, GridY, GridX);
  Data<real, 3> SkinTemperature(GridT, GridY, GridX);
  Data<real, 4> Pressure(GridT, GridZ, GridY, GridX);
  Data<real, 3> SurfacePressure(GridT, GridY, GridX);
  Data<real, 4> Humidity(GridT, GridZ, GridY, GridX);
  Data<real, 4> MeridionalWind(GridT, GridZ, GridY_interf, GridX);
  Data<real, 4> ZonalWind(GridT, GridZ, GridY, GridX_interf);
  Data<real, 4> WindModule(GridT, GridZ, GridY, GridX);
  Data<real, 3> FrictionModule(GridT, GridY, GridX);
  Data<real, 3> SoilWater(GridT, GridY, GridX);
  Data<real, 3> BoundaryHeight(GridT, GridY, GridX);
  Data<real, 2> Roughness(GridY, GridX);
  
  // Output fields.
  Data<real, 4> Richardson(GridT, GridZ, GridY, GridX);
  Data<real, 3> SaturationHumidity(GridT, GridY, GridX);
  Data<real, 3> SurfaceHumidity(GridT, GridY, GridX);
  Data<real, 3> SurfaceRelativeHumidity(GridT, GridY, GridX);
  Data<real, 3> LMO(GridT, GridY, GridX);
  Data<real, 3> Evaporation(GridT, GridY, GridX);
  Data<real, 3> SensibleHeat(GridT, GridY, GridX);
  Data<real, 3> Fm(GridT, GridY, GridX);
  Data<real, 3> Ft(GridT, GridY, GridX);
  Data<real, 4> Kz(GridT, GridZ_interf, GridY, GridX);
  Data<real, 4> PotentialTemperature(GridT, GridZ, GridY, GridX);
  Data<real, 3> SurfacePotentialTemperature(GridT, GridY, GridX);

  cout << " done." << endl;
  cout << endl;


  /////////////////
  // READS INPUT //
  /////////////////

  // Polair format.
  FormatBinary<float> InputMeteo;

  InputMeteo.Read(LUC_file, LUC);
  InputMeteo.Read(roughness_file, Roughness);

  cout << "Extracting fields..."; cout.flush();
  InputMeteo.ReadRecord(dir_in + "SurfaceTemperature.bin",
			record, SurfaceTemperature);
  InputMeteo.ReadRecord(dir_in + "Temperature.bin", record, Temperature);
  InputMeteo.ReadRecord(dir_in + "SkinTemperature.bin",
			record, SkinTemperature);
  InputMeteo.ReadRecord(dir_in + "Richardson.bin", record, Richardson);
  InputMeteo.ReadRecord(dir_in + "SoilWater.bin", record, SoilWater);
  InputMeteo.ReadRecord(dir_in + "SurfacePressure.bin",
			record, SurfacePressure);
  InputMeteo.ReadRecord(dir_in + "Pressure.bin", record, Pressure);
  InputMeteo.ReadRecord(dir_in + "Humidity.bin", record, Humidity);
  InputMeteo.ReadRecord(dir_in + "WindModule.bin", record, WindModule);
  InputMeteo.ReadRecord(dir_in + "MeridionalWind.bin",
			record, MeridionalWind);
  InputMeteo.ReadRecord(dir_in + "ZonalWind.bin", record, ZonalWind);
  InputMeteo.ReadRecord(dir_in + "BoundaryHeight.bin",
			record, BoundaryHeight);

  if (!fluxes_diagnosed)
    {
      InputMeteo.ReadRecord(dir_in + "FrictionModule.bin",
			    record, FrictionModule);
      InputMeteo.ReadRecord(dir_in + "Evaporation.bin", record, Evaporation);
      InputMeteo.ReadRecord(dir_in + "SensibleHeat.bin",
			    record, SensibleHeat);
    }

  InputMeteo.ReadRecord(file_in, record, Kz);

  cout << " done." << endl;
  cout << endl;


  ////////////////////////////
  // OUTPUT DATA PROCESSING //
  ////////////////////////////

  cout << "Computing Kz..."; cout.flush();

  /*** First computes meteorological fields ***/

  ComputePotentialTemperature(Temperature, Pressure, PotentialTemperature);
  ComputePotentialTemperature(SkinTemperature, SurfacePressure,
			      SurfacePotentialTemperature);

  ComputeSaturationHumidity(SkinTemperature, SurfacePressure,
			    SaturationHumidity);

  ComputeSurfaceHumidity_diag(Humidity, SaturationHumidity, SoilWater,
			      LUC, sea_index, SurfaceHumidity);

  SurfaceRelativeHumidity.GetArray() = SurfaceHumidity.GetArray()
    / SaturationHumidity.GetArray();

  /*** Fluxes ***/

  // Computes the Monin-Obukhov length and the friction velocity.
  if (fluxes_diagnosed)
    for (h=0; h<Nt; h++)
      for (j=0; j<Ny; j++)
	for (i=0; i<Nx; i++)
	  {
	    // Roughness.
	    float zo = min(Roughness(j, i), 2.);
	    float zot = zo / 10.;
	    int is_sea;
	    if (LUC(sea_index, j, i) > 0.5)
	      is_sea = 1;
	    else
	      is_sea = 0;
	    _cls(&ZonalWind(h, 0, j, i), &MeridionalWind(h, 0, j, i),
		 &PotentialTemperature(h, 0, j, i),
		 &SurfacePotentialTemperature(h, j, i),
		 &Humidity(h, 0, j, i), &SurfaceHumidity(h, j, i),
		 &is_sea, &GridZ(0), &zo, &zot, &GridZ(0),
		 &LMO(h, j, i), &FrictionModule(h, j, i),
		 &SensibleHeat(h, j, i), &Evaporation(h, j, i),
		 &Fm(h, j, i), &Ft(h, j, i));
	  }
  else
    for (h=0; h<Nt; h++)
      for (j=0; j<Ny; j++)
	for (i=0; i<Nx; i++)
	  {
	    float u_star = FrictionModule(h, j, i);
	    float theta_m = 0.5 * (SurfacePotentialTemperature(h, j, i)
				   + PotentialTemperature(h, 0, j, i));
	    LMO(h, j, i) = - u_star * u_star * u_star * theta_m
	      / (Ka * g * (SensibleHeat(h, j, i)
			   + 0.608 * theta_m * Evaporation(h, j, i)));
	  }

  SensibleHeat.GetArray() += 0.608 * Evaporation.GetArray()
    * SurfaceTemperature.GetArray();

  /*** Kz ***/

  float l_MO, u_star, w_star, ws, qv0;
  float boundary_height, buoyancy_temp,
    atm_pot_temp, prev_pot_temp,
    curr_pot_temp(0);
  float Phi_m;

  for (h=0; h<Nt; h++)
    for (j=0; j<Ny; j++)
      for (i=0; i<Nx; i++)
	{

	  u_star = FrictionModule(h, j, i);
	  l_MO = LMO(h, j, i);
	  qv0 = SensibleHeat(h, j, i);

	  /*** T&M diagnostic ***/
	  if (BL_diag == 1)
	    {
	      if (l_MO < 0.)
		w_star = pow(GridZ(1)
			     * double(g / SurfaceTemperature(h, j, i) * qv0),
			     1./3.);
	      else
		w_star = 0.;
	      
	      ws = pow(u_star * u_star * u_star + 7. * SBL * Ka
		       * w_star * w_star * w_star, 1./3.);
	      
	      buoyancy_temp = ( SurfacePotentialTemperature(h, j, i)
				+ PotentialTemperature(h, 0, j, i) ) / 2.0;
	      prev_pot_temp = atm_pot_temp
		= PotentialTemperature(h, 0, j, i) + C * qv0 / ws;
	      
	      // Boundary-layer height.
	      k = 1;
	      while ( (k < Nz-1)
		      && (PotentialTemperature(h, k, j, i)
			  < (curr_pot_temp = atm_pot_temp
			     + Ric * WindModule(h, k, j, i)
			     * WindModule(h, k, j, i)
			     / (GridZ(k) * g) * buoyancy_temp)) )
		{
		  k++;
		  prev_pot_temp = curr_pot_temp;
		  if (l_MO < 0.)
		    w_star = pow(GridZ(k)
				 * double(g / SurfaceTemperature(h, j, i) * qv0),
				 1./3.);
		  else
		    w_star = 0.;
		  ws = pow(u_star * u_star * u_star + 7. * SBL * Ka
			   * w_star * w_star * w_star, 1./3.);
		  atm_pot_temp = PotentialTemperature(h, 0, j, i)
		    + C * qv0 / ws;
		}
	      if (k == 1)
		boundary_height = GridZ_interf(1);
	      else
		boundary_height = GridZ(k-1)
		  + (PotentialTemperature(h, k-1, j, i) - prev_pot_temp)
		  / (curr_pot_temp - prev_pot_temp
		     + PotentialTemperature(h, k-1, j, i)
		     - PotentialTemperature(h, k, j, i))
		  * (GridZ(k) - GridZ(k-1));
	    }

	  /*** Richardson diagnostic ***/
	  else if (BL_diag == 2)
	    {
	      // Boundary-layer height.
	      k = 0;
	      while ( (k < Nz-1) && (Richardson(h, k, j, i) < 0.21) )
		k++;
	      if (k == 0)
		boundary_height = GridZ_interf(1);
	      else if (k == 4)
		boundary_height = GridZ(k);
	      else
		boundary_height = GridZ(k - 1)
		  + (0.21 - Richardson(h, k-1, j, i))
		  * (GridZ(k) - GridZ(k-1))
		  / (Richardson(h, k, j, i) - Richardson(h, k-1, j, i));
	    }

	  /*** ECMWF height ***/
	  else
	    {
	      boundary_height = max(GridZ_interf(0), BoundaryHeight(h, j, i));
	      boundary_height = min(GridZ_interf(Nz-1), boundary_height);
	    }


	  /*** Kz ***/
	  // Computes ws according to the new boundary height.
	  if (l_MO < 0.)
	    w_star = pow(boundary_height
			 * double(g / SurfaceTemperature(h, j, i) *
				  abs(qv0)), 1./3.);
	  else
	    w_star = 0.;
	  ws = pow(u_star * u_star * u_star + 7. * SBL * Ka
		   * w_star * w_star * w_star, 1./3.);

	  // Computes the nondimensional shear Phi_m and Kz.
	  if (l_MO < 0)
	    for (k=1; k<Nz; k++)
	      {
		if (GridZ_interf(k) < SBL * h)
		  Phi_m = pow(1.0 - 7.0 * GridZ_interf(0) / l_MO, -1.0/3.0);
		else
		  Phi_m = u_star / ws;
		if (GridZ_interf(k) < boundary_height)
		  Kz(h, k, j, i) = u_star * Ka * GridZ_interf(k) / Phi_m
		    * pow(1.0 - GridZ_interf(k) / boundary_height, p);
	      }
	  else
	    for (k=1; k<Nz; k++)
	      if (GridZ_interf(k) < boundary_height)
		{
		  Phi_m = 1.0 + 4.7 * GridZ_interf(k) / l_MO;
		  Kz(h, k, j, i) = u_star * Ka * GridZ_interf(k) / Phi_m
		    * pow(1.0 - GridZ_interf(k) / boundary_height, p);
		}

	}

  if (!apply_vert)
    for (h=0; h<Kz.GetLength(0); h++)
      for (j=0; j<Ny; j++)
	for (i=0; i<Nx; i++)
	  {
	    if (Kz(h, 1, j, i) < Kz_min)
	      Kz(h, 1, j, i) = Kz_min;
	  }
  else
    Kz.ThresholdMin(Kz_min);

  Kz.ThresholdMax(Kz_max);
 
  cout << " done." << endl;


  ////////////////////////
  // WRITES OUTPUT DATA //
  ////////////////////////

  cout << "Writing output files..."; cout.flush();

  FormatBinary<float> OutputMeteo;

  OutputMeteo.Append(Kz, dir_out + "Kz_TM.bin");

  OutputMeteo.Append(SurfaceRelativeHumidity,
		     dir_out + "SurfaceRelativeHumidity.bin");

  if (!fluxes_diagnosed)
    OutputMeteo.Append(LMO, dir_out + "LMO.bin");

  if (fluxes_diagnosed)
    {
      OutputMeteo.Append(Evaporation, dir_out + "Evaporation_diag.bin");
      OutputMeteo.Append(SensibleHeat, dir_out + "SensibleHeat_diag.bin");
      OutputMeteo.Append(FrictionModule, dir_out + "FrictionModule_diag.bin");
      OutputMeteo.Append(Fm, dir_out + "Fm.bin");
      OutputMeteo.Append(Ft, dir_out + "Ft.bin");
      OutputMeteo.Append(LMO, dir_out + "LMO_diag.bin");
    }

  cout << " done." << endl;

  cout << endl;

  END;

  return 0;

}
