// Copyright (C) 2003-2004 CEREA
//     Authors: Vivien Mallet and Vincent Picavet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of a simulation system for air quality.
// 
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


#define METEO_FINE_KZ_


//////////////
// INCLUDES //

#include <cmath>
#include <iostream>
#include <algorithm>
using namespace std;

#define SELDONDATA_DEBUG_LEVEL_4
#define SELDONDATA_WITH_GRIB

#include "AtmoData.hxx"
using namespace AtmoData;

// INCLUDES //
//////////////


int main(int argc, char** argv)
{

  TRY;
  
  cout << endl;

  string configuration_file("MM5-meteo.cfg"), sec_config_file("");

  if (argc != 2 && argc != 3 && argc != 4)
    {
      string mesg  = "Usage:\n";
      mesg += string("  ") + argv[0] + " [main configuration file] [secondary config file] [date]\n";
      mesg += string("  ") + argv[0] + " [main configuration file] [date]\n";
      mesg += string("  ") + argv[0] + " [date]\n\n";
      mesg += "Arguments:\n";
      mesg += "  [main configuration file] (optional): main configuration file. Default: MM5-meteo.cfg\n";
      mesg += "  [secondary configuration file] (optional): secondary configuration file.\n";
      mesg += "  [date]: date in format YYYYMMDD.\n";
      cout << mesg << endl;
      return 1;
    }

  if (argc == 4)
    sec_config_file = argv[2];
  if (argc == 4 || argc == 3)
    configuration_file = argv[1];

  if (!exists(configuration_file))
    throw string("Unable to find configuration file \"")
      + configuration_file + "\".";

  Date date(to_num<int>(argv[argc - 1]));


  ////////////////////////
  // FIRST DECLARATIONS //
  ////////////////////////

  typedef float real;

  int h, i, j, k;

  // Constants.
  const real pi = 3.14159265358979323846264;
  const real r = 287.;
  const real cp = 1005.;

  // Input/output directories and files.
  string directory_out, file_in;
  string Filename3DCross, Filename3DDot, FilenameDotToCross;

  // Input dimensions for meteological data.
  int Nt_in, Nz_in, Ny_in, Nx_in;
  // Input-data steps.
  real Delta_t_in, Delta_y_in, Delta_x_in;
  real t_min_in, y_min_in, x_min_in;

  // Output dimensions.
  int Nt_out, Nz_out, Ny_out, Nx_out;
#ifdef METEO_FINE_KZ
  int Nt_fine_out;
#endif

  // Output steps.
  real Delta_t_out, Delta_y_out, Delta_x_out;
#ifdef METEO_FINE_KZ
  real Delta_t_fine_out;
#endif
  real t_min_out, y_min_out, x_min_out;

  string vertical_levels;
  
  // Kz thresholds.
  real Kz_min;
  bool apply_vert;
  real Kz_max;

  // Clouds.
  real min_height;


  /////////////////////////
  // CONFIGURATION FILES //
  /////////////////////////

  cout << "Reading configuration files..."; cout.flush(); 
 
  ConfigStreams configuration(configuration_file);
  if (exists(sec_config_file))
    configuration.AddFile(sec_config_file);

  // MM5 domain.
  configuration.FindFromBeginning("[MM5]");

  configuration.PeekValue("Nt", Nt_in);
  configuration.PeekValue("Nz", Nz_in);
  configuration.PeekValue("Ny", Ny_in);
  configuration.PeekValue("Nx", Nx_in);

  configuration.PeekValue("Delta_t", Delta_t_in);
  configuration.PeekValue("Delta_y", Delta_y_in);
  configuration.PeekValue("Delta_x", Delta_x_in);

  configuration.PeekValue("t_min", t_min_in);
  configuration.PeekValue("y_min", y_min_in);
  configuration.PeekValue("x_min", x_min_in);

  // Output domain.
  configuration.FindFromBeginning("[domain]");

  configuration.PeekValue("Nt", Nt_out);
  configuration.PeekValue("Nx", Nx_out);
  configuration.PeekValue("Ny", Ny_out);
  configuration.PeekValue("Nz", Nz_out);
  configuration.PeekValue("vertical_levels", vertical_levels);
#ifdef METEO_FINE_KZ
  configuration.PeekValue("Nt_fine", Nt_fine_out);
#endif
  
  configuration.PeekValue("Delta_t", Delta_t_out);
  configuration.PeekValue("Delta_y", Delta_y_out);
  configuration.PeekValue("Delta_x", Delta_x_out);
#ifdef METEO_FINE_KZ
  configuration.PeekValue("Delta_t_fine", Delta_t_fine_out);
#endif

  configuration.PeekValue("t_min", t_min_out);
  configuration.PeekValue("y_min", y_min_out);
  configuration.PeekValue("x_min", x_min_out);

  // Kz.
  configuration.FindFromBeginning("[Kz]");
  configuration.PeekValue("min", Kz_min);
  configuration.PeekValue("max", Kz_max);
  configuration.PeekValue("apply_vert", apply_vert);

  // Files.
  configuration.FindFromBeginning("[meteo_files]");
  configuration.PeekValue("file_in", file_in);
  file_in += date.GetDate("%y-%m-%d");
  configuration.PeekValue("directory_out", directory_out);

  // Clouds.
  configuration.FindFromBeginning("[clouds]");
  configuration.PeekValue("min_height", min_height);

  cout << " done." << endl;


  ///////////
  // GRIDS //
  ///////////

  cout << "Memory allocation for grids..."; cout.flush();

  // Input settings.

  // Input grids.
  RegularGrid<real> GridT_in(t_min_in, Delta_t_in, Nt_in);
  // Grid for Z in height. Z depends on Z, Y, X.
  GeneralGrid<real, 3> GridZ_in(shape(Nz_in, Ny_in - 1, Nx_in - 1),
				1, shape(1, 2, 3));

  // Vertical levels are shared.
  GridZ_in.SetDuplicate(false);

  // Grid for Z in height (interfaces). Z depends on Z, Y, X.
  GeneralGrid<real, 3> GridZ_interf_in(shape(Nz_in + 1, Ny_in - 1, Nx_in - 1),
				       1, shape(1, 2, 3));


  // Vertical levels for dot grids are not the same.
  GeneralGrid<real, 3> GridZ_Dot_in(shape(Nz_in, Ny_in, Nx_in),
				    1, shape(1, 2, 3));
  GridZ_Dot_in.SetDuplicate(false);


  // MM5 Input Grid for Z in height. Z depends on Z, X, Y
  GeneralGrid<real, 3> GridZ_MM5_in(shape(Nz_in, Nx_in - 1, Ny_in - 1),
				    1, shape(1, 2, 3));
  // Vertical levels are shared.
  GridZ_MM5_in.SetDuplicate(false);

  // Vertical levels for dot grids are not the same.
  GeneralGrid<real, 3> GridZ_MM5_Dot_in(shape(Nz_in, Nx_in, Ny_in),
					1, shape(1, 2, 3));
  GridZ_MM5_Dot_in.SetDuplicate(false);

  RegularGrid<real> GridY_in(y_min_in, Delta_y_in, Ny_in - 1);
  RegularGrid<real> GridX_in(x_min_in, Delta_x_in, Nx_in - 1);

  // Interfaces.
  RegularGrid<real> GridX_interf_in(x_min_in - Delta_x_in / 2.,
				    Delta_x_in, Nx_in );
  RegularGrid<real> GridY_interf_in(y_min_in - Delta_y_in / 2.,
				    Delta_y_in, Ny_in );

  // Output settings.

  // Output grids.
  RegularGrid<real> GridT_out(t_min_out, Delta_t_out, Nt_out);
#ifdef METEO_FINE_KZ
  RegularGrid<real> GridT_fine_out(t_min_out, Delta_t_fine_out, Nt_fine_out);
#endif
  RegularGrid<real> GridZ_out(Nz_out);
  // Data may be provided on interfaces.
  RegularGrid<real> GridZ_interf_out(Nz_out + 1);

  // Latlon
  RegularGrid<real> GridY_out(y_min_out, Delta_y_out, Ny_out);
  RegularGrid<real> GridX_out(x_min_out, Delta_x_out, Nx_out);
  RegularGrid<real> GridY_interf_out(y_min_out - Delta_y_out / 2.,
				     Delta_y_out, Ny_out + 1);
  RegularGrid<real> GridX_interf_out(x_min_out - Delta_x_out / 2.,
				     Delta_x_out, Nx_out + 1);

  // Lcc.
  GeneralGrid<real, 2> GridX_2D_out(shape(Ny_out, Nx_out), 2, shape(1,2));
  GeneralGrid<real, 2> GridY_2D_out(shape(Ny_out, Nx_out), 1, shape(1,2));
  GeneralGrid<real, 2> GridX_3D_out(shape(Ny_out, Nx_out), 3, shape(2, 3));
  GeneralGrid<real, 2> GridY_3D_out(shape(Ny_out, Nx_out), 2, shape(2, 3));
  GeneralGrid<real, 2> GridX_3D_Gen_out(shape(Ny_out + 1, Nx_out),
					3, shape(2, 3));
  GeneralGrid<real, 2> GridY_3D_Gen_out(shape(Ny_out, Nx_out + 1),
					2, shape(2, 3));
  GeneralGrid<real, 2> GridX_3D_interf_out(shape(Ny_out, Nx_out + 1),
					   3, shape(2, 3));
  GeneralGrid<real, 2> GridY_3D_interf_out(shape(Ny_out + 1, Nx_out),
					   2, shape(2, 3));

  GridX_2D_out.SetDuplicate(false);
  GridY_2D_out.SetDuplicate(false);
  GridX_3D_out.SetDuplicate(false);
  GridY_3D_out.SetDuplicate(false);
  GridX_3D_Gen_out.SetDuplicate(false);
  GridY_3D_Gen_out.SetDuplicate(false);
  GridX_3D_interf_out.SetDuplicate(false);
  GridY_3D_interf_out.SetDuplicate(false);

  // Reads output altitudes.
  FormatText Heights_out;
  Heights_out.Read(vertical_levels, GridZ_interf_out);
  // Sets values at nodes.
  for (k=0; k<Nz_out; k++)
    GridZ_out(k) = ( GridZ_interf_out(k) + GridZ_interf_out(k+1) ) / 2.0;

  cout << " done." << endl;


  /////////////////
  // OUTPUT DATA //
  /////////////////

  cout << "Memory allocation for output data fields..."; cout.flush();


  // 3D Output fields.
  // Cross fields.
  Data<real, 4> Temperature_out(GridT_out, GridZ_out,
				GridY_3D_out, GridX_3D_out);
  Data<real, 4> Richardson_out(GridT_out, GridZ_out,
			       GridY_3D_out, GridX_3D_out);
  Data<real, 4> Pressure_out(GridT_out, GridZ_out,
			     GridY_3D_out, GridX_3D_out);
  Data<real, 4> Humidity_out(GridT_out, GridZ_out,
			     GridY_3D_out, GridX_3D_out);
  Data<real, 4> LiquidWaterContent_out(GridT_out, GridZ_out, 
				       GridY_3D_out, GridX_3D_out);
  Data<real, 4> Attenuation_out(GridT_out, GridZ_out,
				GridY_3D_out, GridX_3D_out);
  // Dot fields.
  Data<real, 4> MeridionalWind_out(GridT_out, GridZ_out,
				   GridY_3D_interf_out, GridX_3D_Gen_out);
  Data<real, 4> ZonalWind_out(GridT_out, GridZ_out,
			      GridY_3D_Gen_out, GridX_3D_interf_out);

  // Post interpolation computed fields 
  Data<real, 4> WindModule_out(GridT_out, GridZ_out, GridY_out, GridX_out);
  // 2D Output fields.
  Data<real, 3> SurfacePressure_out(GridT_out, GridY_2D_out, GridX_2D_out);
  Data<real, 3> SurfaceTemperature_out(GridT_out, GridY_2D_out, GridX_2D_out);
  Data<real, 3> SkinTemperature_out(GridT_out, GridY_2D_out, GridX_2D_out);
  Data<real, 3> SurfaceRichardson_out(GridT_out, GridY_2D_out, GridX_2D_out);
  Data<real, 3> BoundaryHeight_out(GridT_out, GridY_2D_out, GridX_2D_out);
  Data<real, 3> CloudHeight_out(GridT_out, GridY_2D_out, GridX_2D_out);
  Data<real, 3> FrictionModule_out(GridT_out, GridY_2D_out, GridX_2D_out);
  Data<real, 3> SolarRadiation_out(GridT_out, GridY_2D_out, GridX_2D_out);
  Data<real, 3> ConvectiveRain_out(GridT_out, GridY_2D_out, GridX_2D_out);
  Data<real, 3> Rain_out(GridT_out, GridY_2D_out, GridX_2D_out);
  Data<real, 3> SoilWater_out(GridT_out, GridY_2D_out, GridX_2D_out);
  Data<real, 3> SensibleHeat_out(GridT_out, GridY_2D_out, GridX_2D_out);
  Data<real, 3> Evaporation_out(GridT_out, GridY_2D_out, GridX_2D_out);
  // Post interpolation computed fields.
  Data<real, 3> PARdb_out(GridT_out, GridY_out, GridX_out);
  Data<real, 3> PARdiff_out(GridT_out, GridY_out, GridX_out);
  Data<real, 3> PAR_out(GridT_out, GridY_out, GridX_out);
  Data<real, 3> SurfacePotentialTemperature_out(GridT_out,
						GridY_out, GridX_out);

#ifdef METEO_FINE_KZ
  // FIXME : check grids herein.
  Data<real, 4> Temperature_fine_out(GridT_fine_out, GridZ_out,
				     GridY_out, GridX_out);
  Data<real, 4> MeridionalWind_fine_out(GridT_fine_out, GridZ_out,
					GridY_interf_out, GridX_out);
  Data<real, 4> ZonalWind_fine_out(GridT_fine_out, GridZ_out,
				   GridY_out, GridX_interf_out);
  Data<real, 4> VerticalWind_fine_out(GridT_fine_out, GridZ_interf_out,
				      GridY_out, GridX_out);
  Data<real, 4> Pressure_fine_out(GridT_fine_out, GridZ_out,
				  GridY_out, GridX_out);
  Data<real, 4> PotentialTemperature_fine_out(GridT_fine_out, GridZ_out,
					      GridY_out, GridX_out);
  Data<real, 3> ConvectiveRain_fine_out(GridT_fine_out, GridY_out, GridX_out);
  Data<real, 4> Kz_fine_out(GridT_fine_out, GridZ_interf_out,
			    GridY_out, GridX_out);
#else
  Data<real, 4> PotentialTemperature_out(GridT_out, GridZ_out,
					 GridY_out, GridX_out);
  Data<real, 4> VerticalWind_out(GridT_out, GridZ_interf_out,
				 GridY_out, GridX_out);
  Data<real, 4> Kz_out(GridT_out, GridZ_interf_out, GridY_out, GridX_out);
#endif

  cout << " done." << endl;


  ////////////////////////////////
  // SIGMA TO HEIGHT CONVERSION //
  ////////////////////////////////

  cout << "Conversion from sigma levels to heights..."; cout.flush();

  FormatMM5 InputMeteo;

  // Data to convert z from sigma to heights.
  Data<real, 2> SigmaH(Nt_in, Nz_in);
  // Terrain is 2D Field read from MM5 file (-> XY order).
  Data<real, 3> Terrain(GridT_in, GridX_in, GridY_in);
  // FIXME : check grids.
  Data<real, 3> Terrain_Dot(GridT_in, GridY_interf_in, GridX_interf_in);

  // Reads data to convert Z from sigma levels to heights.
  InputMeteo.ReadWholeField(file_in, "SIGMAH", SigmaH);
  InputMeteo.ReadWholeField(file_in, "TERRAIN", Terrain);
  Terrain.SwitchDimensions(shape(0, 2, 1), GridT_in, GridY_in, GridX_in);

  // Converts Z from sigma levels to heights at cross points.

  // Big header data and general constants.
  Array<int, 2> BHI;
  Array<float, 2> BHR;
  Array<string, 2> BHIC;
  Array<string, 2> BHRC;
  real p00, ptop, Ts0, A, Tiso, Piso, Ziso, aux;
  real ps0, refpress;
  const real g(9.8);
  const real R(287.);

  InputMeteo.ReadBigHeader(file_in, BHI, BHR, BHIC, BHRC);

  p00 = BHR(4, 1);
  ptop = BHR(1, 1);
  Ts0 = BHR(4, 2);
  A = BHR(4, 3);
  Tiso = BHR(4, 4);
  Piso = p00 * exp((Tiso - Ts0) / A);
  aux = log(Piso / p00);
  Ziso = - R * A * aux*aux / (2.0*g) - R * Ts0 * aux / g;

  // FIXME : are Nz_in and Ny_in the right values ?
  for (k = 0; k < SigmaH.GetLength(1); k++)
    for (j = 0; j < Ny_in - 1; j++)
      for (i = 0; i < Nx_in - 1; i++)
	{
	  ps0 = p00 * exp( -Ts0 / A + sqrt( Ts0*Ts0/ (A*A)
					    - 2. * g * Terrain(0, j, i)
					    / (A*R) ) ) - ptop;
	  refpress = SigmaH(0, k) * ps0 + ptop;
	  if (refpress >= Piso)
	    {
	      aux = log(refpress / p00);
	      GridZ_in.Value(0, SigmaH.GetLength(1) - k - 1, j, i) =
		- R * A * aux * aux / (2.0 * g) - R * Ts0 * aux / g
		- Terrain(0, j, i);
	    }
	  else
	    {
	      aux = log(refpress / Piso);
	      GridZ_in.Value(0, SigmaH.GetLength(1) - k - 1, j, i)
		= Ziso - R * Tiso * aux / g - Terrain(0, j, i);
	    }
	}

  // Interfaces.
  for (j = 0; j < Ny_in - 1; j++)
    for (i = 0; i < Nx_in - 1; i++)
      GridZ_interf_in.Value(0, 0, j, i) = 0.;
  for (k = 1; k < SigmaH.GetLength(1); k++)
    for (j = 0; j < Ny_in - 1; j++)
      for (i = 0; i < Nx_in - 1; i++)
	GridZ_interf_in.Value(0, k, j, i) = 2 * GridZ_in.Value(0, k - 1, j, i)
	  - GridZ_interf_in.Value(0, k - 1, j, i);

  // Interpolates Terrain at dot points.
  LinearInterpolationRegular(Terrain, Terrain_Dot);

  // Converts z from sigma levels to heights at dot points.
  for (k = 0; k < SigmaH.GetLength(1); k++)
    for (j = 0; j < Ny_in; j++)
      for (i = 0; i < Nx_in; i++)
	{
	  ps0 = p00 * exp(-Ts0/A + sqrt(Ts0*Ts0 / (A*A)
					- 2. * g * Terrain_Dot(0, j, i)
					/ (A*R))) - ptop;
	  refpress = SigmaH(0,k) * ps0 + ptop;
	  if (refpress >= Piso)
	    {
	      aux = log(refpress / p00);
 	      GridZ_Dot_in.Value(0, SigmaH.GetLength(1) - k - 1, j, i) =
		- R * A * aux*aux / (2.0*g) - R * Ts0 * aux / g
		- Terrain_Dot(0, j, i);
	    }
	  else
	    {
	      aux = log(refpress / Piso);
 	      GridZ_Dot_in.Value(0, SigmaH.GetLength(1) - k - 1, j, i)
		= Ziso - R * Tiso * aux / g - Terrain_Dot(0, j, i);
	    }
	}

  // Frees memory.
  Terrain.Resize();
  Terrain_Dot.Resize();

  cout << " done." << endl;

  cout << "Converting from latlon to MM5 indices..."; cout.flush();

  // Converts from latitude/longitude to MM5 indices.
  LonlatToMM5LccInd<float> Lcc(BHI(0,5), BHI(0,4), BHR(0,10), BHR(0,9),
			       BHR(0,1), BHR(0,2), BHR(0,4), BHR(0,5),
			       BHR(0,0), BHI(0,19));
  
  // 3D output grids.
  for (j = 0; j < Ny_out; j++)
    for (i = 0; i < Nx_out; i++)
      Lcc(x_min_out + Delta_x_out * i, y_min_out + Delta_y_out * j,
	  GridX_3D_out.Value(0, 0, j, i), GridY_3D_out.Value(0, 0, j, i));
  
  // 2D output grids.
  for (j = 0; j < Ny_out; j++)
    for (i = 0; i < Nx_out; i++)
      {
	GridX_2D_out.Value(0, j, i) = GridX_3D_out.Value(0, 0, j, i);
	GridY_2D_out.Value(0, j, i) = GridY_3D_out.Value(0, 0, j, i);
      }

  // 3D output grids on interfaces.
  for (j = 0; j < Ny_out; j++)
    for (i = 0; i < Nx_out + 1; i++)
      Lcc(x_min_out + Delta_x_out * real(i) - 0.5,
	  y_min_out + Delta_y_out * j,
	  GridX_3D_interf_out.Value(0, 0, j, i),
	  GridY_3D_Gen_out.Value(0, 0, j, i));
  for (j = 0; j < Ny_out + 1; j++)
    for (i = 0; i < Nx_out; i++)
      Lcc(x_min_out + Delta_x_out * i,
	  y_min_out + Delta_y_out * real(j) - 0.5,
	  GridX_3D_Gen_out.Value(0, 0, j, i),
	  GridY_3D_interf_out.Value(0, 0, j, i));

  cout << " done." << endl;


  ///////////////////////////
  // INPUT DATA PROCESSING //
  ///////////////////////////

  cout << "Applying transformation to read fields..."; cout.flush();

  // MM5 3D fields.
  // FIXME : The grids are already defined. Is it a problem regarding the
  // order of variables since the grids were not related to any data
  // when created?
  Data<real, 4> PressurePerturbation(GridT_in, GridZ_MM5_in,
				     GridX_in, GridY_in);
  Data<real, 4> Pressure(GridT_in, GridZ_in, GridY_in, GridX_in);
  Data<real, 3> ReferencePressure(GridT_in, GridX_in, GridY_in);

  InputMeteo.ReadWholeField(file_in, "PSTARCRS", ReferencePressure);
  ReferencePressure.SwitchDimensions(shape(0, 2, 1),
				   GridT_in, GridY_in, GridX_in);

  InputMeteo.ReadWholeField(file_in, "PP", PressurePerturbation);
  PressurePerturbation.SwitchDimensions(shape(0, 1, 3, 2), GridT_in, GridZ_in,
					GridY_in, GridX_in);
  PressurePerturbation.ReverseData(1);

  Data<real, 4> Temperature(GridT_in, GridZ_MM5_in, GridX_in, GridY_in);

  InputMeteo.ReadWholeField(file_in, "T", Temperature);
  Temperature.SwitchDimensions(shape(0, 1, 3, 2), GridT_in, GridZ_in,
			       GridY_in, GridX_in);
  Temperature.ReverseData(1);

  Data<real, 3> SkinTemperature(GridT_in, GridX_in, GridY_in);
  InputMeteo.ReadWholeField(file_in, "GROUND T", SkinTemperature);
  SkinTemperature.SwitchDimensions(shape(0, 2, 1),
				   GridT_in, GridY_in, GridX_in);
  
  cout << " done." << endl;

  // Computes pressure.
  // See MM5 documentation, section 11.4.1.
  cout << "Computing pressure..."; cout.flush();

  for (h = 0; h < Nt_in; h++)
    for (k = 0; k < Nz_in; k++)
      for (j = 0; j < Ny_in - 1; j++)
	for (i = 0; i < Nx_in - 1; i++)
	  Pressure(h, k, j, i) = ReferencePressure(h, j, i)
	    * SigmaH(0, Nz_in - k - 1) + ptop
	    + PressurePerturbation(h, k, j, i);

  PressurePerturbation.Resize();
  ReferencePressure.Resize();
  
  cout << " done." << endl;

  // Computes surface pressure.
  // See MM5 documentation, section 7.3.
  cout << "Computing surface pressure..."; cout.flush();

  Data<real, 3> SurfacePressure(GridT_in, GridY_in, GridX_in);
  for (h = 0; h < Nt_in; h++)
    for (j = 0; j < Ny_in - 1; j++)
      for (i = 0; i < Nx_in - 1; i++)
	SurfacePressure(h, j, i) = Pressure(h, 0, j, i) +
	  (Pressure(h, 0, j, i) - Pressure(h, 1, j, i))
	  * GridZ_in.Value(0, 0, j, i)
	  / (GridZ_in.Value(0, 1, j, i) - GridZ_in.Value(0, 0, j, i));

  // Frees memory.
  Terrain.Resize();
  Terrain_Dot.Resize();
  
  cout << " done." << endl;

  // Interpolations.
  cout << "Interpolations..."; cout.flush();

  // Pressure.
  Data<real, 4> Pressure_tmp(GridT_in, GridZ_out, GridY_in, GridX_in);

  LinearInterpolationDimension(Pressure, Pressure_tmp, 1);
  LinearInterpolationRegularToGeneral(Pressure_tmp, Pressure_out);

  LinearInterpolationRegularToGeneral(SurfacePressure, SurfacePressure_out);

  // Temperature.
  Data<real, 4> Temperature_tmp(GridT_in, GridZ_out, GridY_in, GridX_in);

  LinearInterpolationDimension(Temperature, Temperature_tmp, 1);
  LinearInterpolationRegularToGeneral(Temperature_tmp, Temperature_out);

  // Winds.
  // Winds are given at dot points. Z at dots are differents from
  // Z at cross.
  Data<real, 4> MeridionalWind(GridT_in, GridZ_MM5_Dot_in,
			       GridX_interf_in, GridY_interf_in);
  Data<real, 4> ZonalWind(GridT_in, GridZ_MM5_Dot_in,
			  GridX_interf_in, GridY_interf_in);


  InputMeteo.ReadWholeField(file_in, "V", MeridionalWind);
  MeridionalWind.SwitchDimensions(shape(0, 1, 3, 2), GridT_in, GridZ_Dot_in,
				  GridY_interf_in, GridX_interf_in);
  MeridionalWind.ReverseData(1);

  InputMeteo.ReadWholeField(file_in, "U", ZonalWind);
  ZonalWind.SwitchDimensions(shape(0, 1, 3, 2), GridT_in, GridZ_Dot_in,
			     GridY_interf_in, GridX_interf_in);
  ZonalWind.ReverseData(1);

  Data<real, 4> MeridionalWind_tmp(GridT_in, GridZ_out,
				   GridY_interf_in, GridX_interf_in);
  Data<real, 4> ZonalWind_tmp(GridT_in, GridZ_out,
			      GridY_interf_in, GridX_interf_in);

  LinearInterpolationDimension(MeridionalWind, MeridionalWind_tmp, 1);
  LinearInterpolationRegularToGeneral(MeridionalWind_tmp, MeridionalWind_out);

  LinearInterpolationDimension(ZonalWind, ZonalWind_tmp, 1);
  LinearInterpolationRegularToGeneral(ZonalWind_tmp, ZonalWind_out);

  cout << " done." << endl;


  ///////////////////////
  // RICHARDSON NUMBER //
  ///////////////////////

  cout << "Computing Richardson number..."; cout.flush();

  Data<real, 4> PotentialTemperature(GridT_in, GridZ_out, GridY_in, GridX_in);
  Data<real, 3> SurfacePotentialTemperature(GridT_in, GridY_in, GridX_in);

  ComputePotentialTemperature(Temperature_tmp, Pressure_tmp, PotentialTemperature);
  ComputePotentialTemperature(SkinTemperature, SurfacePressure,
			      SurfacePotentialTemperature);

  Data<real, 4> Richardson(GridT_in, GridZ_out, GridY_in, GridX_in);
  Data<real, 3> SurfaceRichardson(GridT_in, GridY_in, GridX_in);

  Data<real, 4> MeridionalWind_cross(GridT_in, GridZ_out, GridY_in, GridX_in);
  Data<real, 4> ZonalWind_cross(GridT_in, GridZ_out, GridY_in, GridX_in);
  Data<real, 3> SurfaceWindModule(GridT_in, GridY_in, GridX_in);

  // Interpolates ZonalWind and Meridional wind at cross points.
  LinearInterpolationRegular(ZonalWind_tmp, ZonalWind_cross);
  LinearInterpolationRegular(MeridionalWind_tmp, MeridionalWind_cross);

  ZonalWind_tmp.Resize();
  MeridionalWind_tmp.Resize();

  // Richardson number.
  ComputeRichardson(ZonalWind_cross, MeridionalWind_cross,
		    PotentialTemperature, Richardson);
  ComputeModule(ZonalWind_cross, MeridionalWind_cross,
		SurfaceWindModule);
  ComputeRichardson(SurfaceWindModule, SurfacePotentialTemperature,
		    PotentialTemperature, SurfaceRichardson);

  PotentialTemperature.Resize();
  SurfacePotentialTemperature.Resize();
  Temperature_tmp.Resize();
  Pressure_tmp.Resize();
  SurfaceWindModule.Resize();

  ZonalWind_cross.Resize();
  MeridionalWind_cross.Resize();
  ZonalWind.Resize();
  MeridionalWind.Resize();

  LinearInterpolationRegularToGeneral(SurfaceRichardson,
				      SurfaceRichardson_out);
  SurfaceRichardson.Resize();

  LinearInterpolationRegularToGeneral(Richardson, Richardson_out);
  Richardson.Resize();

  cout << " done." << endl;


  /////////////////
  // ATTENUATION //
  /////////////////

  cout << "Computing attenuation..." << endl;

  cout << "  + Computing relative humidity and critical relative humidity..."; cout.flush();

  // Reads specific humidity.
  Data<real, 4> Humidity(GridT_in, GridZ_MM5_in, GridX_in, GridY_in);
  InputMeteo.ReadWholeField(file_in, "Q", Humidity);
  Humidity.SwitchDimensions(shape(0, 1, 3, 2), GridT_in, GridZ_in,
			    GridY_in, GridX_in);
  Humidity.ReverseData(1);
  Humidity.Threshold(0., 1.);

  // Relative humidity.
  Data<real, 4> RelativeHumidity(GridT_in, GridZ_in, GridY_in, GridX_in);
  ComputeRelativeHumidity(Humidity, Temperature, Pressure, RelativeHumidity);
  RelativeHumidity.ThresholdMax(1.);
  Temperature.Resize();

  // Critical relative humidity.
  Data<real, 4> CRH(GridT_in, GridZ_in, GridY_in, GridX_in);
  ComputeCriticalRelativeHumidity(SurfacePressure, Pressure, CRH);
  SurfacePressure.Resize();

  cout << " done." << endl;

  cout << "  + Computing cloud profile..."; cout.flush();

  Data<real, 3> BoundaryHeight(GridT_in, GridX_in, GridY_in);
  InputMeteo.ReadWholeField(file_in, "PBL HGT", BoundaryHeight);
  BoundaryHeight.SwitchDimensions(shape(0, 2, 1),
				  GridT_in, GridY_in, GridX_in);
  Data<real, 4> CloudFraction(GridT_in, GridZ_in, GridY_in, GridX_in);
  ComputeCloudFraction(BoundaryHeight, RelativeHumidity, CRH, CloudFraction);
  CRH.Resize();

  Data<int, 4> LowIndices(Nt_in, Ny_in, Nx_in, 2);
  Data<int, 4> MediumIndices(Nt_in, Ny_in, Nx_in, 2);
  Data<int, 4> HighIndices(Nt_in, Ny_in, Nx_in, 2);

  Data<real, 3> LowCloudiness(GridT_in, GridY_in, GridX_in);
  Data<real, 3> MediumCloudiness(GridT_in, GridY_in, GridX_in);
  Data<real, 3> HighCloudiness(GridT_in, GridY_in, GridX_in);

  ComputeCloudiness(CloudFraction, Pressure, GridZ_interf_in, LowIndices,
		    MediumIndices, HighIndices, LowCloudiness,
		    MediumCloudiness, HighCloudiness);
  Pressure.Resize();

  Data<real, 3> CloudHeight(GridT_in, GridY_in, GridX_in);
  ComputeCloudHeight(LowIndices, MediumIndices, HighIndices,
		     GridZ_interf_in, CloudHeight);

  cout << " done." << endl;

  cout << "  + Computing attenuation..."; cout.flush();

  // Reads liquid water content.
  Data<real, 4> LiquidWaterContent(GridT_in, GridZ_MM5_in,
				   GridX_in, GridY_in);
  InputMeteo.ReadWholeField(file_in, "CLW", LiquidWaterContent);
  LiquidWaterContent.SwitchDimensions(shape(0, 1, 3, 2), GridT_in, GridZ_in,
				      GridY_in, GridX_in);
  LiquidWaterContent.ReverseData(1);

  Data<real, 4> Attenuation(GridT_in, GridZ_in, GridY_in, GridX_in);
  ComputeAttenuation_LWC(LiquidWaterContent,
			 LowIndices, MediumIndices, HighIndices,
			 MediumCloudiness, HighCloudiness,
			 date.GetDate(), Attenuation);

  LowIndices.Resize();
  MediumIndices.Resize();
  HighIndices.Resize();

  LowCloudiness.Resize();
  MediumCloudiness.Resize();
  HighCloudiness.Resize();

  cout << " done." << endl;


  ///////////////////////////
  // LINEAR INTERPOLATIONS //
  ///////////////////////////

  cout << "Linear interpolations..." << endl;

  // Interpolates 3D fields with data at cross points.

  // Attenuation.
  Data<real, 4> Attenuation_tmp(GridT_in, GridZ_out, GridY_in, GridX_in);
  LinearInterpolationDimension(Attenuation, Attenuation_tmp, 1);
  LinearInterpolationRegularToGeneral(Attenuation_tmp, Attenuation_out);
  Attenuation_out.Threshold(0., 2.);
  Attenuation_tmp.Resize();
  Attenuation.Resize();
  cout << "  + Attenuation" << endl;

  // Humidity.
  Data<real, 4> Humidity_tmp(GridT_in, GridZ_out, GridY_in, GridX_in);
  LinearInterpolationDimension(Humidity, Humidity_tmp, 1);
  LinearInterpolationRegularToGeneral(Humidity_tmp, Humidity_out);
  Humidity_tmp.Resize();
  Humidity.Resize();
  cout << "  + Humidity" << endl;

  // LiquidWaterContent.
  Data<real, 4> LiquidWaterContent_tmp(GridT_in, GridZ_out,
				       GridY_in, GridX_in);
  LinearInterpolationDimension(LiquidWaterContent, LiquidWaterContent_tmp, 1);
  LinearInterpolationRegularToGeneral(LiquidWaterContent_tmp,
				      LiquidWaterContent_out);
  LiquidWaterContent_tmp.Resize();
  LiquidWaterContent.Resize();
  cout << "  + Liquid Water content" << endl;

  // Interpolates 2D Field (with data at cross points)

  // CloudHeight.
  LinearInterpolationRegularToGeneral(CloudHeight, CloudHeight_out);
  CloudHeight_out.ThresholdMin(min_height);
  CloudHeight.Resize();
  cout << "  + CloudHeight" << endl;

  // SurfaceTemperature.
  Data<real, 3> SurfaceTemperature(GridT_in, GridX_in, GridY_in);
  InputMeteo.ReadWholeField(file_in, "T2", SurfaceTemperature);
  SurfaceTemperature.SwitchDimensions(shape(0, 2, 1),
				      GridT_in, GridY_in, GridX_in);
  LinearInterpolationRegularToGeneral(SurfaceTemperature, SurfaceTemperature_out);
  SurfaceTemperature.Resize();
  cout << "  + SurfaceTemperature" << endl;

  // SkinTemperature.
  LinearInterpolationRegularToGeneral(SkinTemperature, SkinTemperature_out);
  SkinTemperature.Resize();
  cout << "  + SkinTemperature" << endl;

  // SoilWater.
  Data<real, 3> SoilWater(GridT_in, GridX_in, GridY_in);
  InputMeteo.ReadWholeField(file_in, "SOIL W 1", SoilWater);
  SoilWater.SwitchDimensions(shape(0, 2, 1),
			     GridT_in, GridY_in, GridX_in);
  SoilWater.Threshold(0., 1.);
  LinearInterpolationRegularToGeneral(SoilWater, SoilWater_out);
  SoilWater.Resize();
  cout << "  + SoilWater" << endl;

  // SensibleHeat.
  Data<real, 3> SensibleHeat(GridT_in, GridX_in, GridY_in);
  InputMeteo.ReadWholeField(file_in, "SHFLUX", SensibleHeat);
  SensibleHeat.SwitchDimensions(shape(0, 2, 1),
				GridT_in, GridY_in, GridX_in);
  LinearInterpolationRegularToGeneral(SensibleHeat, SensibleHeat_out);
  // Divides by (\rho_{air} * cp).
  for (h = 0; h < Nt_out; h++)
    for (j = 0; j < Ny_out; j++)
      for (i = 0; i < Nx_out; i++)
	SensibleHeat_out(h, j, i) *= r * SkinTemperature_out(h, j, i)
	  / (SurfacePressure_out(h, j, i) * cp);
  SensibleHeat.Resize();
  cout << "  + SensibleHeat" << endl;

  // Evaporation.
  Data<real, 3> LatentHeat(GridT_in, GridX_in, GridY_in);
  InputMeteo.ReadWholeField(file_in, "LHFLUX", LatentHeat);
  LatentHeat.SwitchDimensions(shape(0, 2, 1),
			      GridT_in, GridY_in, GridX_in);
  LinearInterpolationRegularToGeneral(LatentHeat, Evaporation_out);
  // Divides by \rho_{water} * L.
  Evaporation_out.Mlt(1./2.5e9);
  LatentHeat.Resize();
  cout << "  + Evaporation" << endl;

  // SolarRadiation.
  Data<real, 3> SolarRadiation(GridT_in, GridX_in, GridY_in);
  InputMeteo.ReadWholeField(file_in, "SWDOWN", SolarRadiation);
  SolarRadiation.SwitchDimensions(shape(0, 2, 1),
				  GridT_in, GridY_in, GridX_in);
  LinearInterpolationRegularToGeneral(SolarRadiation, SolarRadiation_out);
  SolarRadiation.Resize();
  cout << "  + SolarRadiation" << endl;

  // ConvectiveRain.
  Data<real, 3> ConvectiveRain(GridT_in, GridX_in, GridY_in);
  InputMeteo.ReadWholeField(file_in, "RAIN CON", ConvectiveRain);
  ConvectiveRain.SwitchDimensions(shape(0, 2, 1),
				  GridT_in, GridY_in, GridX_in);
  Decumulate(ConvectiveRain, Nt_in, 0);
  ConvectiveRain.ThresholdMin(0.);
  // Converts cm/step to mm/h.
  ConvectiveRain.Mlt(10. / Delta_t_in);

  // Rain.
  // First reads NonConvectiveRain.
  Data<real, 3> Rain(GridT_in, GridX_in, GridY_in);
  InputMeteo.ReadWholeField(file_in, "RAIN NON", Rain);
  Rain.SwitchDimensions(shape(0, 2, 1), GridT_in, GridY_in, GridX_in);
  Decumulate(Rain, Nt_in, 0);
  Rain.ThresholdMin(0.);
  // Converts cm/step to mm/h.
  Rain.Mlt(10. / Delta_t_in);

  // Adds convective rain to NonConvectiveRain.
  for (h = 0; h < Nt_in; h++)
    for (j = 0; j < Ny_in - 1; j++)
      for (i = 0; i < Nx_in - 1; i++)
	Rain(h, j, i) = Rain(h, j, i) + ConvectiveRain(h, j, i);

  // Interpolates to Rain_out.
  LinearInterpolationRegularToGeneral(Rain, Rain_out);
  Rain_out.ThresholdMin(0.);
  Rain.Resize();

  // Interpolates and frees ConvectiveRain.
  LinearInterpolationRegularToGeneral(ConvectiveRain, ConvectiveRain_out);
  ConvectiveRain_out.ThresholdMin(0.);
  ConvectiveRain.Resize();
  cout << "  + Rain" << endl;

  // FrictionModule.
  Data<real, 3> FrictionModule(GridT_in, GridX_in, GridY_in);
  InputMeteo.ReadWholeField(file_in, "UST", FrictionModule);
  FrictionModule.SwitchDimensions(shape(0, 2, 1),
				  GridT_in, GridY_in, GridX_in);
  LinearInterpolationRegularToGeneral(FrictionModule, FrictionModule_out);
  FrictionModule.Resize();
  cout << "  + FrictionModule" << endl;

  // BoundaryHeight.
  LinearInterpolationRegularToGeneral(BoundaryHeight, BoundaryHeight_out);
  BoundaryHeight.Resize();
  BoundaryHeight_out.ThresholdMin(GridZ_interf_out(1));
  cout << "  + BoundaryHeight" << endl;

  cout << " done." << endl;

  // Resizes grids with latlon coord.
  Pressure_out.ResizeGrid(GridT_out, GridZ_out, GridY_out, GridX_out);
  Temperature_out.ResizeGrid(GridT_out, GridZ_out, GridY_out, GridX_out);
  Humidity_out.ResizeGrid(GridT_out, GridZ_out, GridY_out, GridX_out);
  LiquidWaterContent_out.ResizeGrid(GridT_out, GridZ_out,
				    GridY_out, GridX_out);
  Richardson_out.ResizeGrid(GridT_out, GridZ_out, GridY_out, GridX_out);
  MeridionalWind_out.ResizeGrid(GridT_out, GridZ_out,
				GridY_interf_out, GridX_out);
  ZonalWind_out.ResizeGrid(GridT_out, GridZ_out,
			   GridY_out, GridX_interf_out);
  SurfaceTemperature_out.ResizeGrid(GridT_out, GridY_out, GridX_out);
  SurfacePressure_out.ResizeGrid(GridT_out, GridY_out, GridX_out);
  SkinTemperature_out.ResizeGrid(GridT_out, GridY_out, GridX_out);
  SoilWater_out.ResizeGrid(GridT_out, GridY_out, GridX_out);
  SolarRadiation_out.ResizeGrid(GridT_out, GridY_out, GridX_out);
  ConvectiveRain_out.ResizeGrid(GridT_out, GridY_out, GridX_out);
  Rain_out.ResizeGrid(GridT_out, GridY_out, GridX_out);
  FrictionModule_out.ResizeGrid(GridT_out, GridY_out, GridX_out);
  BoundaryHeight_out.ResizeGrid(GridT_out, GridY_out, GridX_out);
  SurfaceRichardson_out.ResizeGrid(GridT_out, GridY_out, GridX_out);  


  /////////////////////////
  // WINDS AND DIFFUSION //
  /////////////////////////

  // Vertical diffusion (Louis formula, 1979).
  cout << "Computing Kz..."; cout.flush();

#ifdef METEO_FINE_KZ
  // Linear interpolation, on a fine temporal grid,
  // of fields required to compute Kz.
  LinearInterpolationDimension(Temperature_out, Temperature_fine_out, 0);
  LinearInterpolationDimension(Pressure_out, Pressure_fine_out, 0);
  LinearInterpolationDimension(ZonalWind_out, ZonalWind_fine_out, 0);
  LinearInterpolationDimension(MeridionalWind_out,
			       MeridionalWind_fine_out, 0);
  VerticalWind_fine_out.SetZero();
  LinearInterpolationDimension(ConvectiveRain_out,
			       ConvectiveRain_fine_out, 0);

  ComputePotentialTemperature(Temperature_fine_out, Pressure_fine_out,
			      PotentialTemperature_fine_out);

  ComputeLouisKz(ZonalWind_fine_out, MeridionalWind_fine_out,
		 PotentialTemperature_fine_out, Kz_fine_out);
#else
  VerticalWind_out.SetZero();
  ComputePotentialTemperature(Temperature_out, Pressure_out,
			      PotentialTemperature_out);

  ComputeLouisKz(ZonalWind_out, MeridionalWind_out,
		 PotentialTemperature_out, Kz_out);
#endif
 
  real Kz_max_loc;
  int imax(0);

#ifdef METEO_FINE_KZ

  for (h=0; h<Nt_fine_out; h++)
    for (j=0; j<Ny_out; j++)
      for (i=0; i<Nx_out; i++)
	{
	  Kz_max_loc = 0.0;
	  for (k=0; k<Nz_out+1; k++)
	    if (Kz_fine_out(h, k, j, i) >= Kz_max_loc)
	      {
		Kz_max_loc = Kz_fine_out(h, k, j, i);
		imax = k;
	      }
	  if (ConvectiveRain_fine_out(h, j, i) > 1./6.)
	    for (k=imax; k<Nz_out+1; k++)
	      Kz_fine_out(h, k, j, i) = Kz_max_loc;
	  else
	    for (k=imax; k<Nz_out+1; k++)
	      Kz_fine_out(h, k, j, i) = (1. - ConvectiveRain_fine_out(h, j, i)
					 * 6.) * Kz_fine_out(h, k, j, i)
		+ 6. * ConvectiveRain_fine_out(h, j, i) * Kz_max_loc;
	}

  // Kz_min.
  if (!apply_vert)
    for (h=0; h<Nt_fine_out; h++)
      for (j=0; j<Ny_out; j++)
	for (i=0; i<Nx_out; i++)
	  {
	    if (Kz_fine_out(h, 1, j, i) < Kz_min)
	      Kz_fine_out(h, 1, j, i) = Kz_min;
	  }
  else
    for (h=0; h<Nt_fine_out; h++)
      for (k=1; k<Nz_out+1; k++)
	for (j=0; j<Ny_out; j++)
	  for (i=0; i<Nx_out; i++)
	    {
	      if (Kz_fine_out(h, k, j, i) < Kz_min)
		Kz_fine_out(h, k, j, i) = Kz_min;
	    }

#else

  for (h=0; h<Nt_out; h++)
    for (j=0; j<Ny_out; j++)
      for (i=0; i<Nx_out; i++)
	{
	  Kz_max_loc = 0.0;
	  for (k=0; k<Nz_out+1; k++)
	    if (Kz_out(h, k, j, i) >= Kz_max_loc)
	      {
		Kz_max_loc = Kz_out(h, k, j, i);
		imax = k;
	      }
	  if (ConvectiveRain_out(h, j, i) > 1./6.)
	    for (k=imax; k<Nz_out+1; k++)
	      Kz_out(h, k, j, i) = Kz_max_loc;
	  else
	    for (k=imax; k<Nz_out+1; k++)
	      Kz_out(h, k, j, i) = (1. - ConvectiveRain_out(h, j, i) * 6.)
		* Kz_out(h, k, j, i)
		+ 6. * ConvectiveRain_out(h, j, i) * Kz_max_loc;
	}

  // Kz_min.
  if (!apply_vert)
    for (h=0; h<Nt_out; h++)
      for (j=0; j<Ny_out; j++)
	for (i=0; i<Nx_out; i++)
	  {
	    if (Kz_out(h, 1, j, i) < Kz_min)
	      Kz_out(h, 1, j, i) = Kz_min;
	  }
  else
    Kz_out.ThresholdMin(Kz_min);

#endif

  // Kz_max.
#ifdef METEO_FINE_KZ
  Kz_fine_out.ThresholdMax(Kz_max);
#else
  Kz_out.ThresholdMax(Kz_max);
#endif
 
  cout << " done." << endl;
  

  /////////////////////////////////////////
  // PHOTOSYNTHETICALLY ACTIVE RADIATION //
  /////////////////////////////////////////

  // For biogenic emissions.
  cout << "Computing PAR..."; cout.flush();
  real zenith_angle;
  real optical_thickness, visible_beam, visible_diff,
    water_abs, beam, diff, visible_rad, IR_rad,
    ratio, ratio0;
  real fvb, fvd, PARdb, PARdiff;
  real watt2umol(4.6);

  for (h=0; h<Nt_out; h++)
    for (j=0; j<Ny_out; j++)
      for (i=0; i<Nx_out; i++)
	{

	  zenith_angle = ZenithAngle(GridX_out(i), GridY_out(j),
				     date.GetDate(),
				     real(GridT_out(h)
					  - int(GridT_out(h)/24) * 24.))
	    / 180. * pi;

	  if ( (zenith_angle>=1.51844) || (SolarRadiation_out(h, j, i)<=0.) )
	    PAR_out(h, j, i) = 0.;
	  else
	    {
	      optical_thickness = Pressure_out(h, 0, j, i)
		/ 101325. / cos(zenith_angle);
	      visible_beam = 600. * exp(-.185 * optical_thickness)
		* cos(zenith_angle);
	      visible_diff = 0.42 * (600. - visible_beam)
		* cos(zenith_angle);
	      water_abs = 1320. * .077 * pow(2. * optical_thickness, 0.3);
	      beam = (720. * exp(-0.06 * optical_thickness) - water_abs)
		* cos(zenith_angle);
	      diff = 0.65 * (720. - water_abs - beam) * cos(zenith_angle);

	      visible_rad = visible_beam + visible_diff;
	      IR_rad  = beam + diff;
	      ratio  = visible_rad / (visible_rad + IR_rad);
	      ratio0 = SolarRadiation_out(h, j, i) / (visible_rad + IR_rad);

	      if (ratio0 >= 0.89)
		fvb = visible_beam / visible_rad * 0.941124;
	      else if (ratio0 <= 0.21)
		fvb = visible_beam / visible_rad * 9.55e-3;
	      else
		fvb = visible_beam / visible_rad
		  * ( 1. - pow((0.9-ratio0)/0.7, 0.666667) );
	      fvd = 1. - fvb;

	      PARdb  = SolarRadiation_out(h, j, i) * ratio * fvb * watt2umol;
	      PARdiff = SolarRadiation_out(h, j, i) * ratio * fvd * watt2umol;

	      PARdb_out(h, j, i) = PARdb;
	      PARdiff_out(h, j, i) = PARdiff;
	      PAR_out(h, j, i) = PARdb + PARdiff;
	    }
	}

  cout << " done." << endl;


  ////////////////////////
  // WRITES OUTPUT DATA //
  ////////////////////////

  FormatBinary<float> OutputMeteo;

  cout << "Writing data..."; cout.flush();
  OutputMeteo.Append(Pressure_out, directory_out + "Pressure.bin");
  OutputMeteo.Append(SurfacePressure_out, directory_out
		     + "SurfacePressure.bin");
  OutputMeteo.Append(Temperature_out, directory_out + "Temperature.bin");
  OutputMeteo.Append(SurfaceTemperature_out,
		     directory_out + "SurfaceTemperature.bin");
  OutputMeteo.Append(SkinTemperature_out, directory_out
		     + "SkinTemperature.bin");
  OutputMeteo.Append(Richardson_out, directory_out + "Richardson.bin");
  OutputMeteo.Append(SurfaceRichardson_out,
		     directory_out + "SurfaceRichardson.bin");
  OutputMeteo.Append(Humidity_out, directory_out + "Humidity.bin");
  OutputMeteo.Append(Attenuation_out, directory_out + "Attenuation.bin");
  OutputMeteo.Append(CloudHeight_out, directory_out + "CloudHeight.bin");
  OutputMeteo.Append(SolarRadiation_out, directory_out
		     + "SolarRadiation.bin");
  OutputMeteo.Append(Rain_out, directory_out + "Rain.bin");
  OutputMeteo.Append(PARdb_out, directory_out + "PARdb.bin");
  OutputMeteo.Append(PARdiff_out, directory_out + "PARdiff.bin");
  OutputMeteo.Append(PAR_out, directory_out + "PAR.bin");
  OutputMeteo.Append(ZonalWind_out, directory_out + "ZonalWind.bin");
  OutputMeteo.Append(MeridionalWind_out, directory_out
		     + "MeridionalWind.bin");
  ComputeModule(MeridionalWind_out, ZonalWind_out, WindModule_out);
  OutputMeteo.Append(WindModule_out, directory_out + "WindModule.bin");
  TransformZonalWind(ZonalWind_out);
  TransformMeridionalWind(MeridionalWind_out);
  OutputMeteo.Append(ZonalWind_out, directory_out + "U.bin");
  OutputMeteo.Append(MeridionalWind_out, directory_out + "V.bin");
  OutputMeteo.Append(FrictionModule_out, directory_out
		     + "FrictionModule.bin");
  OutputMeteo.Append(BoundaryHeight_out, directory_out
		     + "BoundaryHeight.bin");
#ifdef METEO_FINE_KZ
  OutputMeteo.Append(Kz_fine_out, directory_out + "Kz_Louis.bin");
#else
  OutputMeteo.Append(Kz_out, directory_out + "Kz_Louis.bin");
#endif
  OutputMeteo.Append(SoilWater_out, directory_out + "SoilWater.bin");
  OutputMeteo.Append(SensibleHeat_out, directory_out + "SensibleHeat.bin");
  OutputMeteo.Append(Evaporation_out, directory_out + "Evaporation.bin");
  OutputMeteo.Append(PotentialTemperature_out, directory_out + "PT.bin");
  cout << " done." << endl;

  cout << endl;

  END;

  return 0;

}
