// Copyright (C) 2003-2004 CEREA
//     Author: Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of a simulation system for air quality.
// 
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


//////////////
// INCLUDES //

#include <cmath>
#include <iostream>
#include <algorithm>
using namespace std;

#define SELDONDATA_DEBUG_LEVEL_4
#define SELDONDATA_WITH_GRIB

#include "AtmoData.hxx"
using namespace AtmoData;

// INCLUDES //
//////////////


template <class T>
void exp_(T& x)
{
  x = exp(x);
}


int main(int argc, char** argv)
{

  TRY;
  
  cout << endl;

  string configuration_file("meteo.cfg"), sec_config_file("");

  if (argc != 2 && argc != 3 && argc != 4)
    {
      string mesg  = "Usage:\n";
      mesg += string("  ") + argv[0] + " [main configuration file] [secondary configuration file] [date]\n";
      mesg += string("  ") + argv[0] + " [main configuration file] [date]\n";
      mesg += string("  ") + argv[0] + " [date]\n\n";
      mesg += "Arguments:\n";
      mesg += "  [main configuration file] (optional): main configuration file. Default: meteo.cfg\n";
      mesg += "  [secondary configuration file] (optional): secondary configuration file.\n";
      mesg += "  [date]: date in format YYYYMMDD.\n";
      cout << mesg << endl;
      return 1;
    }

  if (argc == 4)
    sec_config_file = argv[2];
  if (argc == 3 || argc == 4)
    configuration_file = argv[1];

  if (!exists(configuration_file))
    throw string("Unable to find configuration file \"")
      + configuration_file + "\".";

  Date date(convert<int>(argv[argc - 1]));
  Date date_prev(date);
  date_prev.AddDays(-1);


  ////////////////////////
  // FIRST DECLARATIONS //
  ////////////////////////

  typedef float real;

  int h, i, j, k;

  // Constants.
  const real P0 = 101325.;


  /////////////////////////
  // CONFIGURATION FILES //
  /////////////////////////

  cout << "Reading configuration files..."; cout.flush(); 
 
  ConfigStreams configuration(configuration_file);
  if (exists(sec_config_file))
    configuration.AddFile(sec_config_file);

  // Input domain.
  int Nt_in, Nz_in, Ny_in, Nx_in;
  real Delta_t_in, Delta_y_in, Delta_x_in;
  real t_min_in, y_min_in, x_min_in;

  configuration.FindFromBeginning("[ECMWF]");

  configuration.PeekValue("Nt", Nt_in);
  configuration.PeekValue("Nz", Nz_in);
  configuration.PeekValue("Ny", Ny_in);
  configuration.PeekValue("Nx", Nx_in);

  configuration.PeekValue("Delta_t", Delta_t_in);
  configuration.PeekValue("Delta_y", Delta_y_in);
  configuration.PeekValue("Delta_x", Delta_x_in);

  configuration.PeekValue("t_min", t_min_in);
  configuration.PeekValue("y_min", y_min_in);
  configuration.PeekValue("x_min", x_min_in);

  // Output domain.
  int Nt_out, Nz_out, Ny_out, Nx_out;
  real Delta_t_out, Delta_y_out, Delta_x_out;
  real t_min_out, y_min_out, x_min_out;
  string vertical_levels;

  configuration.FindFromBeginning("[domain]");

  configuration.PeekValue("Nt", Nt_out);
  configuration.PeekValue("Nx", Nx_out);
  configuration.PeekValue("Ny", Ny_out);
  configuration.PeekValue("Nz", Nz_out);
  configuration.PeekValue("Delta_t", Delta_t_out);
  configuration.PeekValue("Delta_y", Delta_y_out);
  configuration.PeekValue("Delta_x", Delta_x_out);
  configuration.PeekValue("t_min", t_min_out);
  configuration.PeekValue("y_min", y_min_out);
  configuration.PeekValue("x_min", x_min_out);
  configuration.PeekValue("vertical_levels", vertical_levels);
  
  // Files.
  string directory_in, directory_out;

  configuration.FindFromBeginning("[meteo_files]");

  configuration.PeekValue("directory_in", directory_in);
  configuration.PeekValue("directory_out", directory_out);

  string file_in = directory_in + date.GetDate("ECMWF-%y%m%d.grb");
  string file_in_prev = directory_in + date_prev.GetDate("ECMWF-%y%m%d.grb");

  // Clouds.
  real min_height;

  configuration.FindFromBeginning("[clouds]");

  configuration.PeekValue("min_height", min_height);


  cout << " done." << endl;


  ///////////
  // GRIDS //
  ///////////

  cout << "Memory allocation for data fields..."; cout.flush();

  // Input settings.

  // Input grids.
  RegularGrid<real> GridT_in(t_min_in, Delta_t_in, Nt_in);
  // Vertical levels depend on t, z, y and x.
  GeneralGrid<real, 4> GridZ_in(shape(Nt_in, Nz_in, Ny_in, Nx_in),
				1, shape(0, 1, 2, 3));
  GeneralGrid<real, 4> GridZ_interf_in(shape(Nt_in, Nz_in + 1, Ny_in, Nx_in),
				       1, shape(0, 1, 2, 3));
  RegularGrid<real> GridY_in(y_min_in, Delta_y_in, Ny_in);
  RegularGrid<real> GridX_in(x_min_in, Delta_x_in, Nx_in);
  // Vertical levels are shared.
  GridZ_in.SetDuplicate(false);
  
  // Output settings.

  // Output grids.
  RegularGrid<real> GridT_out(t_min_out, Delta_t_out, Nt_out);
  RegularGrid<real> GridZ_out(Nz_out);
  RegularGrid<real> GridY_out(y_min_out, Delta_y_out, Ny_out);
  RegularGrid<real> GridX_out(x_min_out, Delta_x_out, Nx_out);

  // Data may be provided on interfaces.
  RegularGrid<real> GridZ_interf_out(Nz_out + 1);

  // Reads output altitudes.
  FormatText Heights_out;
  Heights_out.Read(vertical_levels, GridZ_interf_out);
  // Sets values at nodes.
  for (k=0; k<Nz_out; k++)
    GridZ_out(k) = ( GridZ_interf_out(k) + GridZ_interf_out(k+1) ) / 2.0;


  //////////
  // DATA //
  //////////

  // Input fields.
  Data<real, 4> Temperature(GridT_in, GridZ_in, GridY_in, GridX_in);
  Data<real, 3> SurfaceTemperature(GridT_in, GridY_in, GridX_in);
  Data<real, 4> Pressure(GridT_in, GridZ_in, GridY_in, GridX_in);
  Data<real, 4> Pressure_interf(GridT_in, GridZ_interf_in,
				GridY_in, GridX_in);
  Data<real, 3> SurfacePressure(GridT_in, GridY_in, GridX_in);
  Data<real, 4> Humidity(GridT_in, GridZ_in, GridY_in, GridX_in);
  Data<real, 4> RelativeHumidity(GridT_in, GridZ_in, GridY_in, GridX_in);
  Data<real, 4> LiquidWaterContent(GridT_in, GridZ_in, GridY_in, GridX_in);
  Data<real, 4> IceWaterContent(GridT_in, GridZ_in, GridY_in, GridX_in);
  Data<real, 3> LowCloudiness(GridT_in, GridY_in, GridX_in);
  Data<real, 3> LowCloudiness_diag(GridT_in, GridY_in, GridX_in);
  Data<real, 3> MediumCloudiness(GridT_in, GridY_in, GridX_in);
  Data<real, 3> MediumCloudiness_diag(GridT_in, GridY_in, GridX_in);
  Data<real, 3> HighCloudiness(GridT_in, GridY_in, GridX_in);
  Data<real, 3> HighCloudiness_diag(GridT_in, GridY_in, GridX_in);
  Data<real, 3> BoundaryLayerHeight(GridT_in, GridY_in, GridX_in);
  Data<real, 4> CloudFraction(GridT_in, GridZ_in, GridY_in, GridX_in);
  Data<real, 4> CRH(GridT_in, GridZ_in, GridY_in, GridX_in);
  Data<real, 4> Attenuation(GridT_in, GridZ_in, GridY_in, GridX_in);
  Data<real, 3> CloudHeight(GridT_in, GridY_in, GridX_in);

  Data<real, 3> ConvectiveRain(GridT_in, GridY_in, GridX_in);
  Data<real, 3> ConvectiveRain_prev(GridT_in, GridY_in, GridX_in);
  Data<real, 3> LargeScaleRain(GridT_in, GridY_in, GridX_in);
  Data<real, 3> LargeScaleRain_prev(GridT_in, GridY_in, GridX_in);
  
  // Output fields.
  Data<real, 4> Attenuation_out(GridT_out, GridZ_out, GridY_out, GridX_out);
  Data<real, 3> CloudHeight_out(GridT_out, GridY_out, GridX_out);

  Data<real, 3> ConvectiveRain_out(GridT_out, GridY_out, GridX_out);
  Data<real, 3> LargeScaleRain_out(GridT_out, GridY_out, GridX_out);

  cout << " done." << endl;
  cout << endl;


  /////////////////
  // READS INPUT //
  /////////////////

  FormatGrib InputMeteo;

  cout << "Extracting data..."; cout.flush();

  InputMeteo.Read(file_in, 167, SurfaceTemperature);
  SurfaceTemperature.ReverseData(1);

  InputMeteo.Read(file_in, 152, SurfacePressure);
  SurfacePressure.ReverseData(1);

  InputMeteo.Read(file_in, 130, Temperature);
  Temperature.ReverseData(1);
  Temperature.ReverseData(2);

  InputMeteo.Read(file_in, 133, Humidity);
  Humidity.ReverseData(1);
  Humidity.ReverseData(2);

  InputMeteo.Read(file_in, 246, LiquidWaterContent);
  LiquidWaterContent.ReverseData(1);
  LiquidWaterContent.ReverseData(2);

  InputMeteo.Read(file_in, 246, IceWaterContent);
  IceWaterContent.ReverseData(1);
  IceWaterContent.ReverseData(2);

  InputMeteo.Read(file_in, 186, LowCloudiness);
  MediumCloudiness.ReverseData(1);

  InputMeteo.Read(file_in, 187, MediumCloudiness);
  MediumCloudiness.ReverseData(1);

  InputMeteo.Read(file_in, 188, HighCloudiness);
  HighCloudiness.ReverseData(1);

  InputMeteo.Read(file_in, 143, ConvectiveRain);
  ConvectiveRain.ReverseData(1);

  InputMeteo.Read(file_in_prev, 143, ConvectiveRain_prev);
  ConvectiveRain_prev.ReverseData(1);

  InputMeteo.Read(file_in, 142, LargeScaleRain);
  LargeScaleRain.ReverseData(1);

  InputMeteo.Read(file_in_prev, 142, LargeScaleRain_prev);
  LargeScaleRain_prev.ReverseData(1);

  InputMeteo.Read(file_in, 159, BoundaryLayerHeight);
  BoundaryLayerHeight.ReverseData(1);

  // Transformations.

  SurfacePressure.Apply(exp_);

  Humidity.Threshold(0., 1.);
  ConvectiveRain.ThresholdMin(0.);
  ConvectiveRain_prev.ThresholdMin(0.);
  LargeScaleRain.ThresholdMin(0.);
  LargeScaleRain_prev.ThresholdMin(0.);
  Humidity.Threshold(0., 1.);

  Decumulate(ConvectiveRain, 4, 1);
  for (j=0; j<Ny_in; j++)
    for (i=0; i<Nx_in; i++)
      ConvectiveRain(0, j, i) -= ConvectiveRain_prev(Nt_in - 2, j, i);
  // To mm/h.
  ConvectiveRain.Mlt(1000./Delta_t_in);

  Decumulate(LargeScaleRain, 4, 1);
  for (j=0; j<Ny_in; j++)
    for (i=0; i<Nx_in; i++)
      LargeScaleRain(0, j, i) -= LargeScaleRain_prev(Nt_in - 2, j, i);
  // To mm/h.
  LargeScaleRain.Mlt(1000./Delta_t_in);

  cout << " done." << endl;


  ///////////////////////////
  // INPUT DATA PROCESSING //
  ///////////////////////////

  // Computes level heights with pressure levels.
  cout << "Computing level heights in meter..."; cout.flush();

  Data<real, 1> alpha(61), beta(61);
  
  FormatFormattedText InputCoefficients("<e><e><e><e 2>");
  InputCoefficients.Read(directory_in + "hybrid_coefficients.dat",
			 "1", alpha);
  InputCoefficients.Read(directory_in + "hybrid_coefficients.dat", "2", beta);

  alpha.ReverseData(0);
  alpha.Mlt(1. / P0);
  beta.ReverseData(0);

  ComputePressure(alpha, beta, SurfacePressure, Pressure_interf);
  
  for (h=0; h<Nt_in; h++)
    for (k=0; k<Nz_in; k++)
      for (j=0; j<Ny_in; j++)
	for (i=0; i<Nx_in; i++)
	  Pressure(h, k, j, i) = (Pressure_interf(h, k+1, j, i)
				  + Pressure_interf(h, k, j, i)) / 2.0;
  
  ComputeInterfHeight(Pressure_interf, Temperature, GridZ_interf_in);
  ComputeMiddleHeight(Pressure_interf, Temperature,
		      GridZ_interf_in, GridZ_in);

  cout << " done." << endl;


  /////////////////
  // ATTENUATION //
  /////////////////

  cout << "Computing relative humidity and critical relative humidity..."; cout.flush();
  
  ComputeRelativeHumidity(Humidity, Temperature, Pressure, RelativeHumidity);
  RelativeHumidity.ThresholdMax(1.);
  ComputeCriticalRelativeHumidity(SurfacePressure, Pressure, CRH);

  cout << " done." << endl;

  cout << "Computing cloud profile..."; cout.flush();

  ComputeCloudFraction(BoundaryLayerHeight, RelativeHumidity, CRH, CloudFraction);

  Data<int, 4> LowIndices(Nt_in, Ny_in, Nx_in, 2);
  Data<int, 4> MediumIndices(Nt_in, Ny_in, Nx_in, 2);
  Data<int, 4> HighIndices(Nt_in, Ny_in, Nx_in, 2);

  ComputeCloudiness(CloudFraction, Pressure, GridZ_interf_in, LowIndices, MediumIndices,
		    HighIndices, LowCloudiness_diag, MediumCloudiness_diag,
		    HighCloudiness_diag);
  ComputeCloudHeight(LowIndices, MediumIndices, HighIndices, GridZ_interf_in, CloudHeight);

  cout << " done." << endl;

  cout << "Computing attenuation..."; cout.flush();

  ComputeAttenuation_LWC(LiquidWaterContent,
			 LowIndices, MediumIndices, HighIndices,
			 MediumCloudiness, HighCloudiness,
			 date.GetDate(), Attenuation);

  LinearInterpolationOneGeneral(Attenuation, Attenuation_out, 1);
  Attenuation_out.Threshold(0., 2.);

  cout << " done." << endl;
  cout << endl;


  ///////////////////////////
  // LINEAR INTERPOLATIONS //
  ///////////////////////////

  cout << "Linear interpolations..."; cout.flush();
  LinearInterpolationOneGeneral(Attenuation, Attenuation_out, 1);
  Attenuation_out.Threshold(0., 2.);
  LinearInterpolationRegular(CloudHeight, CloudHeight_out);
  CloudHeight_out.ThresholdMin(min_height);

  LinearInterpolationRegular(ConvectiveRain, ConvectiveRain_out);
  LinearInterpolationRegular(LargeScaleRain, LargeScaleRain_out);

  ConvectiveRain_out.ThresholdMin(0.);
  LargeScaleRain_out.ThresholdMin(0.);
  cout << " done." << endl;


  ////////////////////////
  // WRITES OUTPUT DATA //
  ////////////////////////

  FormatBinary<float> OutputMeteo;

  cout << "Writing data..."; cout.flush();
  OutputMeteo.Append(Attenuation_out, directory_out + "Attenuation.bin");
  OutputMeteo.Append(CloudHeight_out, directory_out + "CloudHeight.bin");
  OutputMeteo.Append(ConvectiveRain_out,
		     directory_out + "ConvectiveRain.bin");
  LargeScaleRain_out.GetArray() = LargeScaleRain_out.GetArray()
    + ConvectiveRain_out.GetArray();
  OutputMeteo.Append(LargeScaleRain_out, directory_out + "Rain.bin");
  cout << " done." << endl;

  cout << endl;

  END;

  return 0;

}
