// Copyright (C) 2003-2004 CEREA
//     Author: Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of a simulation system for air quality.
// 
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


//////////////
// INCLUDES //

#include <cmath>
#include <iostream>
#include <algorithm>
using namespace std;

#define SELDONDATA_DEBUG_LEVEL_4
#define SELDONDATA_WITH_GRIB

#include "AtmoData.hxx"
using namespace AtmoData;

// INCLUDES //
//////////////


template <class T>
void exp_(T& x)
{
  x = exp(x);
}

template <class T>
void sqrt_(T& x)
{
  x = sqrt(x);
}

template <class T>
void abs_(T& x)
{
  x = abs(x);
}

// Critical relative humidity.
template <class T>
T CriticalRelativeHumidity(const T& altitude, const T& pressure,
			   const T& ref_pressure)
{
  return 0.85;
}


int main(int argc, char** argv)
{

  TRY;
  
  cout << endl;

  string configuration_file("meteo.cfg"), sec_config_file("");

  if (argc != 2 && argc != 3 && argc != 4)
    {
      string mesg  = "Usage:\n";
      mesg += string("  ") + argv[0] + " [main configuration file] [secondary configuration file] [date]\n";
      mesg += string("  ") + argv[0] + " [main configuration file] [date]\n";
      mesg += string("  ") + argv[0] + " [date]\n\n";
      mesg += "Arguments:\n";
      mesg += "  [main configuration file] (optional): main configuration file. Default: meteo.cfg\n";
      mesg += "  [secondary configuration file] (optional): secondary configuration file.\n";
      mesg += "  [date]: date in format YYYYMMDD.\n";
      cout << mesg << endl;
      return 1;
    }

  if (argc == 4)
    sec_config_file = argv[2];
  if (argc == 3 || argc == 4)
    configuration_file = argv[1];

  if (!exists(configuration_file))
    throw string("Unable to find configuration file \"")
      + configuration_file + "\".";

  Date date(convert<int>(argv[argc - 1]));
  Date date_prev(date);
  date_prev.AddDays(-1);


  ////////////////////////
  // FIRST DECLARATIONS //
  ////////////////////////

  typedef float real;

  int h, i, j, k;

  // Constants.
  const real pi = 3.14159265358979323846264;
  const real P0 = 101325.;
  const real r = 287.;
  const real cp = 1005.;


  /////////////////////////
  // CONFIGURATION FILES //
  /////////////////////////

  cout << "Reading configuration files..."; cout.flush(); 
 
  ConfigStreams configuration(configuration_file);
  if (exists(sec_config_file))
    configuration.AddFile(sec_config_file);

  // Input domain.
  int Nt_in, Nz_in, Ny_in, Nx_in;
  real Delta_t_in, Delta_y_in, Delta_x_in;
  real t_min_in, y_min_in, x_min_in;

  configuration.FindFromBeginning("[ECMWF]");

  configuration.PeekValue("Nt", Nt_in);
  configuration.PeekValue("Nz", Nz_in);
  configuration.PeekValue("Ny", Ny_in);
  configuration.PeekValue("Nx", Nx_in);

  configuration.PeekValue("Delta_t", Delta_t_in);
  configuration.PeekValue("Delta_y", Delta_y_in);
  configuration.PeekValue("Delta_x", Delta_x_in);

  configuration.PeekValue("t_min", t_min_in);
  configuration.PeekValue("y_min", y_min_in);
  configuration.PeekValue("x_min", x_min_in);

  // Output domain.
  int Nt_out, Nz_out, Ny_out, Nx_out;
  real Delta_t_out, Delta_y_out, Delta_x_out;
  real t_min_out, y_min_out, x_min_out;
  string vertical_levels;

  configuration.FindFromBeginning("[domain]");

  configuration.PeekValue("Nt", Nt_out);
  configuration.PeekValue("Nx", Nx_out);
  configuration.PeekValue("Ny", Ny_out);
  configuration.PeekValue("Nz", Nz_out);
  configuration.PeekValue("Delta_t", Delta_t_out);
  configuration.PeekValue("Delta_y", Delta_y_out);
  configuration.PeekValue("Delta_x", Delta_x_out);
  configuration.PeekValue("t_min", t_min_out);
  configuration.PeekValue("y_min", y_min_out);
  configuration.PeekValue("x_min", x_min_out);
  configuration.PeekValue("vertical_levels", vertical_levels);
  
  // Files.
  string directory_in, directory_out;
  string roughness_in_file;

  configuration.FindFromBeginning("[meteo_files]");

  configuration.PeekValue("directory_in", directory_in);
  configuration.PeekValue("directory_out", directory_out);

  configuration.PeekValue("Roughness_in", roughness_in_file);

  string file_in = directory_in + date.GetDate("ECMWF-%y%m%d.grb");
  string file_in_prev = directory_in + date_prev.GetDate("ECMWF-%y%m%d.grb");

  // Files.
  bool richardson_with_roughness;

  configuration.FindFromBeginning("[meteo]");

  configuration.PeekValue("Richardson_with_roughness",
			  richardson_with_roughness);

  cout << " done." << endl;


  ///////////
  // GRIDS //
  ///////////

  cout << "Memory allocation for data fields..."; cout.flush();

  // Input settings.

  // Input grids.
  RegularGrid<real> GridT_in(t_min_in, Delta_t_in, Nt_in);
  // Vertical levels depend on t, z, y and x.
  GeneralGrid<real, 4> GridZ_in(shape(Nt_in, Nz_in, Ny_in, Nx_in),
				1, shape(0, 1, 2, 3));
  GeneralGrid<real, 4> GridZ_interf_in(shape(Nt_in, Nz_in + 1, Ny_in, Nx_in),
				       1, shape(0, 1, 2, 3));
  RegularGrid<real> GridY_in(y_min_in, Delta_y_in, Ny_in);
  RegularGrid<real> GridX_in(x_min_in, Delta_x_in, Nx_in);
  // Vertical levels are shared.
  GridZ_in.SetDuplicate(false);
  
  // Output settings.

  // Output grids.
  RegularGrid<real> GridT_out(t_min_out, Delta_t_out, Nt_out);
  RegularGrid<real> GridZ_out(Nz_out);
  RegularGrid<real> GridY_out(y_min_out, Delta_y_out, Ny_out);
  RegularGrid<real> GridX_out(x_min_out, Delta_x_out, Nx_out);

  // Data may be provided on interfaces.
  RegularGrid<real> GridZ_interf_out(Nz_out + 1);
  RegularGrid<real> GridY_interf_out(y_min_out - Delta_y_out / 2.,
				     Delta_y_out, Ny_out + 1);
  RegularGrid<real> GridX_interf_out(x_min_out - Delta_x_out / 2.,
				     Delta_x_out, Nx_out + 1);

  // Reads output altitudes.
  FormatText Heights_out;
  Heights_out.Read(vertical_levels, GridZ_interf_out);
  // Sets values at nodes.
  for (k=0; k<Nz_out; k++)
    GridZ_out(k) = ( GridZ_interf_out(k) + GridZ_interf_out(k+1) ) / 2.0;


  //////////
  // DATA //
  //////////

  // Input fields.
  Data<real, 2> Roughness(GridY_in, GridX_in);
  Data<real, 4> Temperature(GridT_in, GridZ_in, GridY_in, GridX_in);
  Data<real, 3> SurfaceTemperature(GridT_in, GridY_in, GridX_in);
  Data<real, 3> SkinTemperature(GridT_in, GridY_in, GridX_in);
  Data<real, 4> PotentialTemperature(GridT_in, GridZ_in, GridY_in, GridX_in);
  Data<real, 3> SurfacePotentialTemperature(GridT_in, GridY_in, GridX_in);
  Data<real, 3> SurfaceRichardson(GridT_in, GridY_in, GridX_in);
  Data<real, 4> Richardson(GridT_in, GridZ_in, GridY_in, GridX_in);
  Data<real, 4> Pressure(GridT_in, GridZ_in, GridY_in, GridX_in);
  Data<real, 4> Pressure_interf(GridT_in, GridZ_interf_in,
				GridY_in, GridX_in);
  Data<real, 3> SurfacePressure(GridT_in, GridY_in, GridX_in);
  Data<real, 4> Humidity(GridT_in, GridZ_in, GridY_in, GridX_in);
  Data<real, 4> LiquidWaterContent(GridT_in, GridZ_in, GridY_in, GridX_in);
  Data<real, 3> MediumCloudiness(GridT_in, GridY_in, GridX_in);
  Data<real, 3> HighCloudiness(GridT_in, GridY_in, GridX_in);
  Data<real, 4> MeridionalWind(GridT_in, GridZ_in, GridY_in, GridX_in);
  Data<real, 4> ZonalWind(GridT_in, GridZ_in, GridY_in, GridX_in);
  Data<real, 3> SurfaceWindModule(GridT_in, GridY_in, GridX_in);
  Data<real, 3> SolarRadiation(GridT_in, GridY_in, GridX_in);
  Data<real, 3> SolarRadiation_prev(GridT_in, GridY_in, GridX_in);
  Data<real, 3> BoundaryHeight(GridT_in, GridY_in, GridX_in);
  Data<real, 3> U_star(GridT_in, GridY_in, GridX_in);
  Data<real, 3> V_star(GridT_in, GridY_in, GridX_in);
  Data<real, 3> U_star_prev(GridT_in, GridY_in, GridX_in);
  Data<real, 3> V_star_prev(GridT_in, GridY_in, GridX_in);
  Data<real, 3> SoilWater(GridT_in, GridY_in, GridX_in);
  Data<real, 3> SensibleHeat(GridT_in, GridY_in, GridX_in);
  Data<real, 3> SensibleHeat_prev(GridT_in, GridY_in, GridX_in);
  Data<real, 3> Evaporation(GridT_in, GridY_in, GridX_in);
  Data<real, 3> Evaporation_prev(GridT_in, GridY_in, GridX_in);
  
  // Output fields.
  Data<real, 4> Temperature_out(GridT_out, GridZ_out, GridY_out, GridX_out);
  Data<real, 3> SurfaceTemperature_out(GridT_out, GridY_out, GridX_out);
  Data<real, 3> SkinTemperature_out(GridT_out, GridY_out, GridX_out);
  Data<real, 3> SurfaceRichardson_out(GridT_out, GridY_out, GridX_out);
  Data<real, 4> Richardson_out(GridT_out, GridZ_out, GridY_out, GridX_out);
  Data<real, 4> Pressure_out(GridT_out, GridZ_out, GridY_out, GridX_out);
  Data<real, 3> SurfacePressure_out(GridT_out, GridY_out, GridX_out);
  Data<real, 4> Humidity_out(GridT_out, GridZ_out, GridY_out, GridX_out);
  Data<real, 4> MeridionalWind_out(GridT_out, GridZ_out,
				   GridY_interf_out, GridX_out);
  Data<real, 4> ZonalWind_out(GridT_out, GridZ_out,
			      GridY_out, GridX_interf_out);
  Data<real, 3> BoundaryHeight_out(GridT_out, GridY_out, GridX_out);
  Data<real, 3> U_star_out(GridT_out, GridY_out, GridX_out);
  Data<real, 3> V_star_out(GridT_out, GridY_out, GridX_out);
  Data<real, 4> WindModule_out(GridT_out, GridZ_out, GridY_out, GridX_out);
  Data<real, 3> FrictionModule_out(GridT_out, GridY_out, GridX_out);
  Data<real, 3> SolarRadiation_out(GridT_out, GridY_out, GridX_out);
  Data<real, 3> PARdb_out(GridT_out, GridY_out, GridX_out);
  Data<real, 3> PARdiff_out(GridT_out, GridY_out, GridX_out);
  Data<real, 3> PAR_out(GridT_out, GridY_out, GridX_out);
  Data<real, 3> SoilWater_out(GridT_out, GridY_out, GridX_out);
  Data<real, 3> SensibleHeat_out(GridT_out, GridY_out, GridX_out);
  Data<real, 3> Evaporation_out(GridT_out, GridY_out, GridX_out);

  cout << " done." << endl;
  cout << endl;


  /////////////////
  // READS INPUT //
  /////////////////

  FormatBinary<float> InputBinaryMeteo;
  FormatGrib InputMeteo;

  cout << "Extracting data..."; cout.flush();

  if (richardson_with_roughness)
    InputBinaryMeteo.Read(roughness_in_file, Roughness);

  InputMeteo.Read(file_in, 167, SurfaceTemperature);
  SurfaceTemperature.ReverseData(1);

  InputMeteo.Read(file_in, 235, SkinTemperature);
  SkinTemperature.ReverseData(1);

  InputMeteo.Read(file_in, 152, SurfacePressure);
  SurfacePressure.ReverseData(1);

  InputMeteo.Read(file_in, 130, Temperature);
  Temperature.ReverseData(1);
  Temperature.ReverseData(2);

  InputMeteo.Read(file_in, 133, Humidity);
  Humidity.ReverseData(1);
  Humidity.ReverseData(2);

  InputMeteo.Read(file_in, 246, LiquidWaterContent);
  LiquidWaterContent.ReverseData(1);
  LiquidWaterContent.ReverseData(2);

  InputMeteo.Read(file_in, 187, MediumCloudiness);
  MediumCloudiness.ReverseData(1);

  InputMeteo.Read(file_in, 188, HighCloudiness);
  HighCloudiness.ReverseData(1);

  InputMeteo.Read(file_in, 132, MeridionalWind);
  MeridionalWind.ReverseData(1);
  MeridionalWind.ReverseData(2);

  InputMeteo.Read(file_in, 131, ZonalWind);
  ZonalWind.ReverseData(1);
  ZonalWind.ReverseData(2);

  InputMeteo.Read(file_in, 180, U_star);
  U_star.ReverseData(1);

  InputMeteo.Read(file_in, 181, V_star);
  V_star.ReverseData(1);

  InputMeteo.Read(file_in_prev, 180, U_star_prev);
  U_star_prev.ReverseData(1);

  InputMeteo.Read(file_in_prev, 181, V_star_prev);
  V_star_prev.ReverseData(1);

  InputMeteo.Read(file_in, 169, SolarRadiation);
  SolarRadiation.ReverseData(1);

  InputMeteo.Read(file_in_prev, 169, SolarRadiation_prev);
  SolarRadiation_prev.ReverseData(1);

  InputMeteo.Read(file_in, 159, BoundaryHeight);
  BoundaryHeight.ReverseData(1);

  InputMeteo.Read(file_in, 39, SoilWater);
  SoilWater.ReverseData(1);

  InputMeteo.Read(file_in, 146, SensibleHeat);
  SensibleHeat.ReverseData(1);

  InputMeteo.Read(file_in_prev, 146, SensibleHeat_prev);
  SensibleHeat_prev.ReverseData(1);

  InputMeteo.Read(file_in, 182, Evaporation);
  Evaporation.ReverseData(1);

  InputMeteo.Read(file_in_prev, 182, Evaporation_prev);
  Evaporation_prev.ReverseData(1);

  // Transformations.

  SurfacePressure.Apply(exp_);

  Humidity.Threshold(0., 1.);
  SoilWater.Threshold(0., 1.);
  SolarRadiation.ThresholdMin(0.);
  SolarRadiation_prev.ThresholdMin(0.);

  Decumulate(U_star, 4, 1);
  Decumulate(V_star, 4, 1);
  for (j=0; j<Ny_in; j++)
    for (i=0; i<Nx_in; i++)
      {
	U_star(0, j, i) -= U_star_prev(Nt_in - 2, j, i);
	V_star(0, j, i) -= V_star_prev(Nt_in - 2, j, i);
      }
  U_star.Mlt(1./(Delta_t_in * 3600.));
  V_star.Mlt(1./(Delta_t_in * 3600.));

  // Stress to friction velocity.
  // Divides by \rho_{air}.
  for (h=0; h<Nt_in; h++)
    for (j=0; j<Ny_in; j++)
      for (i=0; i<Nx_in; i++)
	{
	  U_star(h, j, i) *= r * SkinTemperature(h, j, i)
	    / SurfacePressure(h, j, i);
	  V_star(h, j, i) *= r * SkinTemperature(h, j, i)
	    / SurfacePressure(h, j, i);
	}

  Decumulate(SolarRadiation, 4, 1);
  for (j=0; j<Ny_in; j++)
    for (i=0; i<Nx_in; i++)
      SolarRadiation(0, j, i) -= SolarRadiation_prev(Nt_in - 2, j, i);
  SolarRadiation.Mlt(1./(Delta_t_in * 3600.));
  SolarRadiation.ThresholdMin(0.);

  Decumulate(Evaporation, 4, 1);
  for (j=0; j<Ny_in; j++)
    for (i=0; i<Nx_in; i++)
      Evaporation(0, j, i) -= Evaporation_prev(Nt_in - 2, j, i);
  // Multiplies by \rho_{water} / \rho_{air}.
  for (h=0; h<Nt_in; h++)
    for (j=0; j<Ny_in; j++)
      for (i=0; i<Nx_in; i++)
	Evaporation(h, j, i) *= 1000.* r * SkinTemperature(h, j, i)
	  / SurfacePressure(h, j, i);
  // -1. to change the direction of the flux.
  Evaporation.Mlt(-1./(Delta_t_in * 3600.));

  Decumulate(SensibleHeat, 4, 1);
  for (j=0; j<Ny_in; j++)
    for (i=0; i<Nx_in; i++)
      SensibleHeat(0, j, i) -= SensibleHeat_prev(Nt_in - 2, j, i);
  // Divides by (\rho_{air} * cp).
  for (h=0; h<Nt_in; h++)
    for (j=0; j<Ny_in; j++)
      for (i=0; i<Nx_in; i++)
	SensibleHeat(h, j, i) *= r * SkinTemperature(h, j, i)
	  / (SurfacePressure(h, j, i) * cp);
  // -1. to change the direction of the flux.
  SensibleHeat.Mlt(-1./(Delta_t_in * 3600.));

  cout << " done." << endl;


  ///////////////////////////
  // INPUT DATA PROCESSING //
  ///////////////////////////

  // Computes level heights with pressure levels.
  cout << "Computing level heights in meter..."; cout.flush();

  Data<real, 1> alpha(61), beta(61);
  
  FormatFormattedText InputCoefficients("<e><e><e><e 2>");
  InputCoefficients.Read(directory_in + "hybrid_coefficients.dat",
			 "1", alpha);
  InputCoefficients.Read(directory_in + "hybrid_coefficients.dat", "2", beta);

  alpha.ReverseData(0);
  alpha.Mlt(1. / P0);
  beta.ReverseData(0);

  ComputePressure(alpha, beta, SurfacePressure, Pressure_interf);
  
  for (h=0; h<Nt_in; h++)
    for (k=0; k<Nz_in; k++)
      for (j=0; j<Ny_in; j++)
	for (i=0; i<Nx_in; i++)
	  Pressure(h, k, j, i) = (Pressure_interf(h, k+1, j, i)
				  + Pressure_interf(h, k, j, i)) / 2.0;
  
  ComputeInterfHeight(Pressure_interf, Temperature, GridZ_interf_in);
  ComputeMiddleHeight(Pressure_interf, Temperature, GridZ_interf_in, GridZ_in);

  cout << " done." << endl;


  ///////////////////////
  // RICHARDSON NUMBER //
  ///////////////////////

  cout << "Computing Richardson number..."; cout.flush();

  // 3D Richardson number.
  ComputePotentialTemperature(Temperature, Pressure, PotentialTemperature);
  ComputeRichardson(ZonalWind, MeridionalWind,
		    PotentialTemperature, Richardson);

  // Surface Richardson number.
  ComputePotentialTemperature(SkinTemperature, SurfacePressure,
			      SurfacePotentialTemperature);
  ComputeModule(ZonalWind, MeridionalWind, SurfaceWindModule);
  if (richardson_with_roughness)
    // No more than 5m, knowing that the first ECMWF level is at ~9m.
    Roughness.ThresholdMax(5.0);
  if (richardson_with_roughness)
    ComputeRichardson(Roughness, SurfaceWindModule,
		      SurfacePotentialTemperature,
		      PotentialTemperature, SurfaceRichardson);
  else
    ComputeRichardson(SurfaceWindModule, SurfacePotentialTemperature,
		      PotentialTemperature, SurfaceRichardson);

  cout << " done." << endl;


  ///////////////////////////
  // LINEAR INTERPOLATIONS //
  ///////////////////////////

  cout << "Linear interpolations..."; cout.flush();
  LinearInterpolationOneGeneral(Pressure, Pressure_out, 1);
  LinearInterpolationOneGeneral(Temperature, Temperature_out, 1);
  LinearInterpolationOneGeneral(Humidity, Humidity_out, 1);
  LinearInterpolationOneGeneral(MeridionalWind, MeridionalWind_out, 1);
  LinearInterpolationOneGeneral(ZonalWind, ZonalWind_out, 1);
  LinearInterpolationRegular(SolarRadiation, SolarRadiation_out);
  LinearInterpolationRegular(SurfaceRichardson, SurfaceRichardson_out);
  LinearInterpolationOneGeneral(Richardson, Richardson_out, 1);
  LinearInterpolationRegular(BoundaryHeight, BoundaryHeight_out);
  BoundaryHeight_out.ThresholdMin(GridZ_interf_out(1));
  LinearInterpolationRegular(U_star, U_star_out);
  LinearInterpolationRegular(V_star, V_star_out);
  LinearInterpolationRegular(SurfacePressure, SurfacePressure_out);
  LinearInterpolationRegular(SurfaceTemperature, SurfaceTemperature_out);
  LinearInterpolationRegular(SkinTemperature, SkinTemperature_out);
  LinearInterpolationRegular(SoilWater, SoilWater_out);
  LinearInterpolationRegular(Evaporation, Evaporation_out);
  LinearInterpolationRegular(SensibleHeat, SensibleHeat_out);
  Humidity_out.Threshold(0., 1.);
  SoilWater_out.Threshold(0., 1.);
  SolarRadiation_out.ThresholdMin(0.);
  cout << " done." << endl;


  /////////////////
  // WIND MODULE //
  /////////////////

  cout << "Wind module..."; cout.flush();

  ComputeModule(ZonalWind_out, MeridionalWind_out, WindModule_out);

  U_star_out.Apply(abs_);
  V_star_out.Apply(abs_);

  U_star_out.Apply(sqrt_);
  V_star_out.Apply(sqrt_);

  ComputeModule(U_star_out, V_star_out, FrictionModule_out);

  cout << " done." << endl;


  /////////////////////////////////////////
  // PHOTOSYNTHETICALLY ACTIVE RADIATION //
  /////////////////////////////////////////

  cout << "Computing PAR..."; cout.flush();

  real zenith_angle;
  real optical_thickness, visible_beam, visible_diff,
    water_abs, beam, diff, visible_rad, IR_rad,
    ratio, ratio0;
  real fvb, fvd, PARdb, PARdiff;
  real watt2umol(4.6);

  for (h=0; h<Nt_out; h++)
    for (j=0; j<Ny_out; j++)
      for (i=0; i<Nx_out; i++)
	{

	  zenith_angle = ZenithAngle(GridX_out(i), GridY_out(j), date.GetDate(),
				     real(GridT_out(h) - int(GridT_out(h)/24)*24.))
	    /180.*pi;

	  if ( (zenith_angle>=1.51844) || (SolarRadiation_out(h, j, i)<=0.) )
	    PAR_out(h, j, i) = 0.;
	  else
	    {
	      optical_thickness = Pressure_out(h, 0, j, i)
		/ 101325. / cos(zenith_angle);
	      visible_beam = 600. * exp(-.185 * optical_thickness)
		* cos(zenith_angle);
	      visible_diff = 0.42 * (600. - visible_beam)
		* cos(zenith_angle);
	      water_abs = 1320. * .077 * pow(2. * optical_thickness, 0.3);
	      beam = (720. * exp(-0.06 * optical_thickness) - water_abs)
		* cos(zenith_angle);
	      diff = 0.65 * (720. - water_abs - beam) * cos(zenith_angle);

	      visible_rad = visible_beam + visible_diff;
	      IR_rad  = beam + diff;
	      ratio  = visible_rad / (visible_rad + IR_rad);
	      ratio0 = SolarRadiation_out(h, j, i) / (visible_rad + IR_rad);

	      if (ratio0 >= 0.89)
		fvb = visible_beam / visible_rad * 0.941124;
	      else if (ratio0 <= 0.21)
		fvb = visible_beam / visible_rad * 9.55e-3;
	      else
		fvb = visible_beam / visible_rad
		  * ( 1. - pow((0.9-ratio0)/0.7, 0.666667) );
	      fvd = 1. - fvb;

	      PARdb  = SolarRadiation_out(h, j, i) * ratio * fvb * watt2umol;
	      PARdiff = SolarRadiation_out(h, j, i) * ratio * fvd * watt2umol;

	      PARdb_out(h, j, i) = PARdb;
	      PARdiff_out(h, j, i) = PARdiff;
	      PAR_out(h, j, i) = PARdb + PARdiff;
	    }
	}
  cout << " done." << endl;


  ////////////////////////
  // WRITES OUTPUT DATA //
  ////////////////////////

  FormatBinary<float> OutputMeteo;

  cout << "Writing data..."; cout.flush();
  OutputMeteo.Append(Pressure_out, directory_out + "Pressure.bin");
  OutputMeteo.Append(SurfacePressure_out, directory_out + "SurfacePressure.bin");
  OutputMeteo.Append(Temperature_out, directory_out + "Temperature.bin");
  OutputMeteo.Append(SurfaceTemperature_out,
		     directory_out + "SurfaceTemperature.bin");
  OutputMeteo.Append(SkinTemperature_out, directory_out + "SkinTemperature.bin");
  OutputMeteo.Append(Richardson_out, directory_out + "Richardson.bin");
  OutputMeteo.Append(SurfaceRichardson_out,
		     directory_out + "SurfaceRichardson.bin");
  OutputMeteo.Append(Humidity_out, directory_out + "Humidity.bin");
  OutputMeteo.Append(SolarRadiation_out, directory_out + "SolarRadiation.bin");
  OutputMeteo.Append(PARdb_out, directory_out + "PARdb.bin");
  OutputMeteo.Append(PARdiff_out, directory_out + "PARdiff.bin");
  OutputMeteo.Append(PAR_out, directory_out + "PAR.bin");
  OutputMeteo.Append(ZonalWind_out, directory_out + "ZonalWind.bin");
  OutputMeteo.Append(MeridionalWind_out, directory_out + "MeridionalWind.bin");
  TransformZonalWind(ZonalWind_out);
  TransformMeridionalWind(MeridionalWind_out);
  OutputMeteo.Append(ZonalWind_out, directory_out + "U.bin");
  OutputMeteo.Append(MeridionalWind_out, directory_out + "V.bin");
  OutputMeteo.Append(WindModule_out, directory_out + "WindModule.bin");
  OutputMeteo.Append(FrictionModule_out, directory_out + "FrictionModule.bin");
  OutputMeteo.Append(BoundaryHeight_out, directory_out + "BoundaryHeight.bin");
  OutputMeteo.Append(SoilWater_out, directory_out + "SoilWater.bin");
  OutputMeteo.Append(Evaporation_out, directory_out + "Evaporation.bin");
  OutputMeteo.Append(SensibleHeat_out, directory_out + "SensibleHeat.bin");
  cout << " done." << endl;

  cout << endl;

  END;

  return 0;

}
